@@ -2,7 +2,7 @@
 *                 Exim Monitor                   *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This module contains code for scanning the smaill log,
@@ -200,9 +200,9 @@ if (LOG != NULL)
         dest_item *d;
         char *q, *r;
         p += 2;
-        while (isspace(*p)) p++;
+        while (isspace((uschar)*p)) p++;
         q = p;
-        while (*p != 0 && !isspace(*p))
+        while (*p != 0 && !isspace((uschar)*p))
           {
           if (*p++ != '\"') continue;
           while (*p != 0)
@@ -217,7 +217,7 @@ if (LOG != NULL)
         d = find_dest(qq, q, dest_add);
         if (d->parent == NULL)
           {
-          while (isspace(*p)) p++;
+          while (isspace((uschar)*p)) p++;
           if (*p == '<')
             {
             dest_item *dd;
