@@ -2,7 +2,7 @@
 *                 Exim Monitor                   *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -457,7 +457,7 @@ for (i = 0; i < subdir_max; i++)
     pass, add it to the list for subsequent scans, and remember that
     we are dealing with a split directory. */
 
-    if (i == 0 && len == 1 && isalnum(*name))
+    if (i == 0 && len == 1 && isalnum((uschar)*name))
       {
       subdirs[subdir_max++] = *name;
       spool_is_split = TRUE;
@@ -772,7 +772,7 @@ while (p != NULL)
       dd = dd->next;
       }
     if (dd != NULL)
-      text_showf(queue_widget, "                               ...\n");
+      text_showf(queue_widget, "                                     ...\n");
     }
 
   p = p->next;
