@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Code for receiving a message and setting up spool files. */
@@ -87,13 +87,12 @@ if (check_log_space > 0 || check_log_inodes > 0)
     path = string_copy(spool_directory);
   else
     {
+    int sep = ':';              /* Not variable - outside scripts use */
     char *cp;
     char *p = log_file_path;
     char buffer[1024];
 
-    for (path = string_nextinlist(&p, ':', buffer, sizeof(buffer));
-         path != NULL;
-         path = string_nextinlist(&p, ':', buffer, sizeof(buffer)))
+    while ((path = string_nextinlist(&p, &sep, buffer, sizeof(buffer))) != NULL)
       {
       if (strcmp(path, "syslog") != 0) break;
       }
@@ -329,12 +328,10 @@ Arguments:
   fin       a FILE from which to read the message
   fout      a FILE to which to write the message
 
-Returns:    FALSE if the message is too big; TRUE otherwise, even if there
-              is an error while writing - a general check on the output file
-              (using ferror()) is done later to pick up any error states.
+Returns:    One of the END_xxx values indicating why it stopped reading
 */
 
-static BOOL
+static int
 read_message_data(FILE *fin, FILE *fout)
 {
 int ch_state;
@@ -348,18 +345,18 @@ if (!dot_ends)
   while ((ch = (accept_getc)(fin)) != EOF)
     {
     if (drop_cr && ch == '\r') continue;
-    if (fputc(ch, fout) == EOF) return TRUE;  /* => not too big */
+    if (fputc(ch, fout) == EOF) return END_WERROR;
     if (ch == '\n') body_linecount++;
     last_ch = ch;
-    if (++message_size > thismessage_size_limit) return FALSE;
+    if (++message_size > thismessage_size_limit) return END_SIZE;
     }
   if (last_ch != '\n')
     {
-    fputc('\n', fout);
+    if (fputc('\n', fout) == EOF) return END_WERROR;
     body_linecount++;
     message_size++;
     }
-  return TRUE;  /* => not too big, not that there's been no error */
+  return END_EOF;
   }
 
 /* Handle the case when a dot on its own or EOF terminate */
@@ -386,16 +383,14 @@ while ((ch = (accept_getc)(fin)) != EOF)
     break;
 
     case 2:                         /* After "\n." */
-    if (ch == '\n') return TRUE;
-    if (message_size++, fputc('.', fout) == EOF)
-      return TRUE;  /* => not too big, not that there's been no error */
+    if (ch == '\n') return END_DOT;
+    if (message_size++, fputc('.', fout) == EOF) return END_WERROR;
     ch_state = 0;
     break;
     }
 
-  if (fputc(ch, fout) == EOF) break;
-  if (++message_size > thismessage_size_limit)
-    return FALSE;
+  if (fputc(ch, fout) == EOF) return END_WERROR;
+  if (++message_size > thismessage_size_limit) return END_SIZE;
   }
 
 /* Get here if EOF read */
@@ -403,18 +398,18 @@ while ((ch = (accept_getc)(fin)) != EOF)
 switch(ch_state)
   {
   case 2:
-  fputc('.', fout);
+  if (fputc('.', fout) == EOF) return END_WERROR;
   message_size++;
   /* Fall through */
 
   case 0:
-  fputc('\n', fout);
+  if (fputc('\n', fout) == EOF) return END_WERROR;
   body_linecount++;
   message_size++;
   break;
   }
 
-return TRUE;  /* => not too big, not that there's been no error */
+return END_EOF;
 }
 
 
@@ -426,8 +421,7 @@ return TRUE;  /* => not too big, not that there's been no error */
 
 /* This function is called to read the remainder of an SMTP message (after the
 headers), or to skip over it when an error has occurred. In this case, the
-output file is passed as NULL. The function is global, because it is called
-from smtp_in.c for batch SMTP input, to skip a message after an error.
+output file is passed as NULL.
 
 If any line begins with a dot, that character is skipped. The input should only
 be successfully terminated by CR LF . CR LF unless it is local (non-network)
@@ -441,16 +435,12 @@ uncommenting the #definition of STRICT_CRLF at the head of this file.
 Arguments:
   fin       a FILE from which to read the message
   fout      a FILE to which to write the message; NULL if skipping
-  was_EOF   if non_NULL, set TRUE if EOF was encountered
-              (implying truncated message); should be FALSE on entry
 
-Returns:    FALSE if the message is too big; TRUE otherwise, even if there
-              is an error while writing - a general check on the output file
-              (using ferror()) is done later to pick up any error states.
+Returns:    One of the END_xxx values indicating why it stopped reading
 */
 
-BOOL
-accept_read_message_data_smtp(FILE *fin, FILE *fout, BOOL *was_EOF)
+static int
+read_message_data_smtp(FILE *fin, FILE *fout)
 {
 int ch_state = 0;
 register int ch;
@@ -495,8 +485,7 @@ while ((ch = (accept_getc)(fin)) != EOF)
     else
       {
       message_size++;
-      if (fout != NULL && fputc('\r', fout) == EOF)
-        goto END_SMTP_DATA;             /* A few lines below */
+      if (fout != NULL && fputc('\r', fout) == EOF) return END_WERROR;
       if (ch != '\r') ch_state = 1; else continue;
       }
     break;
@@ -507,7 +496,7 @@ while ((ch = (accept_getc)(fin)) != EOF)
           && sender_local
           #endif
           )
-      goto END_SMTP_DATA;               /* A few lines below */
+      return END_DOT;
     if (ch == '\r')
       {
       ch_state = 4;
@@ -517,9 +506,10 @@ while ((ch = (accept_getc)(fin)) != EOF)
     break;
 
     case 4:                             /* After [CR] LF . CR */
-    if (ch == '\n' || (message_size++, (fout != NULL &&
-      fputc('\r', fout) == EOF)))
-        goto END_SMTP_DATA;             /* A few lines below */
+    if (ch == '\n') return END_DOT;
+    message_size++;
+    if (fout != NULL && fputc('\r', fout) == EOF) return END_WERROR;
+
     if (ch == '\r')
       {
       ch_state = 2;
@@ -535,26 +525,42 @@ while ((ch = (accept_getc)(fin)) != EOF)
   message_size++;
   if (fout != NULL)
     {
-    if (fputc(ch, fout) == EOF) goto END_SMTP_DATA;
-    if (message_size > thismessage_size_limit) return FALSE;
+    if (fputc(ch, fout) == EOF) return END_WERROR;
+    if (message_size > thismessage_size_limit) return END_SIZE;
     }
   }
 
 /* Fall through here if EOF encountered. This indicates some kind of error,
 since a correct message is terminated by [CR] LF . [CR] LF. */
 
-if (was_EOF != NULL) *was_EOF = TRUE;
-
-/* Goto is used to get here out of the middle of a switch inside the
-above loop. */
-
-END_SMTP_DATA:
-smtp_reading_data = FALSE;
-
-return TRUE;     /* => not too big, not that there's been no error */
+return END_EOF;
 }
 
 
+
+
+/*************************************************
+*             Swallow SMTP message               *
+*************************************************/
+
+/* This function is called when there has been some kind of error while reading
+an SMTP message, and the remaining data may need to be swallowed. It is global
+because it is called from smtp_closedown() to shut down an incoming call
+tidily.
+
+Argument:    a FILE from which to read the message
+Returns:     nothing
+*/
+
+void
+accept_swallow_smtp(FILE *fin)
+{
+if (message_ended >= END_NOTENDED)
+  message_ended = read_message_data_smtp(fin, NULL);
+}
+
+
+
 /*************************************************
 *                 Accept message                 *
 *************************************************/
@@ -597,7 +603,8 @@ The general actions of this function are:
   . If there is no from: header, generate one.
 
   . Otherwise, if the sender is local, check that from: is correct, and
-    if not, generate a Sender header unless message comes from root.
+    if not, generate a Sender header unless message comes from root, or this
+    feature is disabled.
 
   . Ensure the presence of "to" or "cc" or "bcc".
 
@@ -647,7 +654,6 @@ int  error_rc = (error_handling == ERRORS_SENDER)?
 int  header_size = 256;
 register int ptr = 0;
 
-BOOL dot_ended = FALSE;
 BOOL extracted_bcc = FALSE;
 BOOL extracted_ignored = FALSE;
 BOOL had_zero = FALSE;
@@ -896,7 +902,7 @@ for (;;)
       if (smtp_input)
         {
         smtp_reply = "552 Message header is ridiculously long";
-        (void)accept_read_message_data_smtp(fin, NULL, NULL);  /* Swallow */
+        accept_swallow_smtp(fin);
         goto TIDYUP;                             /* Skip to end of function */
         }
 
@@ -943,11 +949,7 @@ for (;;)
     {
     store_reset(next);
     next = NULL;
-    if (ptr == 2)
-      {
-      dot_ended = TRUE;  /* Flag so no more gets read */
-      smtp_reading_data = FALSE;
-      }
+    if (ptr == 2) message_ended = END_DOT;   /* So no more gets read */
     break;
     }
 
@@ -1059,7 +1061,7 @@ for (;;)
     {
     char *p = next->text;
     while (mac_isgraph(*p) && *p != ':') p++;
-    while (isspace(*p)) p++;
+    while (isspace((uschar)*p)) p++;
 
     /* If not a valid header line, break from the header reading loop, leaving
     next != NULL, indicating that it holds the first line of the body. */
@@ -1211,8 +1213,8 @@ for (h = header_list->next; h != NULL; h = h->next)
       {
       char *start = h->text + 12;
       char *end = start + (int)strlen(start);
-      while (isspace(*start)) start++;
-      while (end > start && isspace(end[-1])) end--;
+      while (isspace((uschar)*start)) start++;
+      while (end > start && isspace((uschar)end[-1])) end--;
       if (*start == '<' && end[-1] == '>')
         {
         start++;
@@ -1238,7 +1240,7 @@ for (h = header_list->next; h != NULL; h = h->next)
       if (!smtp_input)
         {
         char *s = h->text + header_names[hn_from].len + 1;
-        while (isspace(*s)) s++;
+        while (isspace((uschar)*s)) s++;
         if (strncmpic(s, user_login, h->slen - (s - h->text) - 1) == 0)
           {
           header_add(htype_other, "From: %s <%s@%s>\n", user_name, user_login,
@@ -1312,7 +1314,7 @@ if (extract_recip)
     if (h->type == htype_to || h->type == htype_cc || h->type == htype_bcc)
       {
       char *s = strchr(h->text, ':') + 1;
-      while (isspace(*s)) s++;
+      while (isspace((uschar)*s)) s++;
 
       while (*s != 0)
         {
@@ -1353,7 +1355,7 @@ if (extract_recip)
           {
           int len = (int)strlen(s);
           error_block *b = store_get(sizeof(error_block));
-          while (len > 0 && isspace(s[len-1])) len--;
+          while (len > 0 && isspace((uschar)s[len-1])) len--;
           b->next = NULL;
           b->text1 = string_printing(string_copyn(s, len));
           b->text2 = errmess;
@@ -1379,7 +1381,7 @@ if (extract_recip)
 
         *ss = terminator;
         s = ss + (terminator? 1:0);
-        while (isspace(*s)) s++;
+        while (isspace((uschar)*s)) s++;
         }
 
       /* If this was the bcc: header, mark it "old", which means it
@@ -1634,15 +1636,15 @@ if (from_header == NULL)
   }
 
 /* Otherwise, if the sender is local, check that From: is correct, and if not,
-generate a Sender: header. Any previously-existing Sender: header was removed
-above. Note that sender_local, as well as being TRUE if the caller of exim is
-not trusted, is also true if a trusted caller did not supply a -f argument for
-non-smtp input. To allow trusted callers to forge From: without supplying -f,
-we have to test explicitly here. If the From: header contains more than one
-address, then the call to parse_extract_address fails, and a Sender: header is
-inserted, as required. */
+generate a Sender: header, unless disabled. Any previously-existing Sender:
+header was removed above. Note that sender_local, as well as being TRUE if the
+caller of exim is not trusted, is also true if a trusted caller did not supply
+a -f argument for non-smtp input. To allow trusted callers to forge From:
+without supplying -f, we have to test explicitly here. If the From: header
+contains more than one address, then the call to parse_extract_address fails,
+and a Sender: header is inserted, as required. */
 
-else if (sender_local && !trusted_caller)
+else if (sender_local && !trusted_caller && local_from_check)
   {
   BOOL make_sender = TRUE;
   int start, end, domain;
@@ -1651,10 +1653,28 @@ else if (sender_local && !trusted_caller)
     parse_extract_address(strchr(from_header->text, ':') + 1, &errmess,
       &start, &end, &domain, FALSE);
 
-  if (from_address != NULL &&
-    (strcmpic(sender_address, from_address) == 0 ||
-      (domain == 0 && strcmpic(from_address, user_login) == 0)))
+  /* Remove permitted prefixes and suffixes from the local part
+  of the From: address before doing the comparison with the sender. */
+
+  if (from_address != NULL)
+    {
+    int slen;
+    char *at = (domain == 0)? NULL : from_address + domain - 1;
+
+    if (at != NULL) *at = 0;
+    from_address += direct_check_prefix(from_address, local_from_prefix);
+    slen = direct_check_suffix(from_address, local_from_suffix);
+    if (slen > 0)
+      {
+      memmove(from_address+slen, from_address, (int)strlen(from_address)-slen);
+      from_address += slen;
+      }
+    if (at != NULL) *at = '@';
+
+    if (strcmpic(sender_address, from_address) == 0 ||
+      (domain == 0 && strcmpic(from_address, user_login) == 0))
         make_sender = FALSE;
+    }
 
   /* We have to cause the Sender header to be rewritten if there are
   appropriate rewriting rules, because even though sender_address has
@@ -1755,13 +1775,13 @@ DEBUG(9)
   }
 
 /* The headers are now complete in store. If we are running in filter
-testing mode, that is all this function does. Return the value of
-dot_ended. */
+testing mode, that is all this function does. Return TRUE if the message
+ended with a dot. */
 
 if (filter_test != NULL)
   {
   process_info[process_info_len] = 0;
-  return dot_ended;
+  return message_ended == END_DOT;
   }
 
 /* If this is smtp_input, the sender is not local, and sender verification is
@@ -1775,8 +1795,7 @@ if (smtp_input && !sender_local && (sender_verify || sender_try_verify))
 
   if (!verify_sender(&errcode, &errmess))
     {
-    if (!dot_ended)
-      (void)accept_read_message_data_smtp(fin, NULL, NULL);  /* Swallow */
+    accept_swallow_smtp(fin);
     smtp_reply = string_sprintf("%d rejected: %s", errcode, errmess);
 
     /* Send the configurable message, if any */
@@ -1817,7 +1836,7 @@ if (headers_check_syntax)
         ))
       {
       char *s = strchr(h->text, ':') + 1;
-      while (isspace(*s)) s++;
+      while (isspace((uschar)*s)) s++;
 
       parse_allow_group = TRUE;     /* Allow group syntax */
 
@@ -1852,7 +1871,7 @@ if (headers_check_syntax)
           error message. */
 
           t = ss;
-          while (t > s && isspace(t[-1])) t--;
+          while (t > s && isspace((uschar)t[-1])) t--;
 
           /* Add the address which failed to the error message, since in a
           header with very many addresses it is sometimes hard to spot
@@ -1876,8 +1895,7 @@ if (headers_check_syntax)
             {
             if (smtp_input)
               {
-              if (!dot_ended)    /* Swallow */
-                (void)accept_read_message_data_smtp(fin, NULL, NULL);
+              accept_swallow_smtp(fin);
               smtp_reply = string_sprintf("550 Syntax error in '%s' header: %s",
                 hname, errmess);
 
@@ -1911,7 +1929,7 @@ if (headers_check_syntax)
         /* Advance to the next address */
 
         s = ss + (terminator? 1:0);
-        while (isspace(*s)) s++;
+        while (isspace((uschar)*s)) s++;
         }
       }
     }
@@ -1924,13 +1942,15 @@ that writes a "received" log line and ends off, with msg_size set to the size
 to be "logged". For real messages, this value is obtained accurately from the
 written headers and body. Here we fake it with the count of bytes received plus
 the Received: header. The difference is that any other added headers (Date,
-From, etc.) don't get counted here. */
+From, etc.) don't get counted here. If the message wasn't terminated by a '.'
+line, it must have been EOF, which must give the error message. */
 
 if (host_checking)
   {
-  if (!dot_ended) (void)accept_read_message_data_smtp(fin, NULL, NULL);
+  accept_swallow_smtp(fin);
   msg_size = message_size + header_list->slen;
-  goto HOST_CHECKING_WRITE_LOG;
+  if (message_ended == END_DOT) goto HOST_CHECKING_WRITE_LOG;
+  goto HOST_CHECKING_EOF;
   }
 
 
@@ -2000,25 +2020,28 @@ if (next != NULL)
 (indicated by '.'), or might have encountered an error while writing the
 message id or "next" line. */
 
-if (!ferror(data_file) && !(accept_feof)(fin) && !dot_ended)
+if (!ferror(data_file) && !(accept_feof)(fin) && message_ended != END_DOT)
   {
-  BOOL smtp_EOF = FALSE;
-  BOOL size_ok;
-
   if (smtp_input)
     {
-    size_ok = accept_read_message_data_smtp(fin, data_file, &smtp_EOF);
+    message_ended = read_message_data_smtp(fin, data_file);
     accept_linecount++;                /* The terminating "." line */
     }
-  else size_ok = read_message_data(fin, data_file);
+  else message_ended = read_message_data(fin, data_file);
 
   accept_linecount += body_linecount;  /* For BSMTP errors mainly */
 
-  /* Handle premature termination of SMTP (can't happen for non-SMTP input) */
+  /* Handle premature termination of SMTP */
 
-  if (smtp_EOF)
+  if (smtp_input && message_ended == END_EOF)
     {
     unlink(spool_name);                /* Lose data file when closed */
+
+    /* When checking a host with -bh, control jumps here if the incoming
+    message was not terminated by EOF, so as to get the error message. */
+
+    HOST_CHECKING_EOF:
+
     message_id[0] = 0;                 /* Indicate no message accepted */
 
     if (sender_host_unknown)
@@ -2040,12 +2063,12 @@ if (!ferror(data_file) && !(accept_feof)(fin) && !dot_ended)
 
   /* Handle message that is too big */
 
-  if (!size_ok)
+  if (message_ended == END_SIZE)
     {
     unlink(spool_name);                /* Lose the data file when closed */
     if (smtp_input)
       {
-      (void)accept_read_message_data_smtp(fin, NULL, NULL);  /* Swallow */
+      accept_swallow_smtp(fin);
       smtp_reply = "552 Message too large";
       log_write(2, LOG_MAIN|LOG_REJECT, "rejected%s%s%s%s%s <%s>: "
         "message too large: %d recipient%s size=%d max=%d",
@@ -2118,11 +2141,11 @@ if (fflush(data_file) == EOF || ferror(data_file) ||
   if (smtp_input)
     {
     if (input_error)
-      smtp_reply = "451 Error in reading input data";
+      smtp_reply = "451 Error while reading input data";
     else
       {
-      smtp_reply = "451 Error in writing spool file";
-      (void)accept_read_message_data_smtp(fin, NULL, NULL);  /* Swallow */
+      smtp_reply = "451 Error while writing spool file";
+      accept_swallow_smtp(fin);
       }
     message_id[0] = 0;               /* Indicate no message accepted */
     goto TIDYUP;                     /* Skip to end of function */
@@ -2332,7 +2355,7 @@ else
     char *s = subject_header->text + 7;
 
     while (*s != ':') s++;
-    while (isspace(*(++s)));
+    while (isspace((uschar)*(++s)));
     ss = string_printing(s);
 
     strcpy(big_buffer, " T=\"");
