@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with running Exim as a daemon */
@@ -328,6 +328,7 @@ if (pid == 0)
     if ((rc = smtp_setup_msg()) > 0)
       {
       (void) accept_msg(smtp_in, FALSE);
+      search_tidyup();                    /* Close cached databases */
       if (message_id[0] == 0) continue;
       }
     else
@@ -413,45 +414,49 @@ if (pid == 0)
 
     /* If a delivery attempt is required, spin off a new process to handle it.
     If we are not root, we have to re-exec exim unless root can be regained by
-    the use of seteuid. */
+    the use of seteuid, or the security level is 4, which means everything
+    happens unprivileged. */
 
-    else if ((pid = fork()) == 0)
+    else
       {
-      fclose(smtp_in);
-      fclose(smtp_out);
-      smtp_in = NULL;
-      if (security_level >= 2)
+      if ((pid = fork()) == 0)
         {
-        int i = 0;
-        char *argv[11 + MAX_CLMACROS];
-        argv[i++] = exim_path;
-        if (clmacro_count > 0)
+        fclose(smtp_in);
+        fclose(smtp_out);
+        smtp_in = NULL;
+        if (security_level == 3 || security_level == 2)
           {
-          memcpy(argv + i, clmacros, clmacro_count * sizeof(char *));
-          i += clmacro_count;
+          int i = 0;
+          char *argv[11 + MAX_CLMACROS];
+          argv[i++] = exim_path;
+          if (clmacro_count > 0)
+            {
+            memcpy(argv + i, clmacros, clmacro_count * sizeof(char *));
+            i += clmacro_count;
+            }
+          if (debug_level > 0)
+            argv[i++] = string_sprintf("-d%d", debug_level);
+          if (debug_trace_memory != 0) argv[i++] = "-dm";
+          if (dont_deliver) argv[i++] = "-N";
+          if (queue_smtp) argv[i++] = "-odqs";
+          if (queue_remote) argv[i++] = "-odqr";
+          if (config_changed)
+            {
+            argv[i++] = "-C";
+            argv[i++] = config_filename;
+            }
+          argv[i++] = "-Mc";
+          argv[i++] = message_id;
+          argv[i++] = (char *)0;
+          execv(argv[0], argv);
+          log_write(0, LOG_PANIC_DIE, "exec of exim -Mc failed: %s",
+            strerror(errno));
           }
-        if (debug_level > 0)
-          argv[i++] = string_sprintf("-d%d", debug_level);
-        if (debug_trace_memory != 0) argv[i++] = "-dm";
-        if (dont_deliver) argv[i++] = "-N";
-        if (queue_smtp) argv[i++] = "-odqs";
-        if (queue_remote) argv[i++] = "-odqr";
-        if (config_changed)
-          {
-          argv[i++] = "-C";
-          argv[i++] = config_filename;
-          }
-        argv[i++] = "-Mc";
-        argv[i++] = message_id;
-        argv[i++] = (char *)0;
-        execv(argv[0], argv);
-        log_write(0, LOG_PANIC_DIE, "exec of exim -Mc failed: %s",
-          strerror(errno));
-        }
 
-      /* No need to re-exec */
-      (void)deliver_message(message_id, FALSE, FALSE);
-      _exit(EXIT_SUCCESS);
+        /* No need to re-exec */
+        (void)deliver_message(message_id, FALSE, FALSE);
+        _exit(EXIT_SUCCESS);
+        }
       }
 
     if (pid > 0) DEBUG(1) debug_printf("forked delivery process %d\n", pid);
@@ -585,26 +590,21 @@ if (!daemon_listen)
   sprintf(buff+(int)strlen(buff), "-q%s", readconf_printtime(queue_interval));
 else if (queue_interval <= 0) strcat(buff, "-bd");
 
-/* Close all open file descriptors and disconnect from the controlling
-terminal, if we are not debugging. Most modern Unixes seem to have setsid() for
-getting rid of the controlling terminal. For any OS that doesn't, setsid() can
-be #defined as a no-op, or as something else. */
+/* Close all open file descriptors that we know about, and disconnect from the
+controlling terminal, if we are not debugging. Most modern Unixes seem to have
+setsid() for getting rid of the controlling terminal. For any OS that doesn't,
+setsid() can be #defined as a no-op, or as something else. */
 
 if (debug_level <= 0)
   {
   pid_t pid;
-  int fd;
 
   log_close_all();  /* Just in case anything was logged earlier */
-
-  /* Ensure cached files for searches and expands, which might have been
-  opened while reading the configuration, are closed before doing a general
-  close on all file descriptors. Set log_stderr NULL so log doesn't try to
-  use it to copy paniclog output. */
-
-  search_tidyup();
-  for (fd = mac_maxfd; fd >= 0; fd--) close(fd);
-  log_stderr = NULL;
+  search_tidyup();  /* Just in case any were used in reading the config. */
+  close(0);         /* Get rid of stdin/stdout/stderr */
+  close(1);
+  close(2);
+  log_stderr = NULL;  /* So no attempt to copy paniclog output */
 
   /* Fork, in case current process is a process group leader (see 'man
   setsid' for an explanation). */
@@ -639,7 +639,7 @@ if (daemon_listen)
   union sockaddr_46 sin;
 
   #if HAVE_IPV6
-  struct in6_addr anyaddr6 = { IN6ADDR_ANY_INIT };
+  struct in6_addr anyaddr6 = IN6ADDR_ANY_INIT;
   #endif
 
   /* Find the standard SMTP port if no port number given; otherwise
@@ -700,16 +700,14 @@ if (daemon_listen)
     int i;
 
     #if HAVE_IPV6
-    BOOL ipv6_socket, ipv6_address;
-    #endif
+    BOOL ipv6_socket = TRUE;
+    BOOL ipv6_address = strchr(ipa->address, ':') != NULL;
 
     /* A system which has all the IPv6 libraries etc. may nevertheless not
     support IPv6 in the kernel, so if the socket creation fails, fall back
     on creating an IPv4 socket, except when we have an explicit IPv6 address,
     for which we really do need an IPv6 socket. */
 
-    #if HAVE_IPV6
-    ipv6_address = strchr(ipa->address, ':') != NULL;
     listen_sockets[sk] = socket(AF_INET6, SOCK_STREAM, 0);
     if (listen_sockets[sk] < 0)
       {
@@ -719,7 +717,6 @@ if (daemon_listen)
         ipv6_socket = FALSE;
         }
       }
-    else ipv6_socket = TRUE;
 
     /* The IPv4 case */
 
@@ -1271,14 +1268,14 @@ for (;;)
 
   /* Handle being woken by SIGHUP. We know at this point that the result
   of accept() has been dealt with, so we can re-exec exim safely, first
-  closing the listening sockets so that they can be reused. */
+  closing the listening sockets so that they can be reused. All log files
+  get closed by the close-on-exec flag. */
 
   if (sighup_seen)
     {
     int sk;
     log_write(0, LOG_MAIN, "pid %d: SIGHUP received: re-exec daemon",
       getpid());
-    log_close_all();
     for (sk = 0; sk < listen_socket_count; sk++) close(listen_sockets[sk]);
     sighup_argv[0] = exim_path;
     execv(exim_path, sighup_argv);
