@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -522,8 +522,7 @@ if (dblock->senders != NULL)
     {
     if (rc == DEFER)
       {
-      addr->message = string_sprintf("sender check defer in %s director",
-        dblock->name);
+      addr->message = "sender check lookup defer";
       return DEFER;
       }
     DEBUG(9) debug_printf("%s director: sender match failure\n",
@@ -535,6 +534,12 @@ if (dblock->senders != NULL)
 if (dblock->condition != NULL &&
     !expand_check_condition(dblock->condition, dblock->name, "director"))
   {
+  if (search_find_defer)
+    {
+    addr->message = "condition check lookup defer";
+    DEBUG(9) debug_printf("%s\n", addr->message);
+    return DEFER;
+    }
   DEBUG(9) debug_printf("%s director skipped: condition failure\n",
     dblock->name);
   return FAIL_CONDITION;
@@ -585,7 +590,7 @@ if (strcmp(lastname, s) != 0)
   {
   int i = 0;
 
-  if (return_uid != NULL && (isdigit(*s) || *s == '-') &&
+  if (return_uid != NULL && (isdigit((uschar)*s) || *s == '-') &&
        s[strspn(s+1, "0123456789")+1] == 0)
     {
     *return_uid = (uid_t)atoi(s);
@@ -659,7 +664,7 @@ direct_findgroup(char *s, gid_t *return_gid)
 int i = 0;
 struct group *gr;
 
-if ((isdigit(*s) || *s == '-') && s[strspn(s+1, "0123456789")+1] == 0)
+if ((isdigit((uschar)*s) || *s == '-') && s[strspn(s+1, "0123456789")+1] == 0)
   {
   *return_gid = (gid_t)atoi(s);
   return TRUE;
@@ -827,7 +832,7 @@ for (d = directors; d != NULL; d = d->next)
 
   /* Build a host list if fallback hosts is set */
 
-  host_build_hostlist(&(d->fallback_hostlist), d->fallback_hosts);
+  host_build_hostlist(&(d->fallback_hostlist), d->fallback_hosts, FALSE);
   }
 }
 
@@ -855,6 +860,96 @@ for (d = directors; d != NULL; d = d->next)
 
 
 
+/*************************************************
+*         Check local part for prefix            *
+*************************************************/
+
+/* This functionis handed a local part and a list of possible prefixes;
+if any one matches, return the prefix length. A prefix beginning with '*'
+is a wildcard.
+
+Arguments:
+  local_part    the local part to check
+  prefixes      the list of prefixes
+
+Returns:        length of matching prefix or zero
+*/
+
+int
+direct_check_prefix(char *local_part, char *prefixes)
+{
+int sep = 0;
+char *prefix;
+char *listptr = prefixes;
+char prebuf[64];
+
+while ((prefix = string_nextinlist(&listptr, &sep, prebuf, sizeof(prebuf)))
+       != NULL)
+  {
+  int plen = (int)strlen(prefix);
+  if (prefix[0] == '*')
+    {
+    char *p;
+    prefix++;
+    for (p = local_part + (int)strlen(local_part) - (--plen);
+         p >= local_part; p--)
+      if (strncmpic(prefix, p, plen) == 0) return plen + p - local_part;
+    }
+  else
+    if (strncmpic(prefix, local_part, plen) == 0) return plen;
+  }
+
+return 0;
+}
+
+
+
+/*************************************************
+*         Check local part for prefix            *
+*************************************************/
+
+/* This functionis handed a local part and a list of possible suffixes;
+if any one matches, return the suffix length. A suffix ending with '*'
+is a wildcard.
+
+Arguments:
+  local_part    the local part to check
+  suffixes      the list of suffixes
+
+Returns:        length of matching suffix or zero
+*/
+
+int
+direct_check_suffix(char *local_part, char *suffixes)
+{
+int sep = 0;
+int alen = (int)strlen(local_part);
+char *suffix;
+char *listptr = suffixes;
+char sufbuf[64];
+
+while ((suffix = string_nextinlist(&listptr, &sep, sufbuf, sizeof(sufbuf)))
+     != NULL)
+  {
+  int slen = (int)strlen(suffix);
+  if (suffix[slen-1] == '*')
+    {
+    char *p, *pend;
+    pend = local_part + alen - (--slen) + 1;
+    for (p = local_part; p < pend; p++)
+      if (strncmpic(suffix, p, slen) == 0) return alen - (p - local_part);
+    }
+  else
+    if (alen > slen && strncmpic(suffix, local_part + alen - slen, slen) == 0)
+      return slen;
+  }
+
+return 0;
+}
+
+
+
+
 /*************************************************
 *                 Direct one address             *
 *************************************************/
@@ -941,7 +1036,13 @@ for (d = (addr->start_director == NULL)? directors : addr->start_director;
   deliver_set_expansions(addr);
 
   if (route_skip_driver((router_instance *)d, addr->local_part, addr->domain,
-    verify, &more, "director")) continue;
+       verify, &more, "director"))
+    {
+    if (!search_find_defer) continue;
+    addr->message = "lookup defer for domains or local_parts";
+    addr->director = d;
+    return DEFER;
+    }
 
   /* Skip this director if processing EXPN and it doesn't have expn set */
 
@@ -957,46 +1058,15 @@ for (d = (addr->start_director == NULL)? directors : addr->start_director;
 
   if (d->prefix != NULL)
     {
-    BOOL matched = FALSE;
-    char *prefix;
-    char *listptr = d->prefix;
-    char prebuf[64];
-
-    /* Loop for possible list of prefixes. If any one begins with '*',
-    special wildcard checking is done. */
-
-    while ((prefix = string_nextinlist(&listptr, ':', prebuf, sizeof(prebuf)))
-           != NULL)
+    int plen = direct_check_prefix(addr->local_part, d->prefix);
+    if (plen > 0)
       {
-      int plen = (int)strlen(prefix);
-      if (prefix[0] == '*')
-        {
-        char *p;
-        prefix++;
-        for (p = addr->local_part + (int)strlen(addr->local_part) - (--plen);
-             p >= addr->local_part; p--)
-          {
-          if (strncmpic(prefix, p, plen) == 0)
-            {
-            matched = TRUE;
-            plen += p - addr->local_part;
-            break;
-            }
-          }
-        }
-      else matched = (strncmpic(prefix, addr->local_part, plen) == 0);
-
-      if (matched)
-        {
-        oldlocal_part = addr->local_part;
-        addr->local_part += plen;
-        addr->prefix = string_copyn(oldlocal_part, plen);
-        DEBUG(9) debug_printf("stripped prefix %s\n", addr->prefix);
-        break;
-        }
+      oldlocal_part = addr->local_part;
+      addr->local_part += plen;
+      addr->prefix = string_copyn(oldlocal_part, plen);
+      DEBUG(9) debug_printf("stripped prefix %s\n", addr->prefix);
       }
-
-    if (!matched && !d->prefix_optional)
+    else if (!d->prefix_optional)
       {
       DEBUG(9) debug_printf("%s director skipped: prefix mismatch\n",
         d->name);
@@ -1010,50 +1080,20 @@ for (d = (addr->start_director == NULL)? directors : addr->start_director;
 
   if (d->suffix != NULL)
     {
-    BOOL matched = FALSE;
-    char *suffix;
-    char *listptr = d->suffix;
-    char sufbuf[64];
-
-    while ((suffix = string_nextinlist(&listptr, ':', sufbuf, sizeof(sufbuf)))
-         != NULL)
+    int slen = direct_check_suffix(addr->local_part, d->suffix);
+    if (slen > 0)
       {
-      int slen = (int)strlen(suffix);
-      int alen = (int)strlen(addr->local_part);
-
-      if (suffix[slen-1] == '*')
-        {
-        char *p;
-        char *pend = addr->local_part + alen - (--slen) + 1;
-        for (p = addr->local_part; p < pend; p++)
-          {
-          if (strncmpic(suffix, p, slen) == 0)
-            {
-            matched = TRUE;
-            slen = alen - (p - addr->local_part);
-            break;
-            }
-          }
-        }
-
-      else matched = alen > slen &&
-        strncmpic(suffix, addr->local_part + alen - slen, slen) == 0;
-
-      if (matched)
-        {
-        suffixend = alen - slen;
-        suffixchar = addr->local_part[suffixend];
-        addr->suffix = string_copy(addr->local_part+suffixend);
-        addr->local_part[suffixend] = 0;
-        DEBUG(9) debug_printf("stripped suffix %s\n", addr->suffix);
-        break;
-        }
+      suffixend = (int)strlen(addr->local_part) - slen;
+      suffixchar = addr->local_part[suffixend];
+      addr->suffix = string_copy(addr->local_part+suffixend);
+      addr->local_part[suffixend] = 0;
+      DEBUG(9) debug_printf("stripped suffix %s\n", addr->suffix);
       }
 
     /* If failed to match the suffix, restore any change the prefix
     handler made. */
 
-    if (!matched && !d->suffix_optional)
+    else if (!d->suffix_optional)
       {
       DEBUG(9) debug_printf("%s director skipped: suffix mismatch\n",
         d->name);
