@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -39,8 +39,12 @@ optionlist aliasfile_director_options[] = {
       (void *)(offsetof(aliasfile_director_options_block, file_transport)) },
   { "forbid_file",        opt_bool,
       (void *)(offsetof(aliasfile_director_options_block, forbid_file)) },
+  { "forbid_include",     opt_bool,
+      (void *)(offsetof(aliasfile_director_options_block, forbid_include)) },
   { "forbid_pipe",        opt_bool,
       (void *)(offsetof(aliasfile_director_options_block, forbid_pipe)) },
+  { "forbid_special",        opt_bool,
+      (void *)(offsetof(aliasfile_director_options_block, forbid_special)) },
   { "freeze_missing_include", opt_bool,
       (void *)(offsetof(aliasfile_director_options_block, freeze_missing_include)) },
   { "include_domain",     opt_bool,
@@ -112,6 +116,8 @@ aliasfile_director_options_block aliasfile_director_option_defaults = {
   -1,       /* search_type */
   NULL,     /* owners */
   NULL,     /* owngroups */
+  FALSE,    /* forbid_include */
+  FALSE,    /* forbid_special */
   FALSE,    /* include_domain */
   FALSE,    /* expand */
   FALSE,    /* optional */
@@ -345,7 +351,8 @@ a list of queries to try. */
 if (ob->queries != NULL)
   {
   char *onequery;
-  while ((onequery = string_nextinlist(&query, ':', big_buffer,
+  int sep = 0;
+  while ((onequery = string_nextinlist(&query, &sep, big_buffer,
           BIG_BUFFER_SIZE)) != NULL)
     {
     aliastext = search_find(handle, filename, onequery, ob->partial_match,
@@ -357,12 +364,12 @@ else
   aliastext = search_find(handle, filename, query, ob->partial_match, NULL,
     &error);
 
-/* The lookup did not succeed. Defer if it deferred; otherwise this director
-fails to handle the address. */
+/* The lookup did not succeed. Defer if it deferred unless optional is set;
+otherwise this director fails to handle the address. */
 
 if (aliastext == NULL)
   {
-  if (search_find_defer)
+  if (search_find_defer && !ob->optional)
     {
     addr->message = string_sprintf("search_find deferred: %s", error);
     return DEFER;
@@ -423,32 +430,8 @@ if (transport != NULL)
   addr->current_dir = dblock->current_directory;
   addr->extra_headers = extra_headers;
   addr->remove_headers = remove_headers;
-
-  direct_set_ugid(addr, &ugid);
-
-  if (transport->info->local)
-    {
-    addr->next = *addr_local;
-    *addr_local = addr;
-    }
-
-  else
-    {
-    addr->next = *addr_remote;
-    *addr_remote = addr;
-    addr->fallback_hosts = dblock->fallback_hostlist;
-    }
-
-  DEBUG(2)
-    {
-    debug_printf("  queued for %s transport ", transport->name);
-    if (addr->uid_set) debug_printf("uid=%ld ", (long int)(addr->uid));
-      else debug_printf("uid=unset ");
-    if (addr->gid_set) debug_printf("gid=%ld\n", (long int)(addr->gid));
-      else debug_printf("gid=unset\n");
-    }
-
-  return OK;
+  return route_queue(addr, addr_local, addr_remote, (router_instance *)dblock,
+    &ugid, "director")? OK : ERROR;
   }
 
 /* If qualify_preserve_domain is set, we temporarily reset the value of
@@ -470,10 +453,10 @@ reset_point = store_get(0);
 
 extracted = parse_extract_addresses(aliastext, addr->domain, &generated,
   &error,
-  FALSE,           /* No addresses is an error */
-  TRUE,            /* Allow :blackhole: etc. */
-  ob->rewrite,     /* Rewrite if required */
-  NULL,            /* No restriction on :include: files */
+  FALSE,                   /* No addresses is an error */
+  !ob->forbid_special,     /* Controls :blackhole: etc. */
+  ob->rewrite,             /* Rewrite if required */
+  ob->forbid_include? "*" : NULL,  /* Restriction on :include: files */
   ob->skip_syntax_errors? &eblock : NULL);
 
 /* Restore the qualify domain, and reset to the main pool for the generation of
