@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -57,11 +57,13 @@ optionlist forwardfile_director_options[] = {
   { "forbid_filter_perl",        opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, forbid_filter_perl)) },
   #endif
+  { "forbid_filter_reply",opt_bool,
+      (void *)(offsetof(forwardfile_director_options_block, forbid_reply)) },
   { "forbid_include",     opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, forbid_include)) },
   { "forbid_pipe",        opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, forbid_pipe)) },
-  { "forbid_reply",        opt_bool,
+  { "forbid_reply",        opt_bool|opt_hidden,
       (void *)(offsetof(forwardfile_director_options_block, forbid_reply)) },
   { "freeze_missing_include", opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, freeze_missing_include)) },
@@ -216,14 +218,16 @@ if (dblock->transport != NULL || dblock->expand_transport != NULL)
     "a transport is not allowed for this director", dblock->name);
 
 /* Onetime aliases can only be real addresses. If filter is not set,
-force forbid_reply, since a non-filter file can't generate one anyway. */
+force forbid_reply, since a non-filter file can't generate replies. */
 
 if (!ob->filter) ob->forbid_reply = TRUE;
 
-if (ob->one_time && (!ob->forbid_pipe || !ob->forbid_file || !ob->forbid_reply))
+if (ob->one_time && (!ob->forbid_pipe || !ob->forbid_file ||
+      !ob->forbid_reply))
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
-    "forbid_pipe, forbid_file, and forbid_reply must be set when one_time is set",
-      dblock->name);
+    "forbid_pipe, forbid_file, and forbid_filter_reply must be set when "
+    "one_time is set",
+    dblock->name);
 }
 
 
@@ -633,7 +637,7 @@ is not enabled. */
 
 s = filebuf;
 tag = "# exim filter";
-while (isspace(*s)) s++;           /* Skips initial blank lines */
+while (isspace((uschar)*s)) s++;     /* Skips initial blank lines */
 for (; *tag != 0; s++, tag++)
   {
   if (*tag == ' ')
@@ -753,16 +757,16 @@ else
   to the directory or lock out if none. */
 
   extracted = parse_extract_addresses(filebuf,
-    addr->domain,                              /* to qualify \name */
-    &generated,                                /* where to hang them */
-    &error,                                    /* for errors */
-    TRUE,                                      /* no addresses => no error */
-    FALSE,                                     /* don't recognize :blackhole: */
-    ob->rewrite,                               /* rewrite if configured */
-    ob->forbid_include? "*" :                  /* includes forbidden */
-    restore_uid? NULL :                        /* if seteuid, try all */
-    (directory == NULL)? "*" :                 /* if no directory, lock out */
-    directory,                                 /* else restrain to directory */
+    addr->domain,                      /* to qualify \name */
+    &generated,                        /* where to hang them */
+    &error,                            /* for errors */
+    TRUE,                              /* no addresses => no error */
+    FALSE,                             /* don't recognize :blackhole: etc. */
+    ob->rewrite,                       /* rewrite if configured */
+    ob->forbid_include? "*" :          /* includes forbidden */
+    restore_uid? NULL :                /* if seteuid, try all */
+    (directory == NULL)? "*" :         /* if no directory, lock out */
+    directory,                         /* else restrain to directory */
     ob->skip_syntax_errors? &eblock : NULL);
 
   store_pool = POOL_MAIN;                      /* Reset for error strings */
