@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -146,7 +146,7 @@ if (dblock->home_directory != NULL) addr->home_dir = dblock->home_directory;
 addr->current_dir = dblock->current_directory;
 
 return route_queue(addr, addr_local, addr_remote, (router_instance *)dblock,
-  "director")? OK : ERROR;
+  NULL, "director")? OK : ERROR;
 }
 
 /* End of director/localuser.c */
