@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -166,7 +166,7 @@ if (ob->new_address == NULL)
   addr->home_dir = dblock->home_directory;
   addr->current_dir = dblock->current_directory;
   return route_queue(addr, addr_local, addr_remote, (router_instance *)dblock,
-    "director")? OK : ERROR;
+    NULL, "director")? OK : ERROR;
   }
 
 
@@ -258,7 +258,7 @@ if (transport != NULL)
   /* The generic user/group values are picked up by this function */
 
   yield = route_queue(generated, addr_local, addr_remote,
-    (router_instance *)dblock, "director")? OK : ERROR;
+    (router_instance *)dblock, NULL, "director")? OK : ERROR;
   }
 
 
