@@ -2,30 +2,13 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for interfacing with the DNS. */
 
 #include "exim.h"
 
-/* While IPv6 is yet young: the definition of T_AAAA may not be included in
-arpa/nameser.h. Fudge it here. */
-
-#if HAVE_IPV6
-#ifndef T_AAAA
-#define T_AAAA 28
-#endif
-#endif
-
-
-/* Ancient systems (e.g. SunOS4) don't appear to have T_TXT defined in their
-header files. */
-
-#ifndef T_TXT
-#define T_TXT 16
-#endif
-
 
 static unsigned char *dns_answer = NULL;
 static int dns_answerlen = 0;
@@ -85,13 +68,16 @@ Arguments:
   length      length of the buffer
 
 Returns:      length of the expanded text
+              NOTE: this is not the same as the yield of dn_expand(), which is
+                "the size of *compressed* name", to quote the man page.
 */
 
 int
 dns_expand(unsigned char *ptr, char *buffer, int length)
 {
-return dn_expand(dns_answer, dns_answer + dns_answerlen, ptr,
+(void)dn_expand(dns_answer, dns_answer + dns_answerlen, ptr,
     (DN_EXPAND_ARG4_TYPE)buffer, length);
+return (int)strlen(buffer);
 }
 
 
@@ -243,6 +229,15 @@ Returns:    DNS_SUCCEED   successful lookup
 int
 dns_basic_lookup(char *name, int type)
 {
+/* If we are running in the test harness, recognize a couple of special
+names that always give error returns. This makes it straightforward to
+test the handling of DNS errors. */
+
+if (strcmp(running_status, "<<<testing>>>") == 0)
+  {
+  if (strcmp(name, "test.again.dns") == 0) return DNS_AGAIN;
+  if (strcmp(name, "test.fail.dns") == 0) return DNS_FAIL;
+  }
 
 /* If configured, check the hygene of the name passed to lookup. Otherwise,
 although DNS lookups may give REFUSED at the lower level, some resolvers
