@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -43,12 +43,16 @@ are needed by any of the other headers, including system headers. */
 #include <limits.h>
 #endif
 
-/* Just in case some aged system doesn't define it... */
+/* Just in case some aged system doesn't define them... */
 
 #ifndef INT_MAX
 #define INT_MAX 2147483647
 #endif
 
+#ifndef UCHAR_MAX
+#define UCHAR_MAX 255
+#endif
+
 #include <sys/types.h>
 #include <dirent.h>
 #include <netdb.h>
@@ -175,6 +179,20 @@ present Exim doesn't actually use it. */
 #include <arpa/inet.h>
 #include <arpa/nameser.h>
 
+/* While IPv6 is still young the definition of T_AAAA may not be included in
+arpa/nameser.h. Fudge it here. */
+
+#ifndef T_AAAA
+#define T_AAAA 28
+#endif
+
+/* Ancient systems (e.g. SunOS4) don't appear to have T_TXT defined in their
+header files. */
+
+#ifndef T_TXT
+#define T_TXT 16
+#endif
+
 /* The resolv.h header defines __P(x) on some Solaris 2.5.1 systems (without
 checking that it is already defined, in fact). This conflicts with other
 headers that behave likewise (see below), leading to compiler warnings. Arrange
