@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -188,8 +188,9 @@ static unsigned int prime[] = {
 *************************************************/
 
 /* This function is called to expand a string, and test the result for a "true"
-or "false" value. Failure of the expansion is panic-worthy. All store used by
-the function can be released on exit.
+or "false" value. Failure of the expansion is panic-worthy, unless forced or
+the result of a lookup defer. All store used by the function can be released on
+exit.
 
 Arguments:
   condition     the condition string
@@ -205,7 +206,7 @@ expand_check_condition(char *condition, char *m1, char *m2)
 int rc;
 void *reset_point = store_get(0);
 char *ss = expand_string(condition);
-if (ss == NULL && !expand_string_forcedfail)
+if (ss == NULL && !expand_string_forcedfail && !search_find_defer)
   log_write(0, LOG_MAIN|LOG_PANIC_DIE, "failed to expand condition\"%s\" "
     "for %s %s: %s", condition, m1, m2, expand_string_message);
 if (ss == NULL) return FALSE;
@@ -239,7 +240,7 @@ static char *
 read_name(char *name, int max, char *s, char *extras)
 {
 int ptr = 0;
-while (*s != 0 && (isalnum(*s) || strchr(extras, *s) != NULL))
+while (*s != 0 && (isalnum((uschar)*s) || strchr(extras, *s) != NULL))
   {
   if (ptr < max-1) name[ptr++] = *s;
   s++;
@@ -303,7 +304,7 @@ static char *
 read_number(int *n, char *s)
 {
 *n = 0;
-while (isdigit(*s)) *n = *n * 10 + (*s++ - '0');
+while (isdigit((uschar)*s)) *n = *n * 10 + (*s++ - '0');
 return s;
 }
 
@@ -329,7 +330,7 @@ char *
 expand_getkeyed(char *key, char *s)
 {
 int length = (int)strlen(key);
-while (isspace(*s)) s++;
+while (isspace((uschar)*s)) s++;
 
 /* Loop to search for the key */
 
@@ -339,11 +340,11 @@ while (*s != 0)
   char *data;
   char *subkey = s;
 
-  while (*s != 0 && *s != '=' && !isspace(*s)) s++;
+  while (*s != 0 && *s != '=' && !isspace((uschar)*s)) s++;
   subkeylength = s - subkey;
 
-  while (isspace(*s)) s++;
-  if (*s == '=') while (isspace(*(++s)));
+  while (isspace((uschar)*s)) s++;
+  if (*s == '=') while (isspace((uschar)(*(++s))));
 
   /* For now, just find the end of the data field - interpret quoted
   string later if it is actually wanted. */
@@ -357,7 +358,7 @@ while (*s != 0)
       }
     if (*s == '\"') s++;
     }
-  else while (*s != 0 && !isspace(*s)) s++;
+  else while (*s != 0 && !isspace((uschar)*s)) s++;
   datalength = s - data;
 
   /* If keys match, set up the subfield as the yield and return. If
@@ -370,10 +371,10 @@ while (*s != 0)
 
     if (*data == '\"')
       {
-      int i = 0;
-      for (;;)
+      int ch, i;
+      for (i = 0;;)
         {
-        int ch = *(++data);
+        ch = *(++data);
         if (ch == 0 || ch == '\"') break;
         if (ch == '\\') ch = string_interpret_escape(&data);
         yield[i++] = ch;
@@ -392,7 +393,7 @@ while (*s != 0)
 
   /* Move on to next subkey */
 
-  while (isspace(*s)) s++;
+  while (isspace((uschar)*s)) s++;
   }
 
 return NULL;
@@ -588,7 +589,7 @@ while (last > first)
       if (h->type == htype_from && s == NULL) s = strchr(h->text, ':') + 1;
       }
     if (s == NULL) return "";
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
 
     /* Disregard final \n in header (but note that isspace() might already
     have skipped over it if the header is empty). */
@@ -600,6 +601,7 @@ while (last > first)
     or if there is only one recipient, but not elsewhere. */
 
     case vtype_recipients:
+
     if (!enable_dollar_recipients) return NULL; else
       {
       int size = 128;
@@ -661,7 +663,7 @@ for (h = header_list; h != NULL && size < 64 * 1024; h = h->next)
       if (yield == NULL)
         {
         yield = h->text + len;
-        while (isspace(*yield)) yield++;
+        while (isspace((uschar)*yield)) yield++;
         }
 
       /* If this is not the first header, see if it's the second, and in that
@@ -721,7 +723,7 @@ char name[256];
 
 for (;;)
   {
-  while (isspace(*s)) s++;
+  while (isspace((uschar)*s)) s++;
   if (*s == '!') { testfor = !testfor; s++; } else break;
   }
 
@@ -808,7 +810,7 @@ else if (strcmp(name, "exists") == 0
   {
   char *sub;
   struct stat statbuf;
-  while (isspace(*s)) s++;
+  while (isspace((uschar)*s)) s++;
   if (*s != '{') goto COND_FAILED_CURLY;
   sub = expand_string_internal(s+1, TRUE, &s, FALSE);
   if (sub == NULL) return NULL;
@@ -848,7 +850,7 @@ symbolic operators for numeric testing
 else if (strcmp(name, "eq") == 0 ||
          strcmp(name, "match") == 0 ||
          strcmp(name, "crypteq") == 0 ||
-         !isalpha(name[0]))
+         !isalpha((uschar)name[0]))
   {
   int i;
   int roffset;
@@ -859,7 +861,7 @@ else if (strcmp(name, "eq") == 0 ||
 
   for (i = 0; i < 2; i++)
     {
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (*s != '{') goto COND_FAILED_CURLY;
     sub[i] = expand_string_internal(s+1, TRUE, &s, yield == NULL);
     if (sub[i] == NULL) return NULL;
@@ -867,7 +869,7 @@ else if (strcmp(name, "eq") == 0 ||
 
     /* Convert to numerical if required */
 
-    if (!isalpha(name[0]))
+    if (!isalpha((uschar)name[0]))
       {
       char *endptr;
       num[i] = (int)strtol((const char *)sub[i], &endptr, 10);
@@ -971,16 +973,16 @@ else if (strcmp(name, "or") == 0 || strcmp(name, "and") == 0)
   BOOL and = strcmp(name, "and") == 0;
   int comb = and;
 
-  while (isspace(*s)) s++;
+  while (isspace((uschar)*s)) s++;
   if (*s++ != '{') goto COND_FAILED_CURLY;
 
   for (;;)
     {
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (*s != '{') break;
     s = eval_condition(s+1, ptr);
     if (s == NULL) return NULL;
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (*s++ != '}') goto COND_FAILED_CURLY;
     if (yield != NULL)
       {
@@ -1017,6 +1019,107 @@ return NULL;
 
 
 
+/*************************************************
+*            Read and expand n substrings        *
+*************************************************/
+
+/* This function is called to read and expand argument substrings for various
+expansion items.
+
+Arguments:
+  sub        points to vector of pointers to set
+  n          number of substrings
+  sptr       points to current string pointer
+
+Returns:     0 OK; string pointer updated
+             1 curly bracketing error
+             2 other error
+*/
+
+static int
+read_subs(char **sub, int n, char **sptr)
+{
+int i;
+char *s = *sptr;
+
+while (isspace((uschar)*s)) s++;
+for (i = 0; i < n; i++)
+  {
+  if (*s != '{') return 1;
+  sub[i] = expand_string_internal(s+1, TRUE, &s, FALSE);
+  if (sub[i] == NULL) return 2;
+  while (isspace((uschar)*s)) s++;
+  if (*s++ != '}') return 1;
+  while (isspace((uschar)*s)) s++;
+  }
+if (*s++ != '}') return 1;
+
+*sptr = s;
+return 0;
+}
+
+
+
+
+/*************************************************
+*          Save numerical variables              *
+*************************************************/
+
+/* This function is called from items such as "if" that want to preserve and
+restore the numbered variables.
+
+Arguments:
+  save_expand_string    points to an array of pointers to set
+  save_expand_nlength   points to an array of ints for the lengths
+
+Returns:                the value of expand max to save
+*/
+
+static int
+save_expand_strings(char **save_expand_nstring, int *save_expand_nlength)
+{
+int i;
+for (i = 0; i <= expand_nmax; i++)
+  {
+  save_expand_nstring[i] = expand_nstring[i];
+  save_expand_nlength[i] = expand_nlength[i];
+  }
+return expand_nmax;
+}
+
+
+
+/*************************************************
+*           Restore numerical variables          *
+*************************************************/
+
+/* This function restored saved values of numerical strings.
+
+Arguments:
+  save_expand_nmax      the number of strings to restore
+  save_expand_string    points to an array of pointers
+  save_expand_nlength   points to an array of ints
+
+Returns:                nothing
+*/
+
+static void
+restore_expand_strings(int save_expand_nmax, char **save_expand_nstring,
+  int *save_expand_nlength)
+{
+int i;
+expand_nmax = save_expand_nmax;
+for (i = 0; i <= expand_nmax; i++)
+  {
+  expand_nstring[i] = save_expand_nstring[i];
+  expand_nlength[i] = save_expand_nlength[i];
+  }
+}
+
+
+
+
+
 /*************************************************
 *                 Expand string                  *
 *************************************************/
@@ -1024,33 +1127,35 @@ return NULL;
 /* Returns either an unchanged string, or the expanded string in stacking pool
 store. Interpreted sequences are:
 
-   \...                  normal escaping rules
-   $name                 substitutes the variable
-   ${name}               ditto
-   ${op:string}          operates on the string value
+   \...                    normal escaping rules
+   $name                   substitutes the variable
+   ${name}                 ditto
+   ${op:string}            operates on the string value
    ${extract {key} {string}}
-                         extracts keyed substring; null if not found
+                           extracts keyed substring; null if not found
    ${extract {field} {separator} {string}}
-                         extracts numbered field from separated string;
-                         null if not found; field 0 is the whole string
-   ${if cond {s1} {s2}}  conditional; the yielded string is expanded
-                         {s2} can be replaced by "fail" or be omitted
-   ${perl{sub}{a1}...    call Perl (if configured)
+                           extracts numbered field from separated string;
+                           null if not found; field 0 is the whole string
+   ${if cond {s1} {s2}}    conditional; the yielded string is expanded
+                           {s2} can be replaced by "fail" or be omitted
+   ${perl{sub}{a1}...      call Perl (if configured)
+   ${sg {source}{s1}{s2}}  global substitution
+   ${tr {source}{s1}{s2}}  translate characters
 
 One-key+file lookups:
    ${lookup{key}search-type{file}{found}{not-found}}
-                         the key, file, & strings are expanded; $value available
-                         and {not-found} can be replaced by "fail" or be
-                         omitted.
-                         The key can in fact consist of mainkey:subkey, in
-                         which case a subfield is extracted from the found
-                         string, which must consist of key=value pairs.
+                           the key, file, & strings are expanded; $value
+                           is available in {found}, and {not-found} can be
+                           replaced by "fail" or be omitted.
+                           The key can in fact consist of mainkey:subkey, in
+                           which case a subfield is extracted from the found
+                           string, which must consist of key=value pairs.
 
 Database-query lookups:
    ${lookup search-type{query}{found}{not-found}}
-                         the query and strings are expanded; $value available
-                         and {not-found} can be replaced by "fail" or be
-                         omitted.
+                           the query and strings are expanded; $value available
+                           and {not-found} can be replaced by "fail" or be
+                           omitted.
 
 Operators:
    domain                  extracts domain from an address
@@ -1177,7 +1282,7 @@ while (*s != 0)
   "$header_". A non-existent header yields a NULL value; nothing is
   inserted. */
 
-  if (isalpha(*(++s)))
+  if (isalpha((uschar)(*(++s))))
     {
     int choplen = 0;
     s = read_name(name, sizeof(name), s, "_");
@@ -1206,7 +1311,7 @@ while (*s != 0)
     continue;
     }
 
-  if (isdigit(*s))
+  if (isdigit((uschar)*s))
     {
     int n;
     s = read_number(&n, s);
@@ -1227,7 +1332,7 @@ while (*s != 0)
   /* After { there can be various things, but they all start with
   an initial word, except for a number for a string match variable. */
 
-  if (isdigit(*(++s)))
+  if (isdigit((uschar)(*(++s))))
     {
     int n;
     s = read_number(&n, s);
@@ -1242,7 +1347,7 @@ while (*s != 0)
     continue;
     }
 
-  if (!isalpha(*s))
+  if (!isalpha((uschar)*s))
     {
     expand_string_message = "letter or digit expected after ${";
     goto EXPAND_FAILED;
@@ -1265,14 +1370,8 @@ while (*s != 0)
     char *sub1, *sub2;
     char *save_expand_nstring[EXPAND_MAXN+1];
     int save_expand_nlength[EXPAND_MAXN+1];
-    int save_expand_nmax = expand_nmax;
-    int i;
-
-    for (i = 0; i <= expand_nmax; i++)
-      {
-      save_expand_nstring[i] = expand_nstring[i];
-      save_expand_nlength[i] = expand_nlength[i];
-      }
+    int save_expand_nmax =
+      save_expand_strings(save_expand_nstring, save_expand_nlength);
 
     s = eval_condition(s, skipping? NULL : &cond);
     if (s == NULL) goto EXPAND_FAILED;  /* message already set */
@@ -1281,7 +1380,7 @@ while (*s != 0)
     enclosed in braces, with optional space between them and
     after them. Then there must be a final } to end. */
 
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (*s != '{') goto EXPAND_FAILED_CURLY;
 
     /* The skipping flag is already TRUE if we are in the middle of
@@ -1298,7 +1397,7 @@ while (*s != 0)
     expand_string_forcedfail = FALSE;
     if (*s++ != '}') goto EXPAND_FAILED_CURLY;
 
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (*s == '}') sub2 = NULL;
     else if (*s == '{')
       {
@@ -1328,7 +1427,7 @@ while (*s != 0)
         }
       }
 
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (*s++ != '}') goto EXPAND_FAILED_CURLY;
 
     /* Add the appropriate string to the result */
@@ -1341,13 +1440,8 @@ while (*s != 0)
     /* Restore external setting of expansion variables for continuation
     at this level. */
 
-    expand_nmax = save_expand_nmax;
-    for (i = 0; i <= expand_nmax; i++)
-      {
-      expand_nstring[i] = save_expand_nstring[i];
-      expand_nlength[i] = save_expand_nlength[i];
-      }
-
+    restore_expand_strings(save_expand_nmax, save_expand_nstring,
+      save_expand_nlength);
     continue;
     }
 
@@ -1362,6 +1456,7 @@ while (*s != 0)
     {
     int stype;
     int pv = -1;
+    int expand_setup = 0;
     BOOL looked_up;
     char *ss, *key, *filename, *sub1;
     char *lookup_errmsg = NULL;
@@ -1369,57 +1464,49 @@ while (*s != 0)
 
     char *save_expand_nstring[EXPAND_MAXN+1];
     int save_expand_nlength[EXPAND_MAXN+1];
-    int save_expand_nmax = expand_nmax;
-    int expand_setup = 0;
-    int i;
+    int save_expand_nmax =
+      save_expand_strings(save_expand_nstring, save_expand_nlength);
 
     if (expand_forbid_lookup)
       {
       expand_string_message = "File lookups are not permitted";
       goto EXPAND_FAILED;
       }
-
-    for (i = 0; i <= expand_nmax; i++)
-      {
-      save_expand_nstring[i] = expand_nstring[i];
-      save_expand_nlength[i] = expand_nlength[i];
-      }
-
     lookup_value = NULL;
 
     /* Get the key we are to look up for single-key+file style lookups.
     Otherwise set the key NULL pro-tem. */
 
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (*s == '{')
       {
       key = expand_string_internal(s+1, TRUE, &s, FALSE);
       if (key == NULL) goto EXPAND_FAILED;
       if (*s++ != '}') goto EXPAND_FAILED_CURLY;
-      while (isspace(*s)) s++;
+      while (isspace((uschar)*s)) s++;
       }
     else key = NULL;
 
     /* Find out the type of database */
 
-    if (!isalpha(*s))
+    if (!isalpha((uschar)*s))
       {
       expand_string_message = "missing lookup type";
       goto EXPAND_FAILED;
       }
     s = read_name(name, sizeof(name), s, "-_");
     ss = name;
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
 
     /* Handle request for partial lookup */
 
     if (strncmp(ss, "partial", 7) == 0)
       {
       ss += 7;
-      if (isdigit (*ss))
+      if (isdigit ((uschar)*ss))
         {
         pv = 0;
-        while (isdigit(*ss)) pv = pv*10 + *ss++ - '0';
+        while (isdigit((uschar)*ss)) pv = pv*10 + *ss++ - '0';
         }
       else pv = 2;
       if (*ss++ != '-')
@@ -1439,13 +1526,13 @@ while (*s != 0)
     if (strncmp(s, "*@", 2) == 0)
       {
       s += 2;
-      while (isspace(*s)) s++;
+      while (isspace((uschar)*s)) s++;
       pv += 1024 + 2048;
       }
     else if (*s == '*')
       {
       s++;
-      while (isspace(*s)) s++;
+      while (isspace((uschar)*s)) s++;
       pv += 1024;
       }
 
@@ -1492,7 +1579,7 @@ while (*s != 0)
     filename = expand_string_internal(s+1, TRUE, &s, FALSE);
     if (filename == NULL) goto EXPAND_FAILED;
     if (*s++ != '}') goto EXPAND_FAILED_CURLY;
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
 
     /* If this isn't a single-key+file lookup, re-arrange the variables
     to be appropriate for the search_ functions. */
@@ -1542,7 +1629,7 @@ while (*s != 0)
     skipping in the call if the lookup failed (this will always be the case
     if we are already skipping). */
 
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (*s != '{') goto EXPAND_FAILED_CURLY;
     sub1 = expand_string_internal(s+1, TRUE, &s, !looked_up);
 
@@ -1564,7 +1651,7 @@ while (*s != 0)
     We must set skipping in the nested call if this lookup succeeded, or if
     we are already skipping. */
 
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (*s == '{')
       {
       char *sub2 = expand_string_internal(s+1, TRUE, &s, looked_up || skipping);
@@ -1587,7 +1674,7 @@ while (*s != 0)
         {
         if (!looked_up)
           {
-          while (isspace(*s)) s++;
+          while (isspace((uschar)*s)) s++;
           if (*s++ != '}') goto EXPAND_FAILED_CURLY;
           expand_string_message =
             string_sprintf("lookup failed and \"fail\" requested%s%s",
@@ -1604,18 +1691,14 @@ while (*s != 0)
         }
       }
 
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (*s++ != '}') goto EXPAND_FAILED_CURLY;
 
     /* Restore external setting of expansion variables for carrying on
     at this level. */
 
-    expand_nmax = save_expand_nmax;
-    for (i = 0; i <= expand_nmax; i++)
-      {
-      expand_nstring[i] = save_expand_nstring[i];
-      expand_nlength[i] = save_expand_nlength[i];
-      }
+    restore_expand_strings(save_expand_nmax, save_expand_nstring,
+      save_expand_nlength);
 
     /* Restore lookup_value and continue */
 
@@ -1644,14 +1727,14 @@ while (*s != 0)
       goto EXPAND_FAILED;
       }
 
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (*s != '{') goto EXPAND_FAILED_CURLY;
     sub_name = expand_string_internal(s+1, TRUE, &s, FALSE);
     if (sub_name == NULL) goto EXPAND_FAILED;
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (*s++ != '}') goto EXPAND_FAILED_CURLY;
 
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
 
     while (*s == '{')
       {
@@ -1664,9 +1747,9 @@ while (*s != 0)
         }
       sub_arg[i] = expand_string_internal(s+1, TRUE, &s, FALSE);
       if (sub_arg[i++] == NULL) goto EXPAND_FAILED;
-      while (isspace(*s)) s++;
+      while (isspace((uschar)*s)) s++;
       if (*s++ != '}') goto EXPAND_FAILED_CURLY;
-      while (isspace(*s)) s++;
+      while (isspace((uschar)*s)) s++;
       }
 
     if (*s++ != '}') goto EXPAND_FAILED_CURLY;
@@ -1724,6 +1807,150 @@ while (*s != 0)
     }
   #endif /* EXIM_PERL */
 
+  /* Handle character translation for "tr" */
+
+  if (strcmp(name, "tr") == 0)
+    {
+    int oldptr = ptr;
+    int o2m;
+    char *sub[3];
+
+    switch(read_subs(sub, 3, &s))
+      {
+      case 1: goto EXPAND_FAILED_CURLY;
+      case 2: goto EXPAND_FAILED;
+      }
+
+    yield = string_cat(yield, &size, &ptr, sub[0], (int)strlen(sub[0]));
+    o2m = (int)strlen(sub[2]) - 1;
+
+    if (o2m >= 0) for (; oldptr < ptr; oldptr++)
+      {
+      char *m = strrchr(sub[1], yield[oldptr]);
+      if (m != NULL)
+        {
+        int o = m - sub[1];
+        yield[oldptr] = sub[2][(o < o2m)? o : o2m];
+        }
+      }
+
+    continue;
+    }
+
+  /* Handle global substitution for "sg" - like Perl's s/xxx/yyy/g operator.
+  We have to save the numerical variables and restore them afterwards. */
+
+  if (strcmp(name, "sg") == 0)
+    {
+    pcre *re;
+    int moffset, moffsetextra, slen;
+    int roffset;
+    int emptyopt;
+    const char *rerror;
+    char *subject;
+    char *sub[3];
+
+    char *save_expand_nstring[EXPAND_MAXN+1];
+    int save_expand_nlength[EXPAND_MAXN+1];
+    int save_expand_nmax =
+      save_expand_strings(save_expand_nstring, save_expand_nlength);
+
+    switch(read_subs(sub, 3, &s))
+      {
+      case 1: goto EXPAND_FAILED_CURLY;
+      case 2: goto EXPAND_FAILED;
+      }
+
+    /* Compile the regular expression */
+
+    re = pcre_compile(sub[1], PCRE_COPT, &rerror, &roffset, NULL);
+    if (re == NULL)
+      {
+      expand_string_message = string_sprintf("regular expression error in "
+        "\"%s\": %s at offset %d", sub[1], rerror, roffset);
+      goto EXPAND_FAILED;
+      }
+
+    /* Now run a loop to do the substitutions as often as necessary. It ends
+    when there are no more matches. Take care over matches of the null string;
+    do the same thing as Perl does. */
+
+    subject = sub[0];
+    slen = (int)strlen(sub[0]);
+    moffset = moffsetextra = 0;
+    emptyopt = 0;
+
+    for (;;)
+      {
+      int ovector[3*(EXPAND_MAXN+1)];
+      int n = pcre_exec(re, NULL, subject, slen, moffset + moffsetextra,
+        PCRE_EOPT | emptyopt, ovector, sizeof(ovector)/sizeof(int));
+      int nn;
+      char *insert;
+
+      /* No match - if we previously set PCRE_NOTEMPTY after a null match, this
+      is not necessarily the end. We want to repeat the match from one
+      character further along, but leaving the basic offset the same (for
+      copying below). We can't be at the end of the string - that was checked
+      before setting PCRE_NOTEMPTY. If PCRE_NOTEMPTY is not set, we are
+      finished; copy the remaining string and end the loop. */
+
+      if (n < 0)
+        {
+        if (emptyopt != 0)
+          {
+          moffsetextra = 1;
+          emptyopt = 0;
+          continue;
+          }
+        yield = string_cat(yield, &size, &ptr, subject+moffset, slen-moffset);
+        break;
+        }
+
+      /* Match - set up for expanding the replacement. */
+
+      if (n == 0) n = EXPAND_MAXN + 1;
+      expand_nmax = 0;
+      for (nn = 0; nn < n*2; nn += 2)
+        {
+        expand_nstring[expand_nmax] = subject + ovector[nn];
+        expand_nlength[expand_nmax++] = ovector[nn+1] - ovector[nn];
+        }
+      expand_nmax--;
+
+      /* Copy the characters before the match, plus the expanded insertion. */
+
+      yield = string_cat(yield, &size, &ptr, subject + moffset,
+        ovector[0] - moffset);
+      insert = expand_string(sub[2]);
+      if (insert == NULL) goto EXPAND_FAILED;
+      yield = string_cat(yield, &size, &ptr, insert, (int)strlen(insert));
+
+      moffset = ovector[1];
+      moffsetextra = 0;
+      emptyopt = 0;
+
+      /* If we have matched an empty string, first check to see if we are at
+      the end of the subject. If so, the loop is over. Otherwise, mimic
+      what Perl's /g options does. This turns out to be rather cunning. First
+      we set PCRE_NOTEMPTY and PCRE_ANCHORED and try the match a non-empty
+      string at the same point. If this fails (picked up above) we advance to
+      the next character. */
+
+      if (ovector[0] == ovector[1])
+        {
+        if (ovector[0] == slen) break;
+        emptyopt = PCRE_NOTEMPTY | PCRE_ANCHORED;
+        }
+      }
+
+    /* All done - restore numerical variables. */
+
+    restore_expand_strings(save_expand_nmax, save_expand_nstring,
+      save_expand_nlength);
+    continue;
+    }
+
   /* Handle keyed and numbered substring extraction. They are distinguised
   by different numbers of arguments. */
 
@@ -1733,7 +1960,7 @@ while (*s != 0)
     char *sub[4];
     for (i = 0; i < 3; i++)
       {
-      while (isspace(*s)) s++;
+      while (isspace((uschar)*s)) s++;
       if (*s == '{')
         {
         sub[i] = expand_string_internal(s+1, TRUE, &s, FALSE);
@@ -1748,7 +1975,7 @@ while (*s != 0)
         }
       }
 
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (*s++ != '}') goto EXPAND_FAILED_CURLY;
 
     /* If there are three arguments, this is a numbered field extraction,
@@ -1906,7 +2133,7 @@ while (*s != 0)
       char *t = sub - 1;
       while (*(++t) != 0)
         {
-        if (*t != '_' && *t != '-' && *t != '.' && !isalnum(*t))
+        if (*t != '_' && *t != '-' && *t != '.' && !isalnum((uschar)*t))
           { needs_quote = TRUE; break; }
         }
 
@@ -1945,7 +2172,8 @@ while (*s != 0)
       char *t = sub - 1;
       while (*(++t) != 0)
         {
-        if (!isalnum(*t)) yield = string_cat(yield, &size, &ptr, "\\", 1);
+        if (!isalnum((uschar)*t))
+          yield = string_cat(yield, &size, &ptr, "\\", 1);
         yield = string_cat(yield, &size, &ptr, t, 1);
         }
       continue;
@@ -1971,7 +2199,7 @@ while (*s != 0)
     characters from the set a-z; hash_n_m makes a hash of length n, but
     uses m characters from the set a-zA-Z0-9.
 
-    nhash_n returns a single number between 0 and n (in text form), while
+    nhash_n returns a single number between 0 and n-1 (in text form), while
     nhash_n_m returns a div/mod hash as two numbers "a/b". The first lies
     between 0 and n-1 and the second between 0 and m-1. */
 
@@ -2015,7 +2243,7 @@ while (*s != 0)
           value2 = 0;
           num++;
           }
-        else if (!isdigit(*num))
+        else if (!isdigit((uschar)*num))
           {
           expand_string_message =
             string_sprintf("non-digit after underscore in \"%s\"", name);
@@ -2299,6 +2527,7 @@ char buffer[1024];
 
 debug_level = 1;
 debug_file = stderr;
+debug_fd = fileno(debug_file);
 debug_trace_memory = 0;
 big_buffer = malloc(big_buffer_size);
 
@@ -2321,6 +2550,9 @@ for (i = 1; i < argc; i++)
       #ifdef LOOKUP_MYSQL
       mysql_servers = argv[i];
       #endif
+      #ifdef LOOKUP_PGSQL
+      pgsql_servers = argv[i];
+      #endif
       }
   #ifdef EXIM_PERL
   else opt_perl_startup = argv[i];
