@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -241,7 +241,7 @@ nextsigchar(char *ptr, BOOL comment_allowed)
 {
 for (;;)
   {
-  while (isspace(*ptr))
+  while (isspace((uschar)*ptr))
     {
     if (*ptr == '\n') line_number++;
     ptr++;
@@ -278,7 +278,7 @@ char *
 nextword(char *ptr, char *buffer, int size, BOOL bracket)
 {
 char *bp = buffer;
-while (*ptr != 0 && !isspace(*ptr) &&
+while (*ptr != 0 && !isspace((uschar)*ptr) &&
        (!bracket || (*ptr != '(' && *ptr != ')')))
   {
   if (bp - buffer < size - 1) *bp++ = *ptr++; else
@@ -327,15 +327,15 @@ while (*(++ptr) != 0 && *ptr != '\"' && *ptr != '\n')
 
   if (*ptr != '\\') *bp++ = *ptr; else
     {
-    if (isspace(ptr[1]))    /* \<whitespace>NL<whitespace> ignored */
+    if (isspace((uschar)ptr[1]))    /* \<whitespace>NL<whitespace> ignored */
       {
       char *p = ptr + 1;
-      while (*p != '\n' && isspace(*p)) p++;
+      while (*p != '\n' && isspace((uschar)*p)) p++;
       if (*p == '\n')
         {
         line_number++;
         ptr = p;
-        while (ptr[1] != '\n' && isspace(ptr[1])) ptr++;
+        while (ptr[1] != '\n' && isspace((uschar)ptr[1])) ptr++;
         continue;
         }
       }
@@ -376,7 +376,7 @@ int value, count;
 if (sscanf(s, "%i%n", &value, &count) != 1) return 0;
 if (tolower(s[count]) == 'k') { value *= 1024; count++; }
 if (tolower(s[count]) == 'm') { value *= 1024*1024; count++; }
-while (isspace(s[count])) count++;
+while (isspace((uschar)(s[count]))) count++;
 if (s[count] != 0) return 0;
 *OK = TRUE;
 return value;
@@ -890,7 +890,8 @@ switch (command)
 
   /* Logwrite, logfile, pipe, and testprint all take a single argument, save
   and logfile can have an option second argument for the mode, and deliver can
-  have "errors_to <address>" in a system filter. */
+  have "errors_to <address>" in a system filter, or in a user filter if the
+  address is the current one. */
 
   case deliver_command:
   case logfile_command:
@@ -960,7 +961,9 @@ switch (command)
 
     /* The argument for the logwrite command must end in a newline, and the save
     and logfile commands can have an optional mode argument. The deliver
-    command can have an optional "errors_to <address>" for a system filter. */
+    command can have an optional "errors_to <address>" for a system filter,
+    or for a user filter if the address is the user's address. Accept the
+    syntax here - the check is later. */
 
     else
       {
@@ -974,7 +977,7 @@ switch (command)
 
       if (command == save_command || command == logfile_command)
         {
-        if (isdigit(*ptr))
+        if (isdigit((uschar)*ptr))
           {
           ptr = nextword(ptr, buffer, sizeof(buffer), FALSE);
           second_argument = (void *)strtol(buffer, NULL, 8);
@@ -982,7 +985,7 @@ switch (command)
         else second_argument = (void *)(-1);
         }
 
-      else if (command == deliver_command && system_filtering)
+      else if (command == deliver_command)
         {
         char *save_ptr = ptr;
         ptr = nextword(ptr, buffer, sizeof(buffer), FALSE);
@@ -1809,10 +1812,21 @@ while (commands != NULL)
       }
 
     /* Stick the errors address into a simple variable, as it will
-    be referenced a few times. */
+    be referenced a few times. Check that the caller is permitted to
+    specify it. */
 
     s = expargs[1];
 
+    if (s != NULL && !system_filtering)
+      {
+      char *ownaddress = expand_string("$local_part@$domain");
+      if (strcmpic(ownaddress, s) != 0)
+        {
+        *error_pointer = "errors_to must point to the caller's address";
+        return FALSE;
+        }
+      }
+
     /* Test case: report what would happen */
 
     if (filter_test != NULL)
@@ -1844,7 +1858,7 @@ while (commands != NULL)
 
       store_pool = POOL_MAIN;
       addr = deliver_make_addr(expargs[0], TRUE);  /* TRUE => copy s */
-      addr->errors_address = expargs[1];           /* Default is NULL */
+      addr->errors_address = s;                    /* Default is NULL */
       addr->ignore_error = commands->noerror;
       addr->next = *generated;
       *generated = addr;
@@ -2042,7 +2056,7 @@ while (commands != NULL)
 
       else if (adding)
         {
-        while (isspace(*s)) s++;
+        while (isspace((uschar)*s)) s++;
         if (s[0] != 0)
           {
           header_add(htype_other, "%s%s", s, (s[(int)strlen(s)-1] == '\n')?
@@ -2055,11 +2069,12 @@ while (commands != NULL)
         header_line *h;
         for (h = header_list; h != NULL; h = h->next)
           {
+          int sep = 0;
           char *ss, *tt;
           char *list = s;
           char buffer[128];
           if (h->type == htype_old) continue;
-          while ((ss = string_nextinlist(&list, ':', buffer, sizeof(buffer)))
+          while ((ss = string_nextinlist(&list, &sep, buffer, sizeof(buffer)))
                  != NULL)
             {
             int len = (int)strlen(ss);
@@ -2180,7 +2195,7 @@ while (commands != NULL)
             command_list[commands->command]);
           return FALSE;
           }
-        else if (i < mailarg_index_text && c == '\n' && !isspace(p[1]))
+        else if (i < mailarg_index_text && c == '\n' && !isspace((uschar)p[1]))
           {
           *error_pointer = string_sprintf("\\n not followed by space in \"%s\" "
             "in %s command", string_printing(s),
@@ -2225,7 +2240,7 @@ while (commands != NULL)
       {
       char *to = (char *)(commands->args[mailarg_index_to]);
       if (to == NULL) to = expand_string("$reply_address");
-      while (isspace(*to)) to++;
+      while (isspace((uschar)*to)) to++;
 
       DEBUG(10)
         {
@@ -2559,7 +2574,7 @@ it does not, treat it as an ordinary .forward file and check that. */
 
 s = filebuf;
 tag = "# exim filter";
-while (isspace(*s)) s++;           /* Skips initial blank lines */
+while (isspace((uschar)*s)) s++;      /* Skips initial blank lines */
 for (; *tag != 0; s++, tag++)
   {
   if (*tag == ' ')
