@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* All the global variables are defined together in this one module, so that
@@ -65,6 +65,10 @@ char  *ldap_default_servers   = NULL;
 char  *mysql_servers          = NULL;
 #endif
 
+#ifdef LOOKUP_PGSQL
+char *pgsql_servers	      = NULL;
+#endif
+
 #ifdef SUPPORT_MOVE_FROZEN_MESSAGES
 BOOL   move_frozen_messages   = FALSE;
 #endif
@@ -102,6 +106,7 @@ char  *address_pipe           = NULL;
 BOOL   address_test_mode      = FALSE;
 gid_t *admin_groups           = NULL;
 BOOL   admin_user             = FALSE;
+BOOL   allow_mx_to_ip         = FALSE;
 BOOL   allow_unqualified_recipient = TRUE;    /* For local messages */
 BOOL   allow_unqualified_sender = TRUE;       /* Reset for SMTP */
 BOOL   always_bcc             = FALSE;
@@ -150,6 +155,7 @@ int    continue_sequence      = 1;
 char  *continue_transport     = NULL;
 
 BOOL   daemon_listen          = FALSE;
+int    debug_fd               = -1;
 FILE  *debug_file             = NULL;
 int    debug_level            = -1;
 pid_t  debug_pid              = 0;
@@ -358,6 +364,9 @@ char  *local_domains          = NULL;
 BOOL   local_domains_include_host = FALSE;
 BOOL   local_domains_include_host_literals = FALSE;
 BOOL   local_error_message    = FALSE;
+BOOL   local_from_check     = TRUE;
+char  *local_from_prefix    = NULL;
+char  *local_from_suffix    = NULL;
 char  *local_interfaces       = NULL;
 ip_address_item *local_interface_data = NULL;
 char  *local_part_data        = NULL;
@@ -391,6 +400,7 @@ char  *message_body           = NULL;
 char  *message_body_end       = NULL;
 int    message_body_size      = 0;
 int    message_body_visible   = 500;
+int    message_ended          = END_NOTSTARTED;
 char  *message_filter         = NULL;
 
 char  *message_filter_directory_transport = NULL;
@@ -589,6 +599,7 @@ char  *self_hostname          = NULL;
 char  *sender_address         = NULL;
 BOOL   sender_address_forced  = FALSE;
 char  *sender_address_relay   = NULL;
+char  *sender_address_relay_hosts = "*";
 BOOL   sender_address_rewritten = FALSE;
 char  *sender_fullhost        = NULL;
 char  *sender_helo_name       = NULL;
@@ -632,7 +643,6 @@ BOOL   smtp_input             = FALSE;
 int    smtp_load_reserve      = -1;
 FILE  *smtp_out               = NULL;
 int    smtp_port              = -1;
-BOOL   smtp_reading_data      = FALSE;
 int    smtp_receive_timeout   = 5*60;
 char  *smtp_etrn_command      = NULL;
 char  *smtp_etrn_hosts        = NULL;
@@ -652,6 +662,8 @@ int    store_pool             = POOL_MAIN;
 BOOL   strip_excess_angle_brackets = FALSE;
 BOOL   strip_trailing_dot     = FALSE;
 
+BOOL   timestamps_utc         = FALSE;
+
 transport_instance *transports = NULL;
 
 transport_instance *transport_message_filter_directory = NULL;
@@ -757,7 +769,7 @@ char  *warnmsg_recipients     = NULL;
 char  *verify_address_parse_error = NULL;
 char  *verify_forced_errmsg   = NULL;
 BOOL   verify_only            = FALSE;
-char  *version_copyright      = "Copyright (c) University of Cambridge 1999";
+char  *version_copyright      = "Copyright (c) University of Cambridge 2000";
 char  *version_date           = "?";
 char  *version_cnumber        = "????";
 char  *version_string         = "?";
