@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* All the global variables are defined together in this one header, so that
@@ -37,6 +37,10 @@ extern char  *ldap_default_servers;   /* List of default servers */
 extern char  *mysql_servers;          /* List of servers and connect info */
 #endif
 
+#ifdef LOOKUP_PGSQL
+extern char  *pgsql_servers;          /* List of servers and connect info */
+#endif
+
 #ifdef SUPPORT_MOVE_FROZEN_MESSAGES
 extern BOOL   move_frozen_messages;   /* Get them out of the normal directory */
 #endif
@@ -74,6 +78,7 @@ extern char  *address_pipe;           /* Pipe command when delivering to one */
 extern BOOL   address_test_mode;      /* True for -bt */
 extern gid_t *admin_groups;           /* List of admin groups */
 extern BOOL   admin_user;             /* True if caller can do admin */
+extern BOOL   allow_mx_to_ip;         /* Allow MX records to -> ip address */
 extern BOOL   allow_unqualified_recipient; /* As it says */
 extern BOOL   allow_unqualified_sender; /* Ditto */
 extern BOOL   always_bcc;             /* Use bcc in all adding cases */
@@ -110,6 +115,7 @@ extern int    continue_sequence;      /* Sequence num for continued delivery */
 extern char  *continue_transport;     /* Transport for continued delivery */
 
 extern BOOL   daemon_listen;          /* True if listening required */
+extern int    debug_fd;               /* The fd for debug_file */
 extern FILE  *debug_file;             /* Where to write debugging info */
 extern int    debug_level;            /* Just what its name implies */
 extern pid_t  debug_pid;              /* Pid to add to debug output */
@@ -238,6 +244,9 @@ extern char  *local_domains;          /* List of local domains */
 extern BOOL   local_domains_include_host; /* Always include local host */
 extern BOOL   local_domains_include_host_literals; /* Plus literals */
 extern BOOL   local_error_message;    /* True if handling one of these */
+extern BOOL   local_from_check;       /* For adding Sender: */
+extern char  *local_from_prefix;      /* Permitted prefixes */
+extern char  *local_from_suffix;      /* Permitted suffixes */
 extern char  *local_interfaces;       /* For forcing specific interfaces */
 extern ip_address_item *local_interface_data; /* List of local interface addresses */
 extern char  *local_part_data;        /* From local part lookup */
@@ -271,6 +280,7 @@ extern char  *message_body;           /* Start of message body for filter */
 extern char  *message_body_end;       /* End of message body for filter */
 extern int    message_body_size;      /* Sic */
 extern int    message_body_visible;   /* Amount visible in message_body */
+extern int    message_ended;          /* State of message reading and how ended */
 extern char  *message_filter;         /* Name of system filter file */
 
 /* Special transports for message filters (cf address_xxx, which are done
@@ -424,6 +434,7 @@ extern char  *self_hostname;          /* Self host after routing->directors */
 extern char  *sender_address;         /* Envelope sender */
 extern BOOL   sender_address_forced;  /* Set by -f */
 extern char  *sender_address_relay;   /* Only non-nearby relay from these addresses */
+extern char  *sender_address_relay_hosts; /* ... if from these hosts */
 extern BOOL   sender_address_rewritten; /* True if rewritten by verify */
 extern char  *sender_fullhost;        /* Sender host name + address */
 extern char  *sender_helo_name;       /* Host name from HELO/EHLO */
@@ -469,7 +480,6 @@ extern int    smtp_load_reserve;      /* Only from reserved if load > this */
 extern FILE  *smtp_out;               /* Incoming SMTP output file */
 extern int    smtp_port;              /* Alternate SMTP port number */
 extern char  *smtp_read_error;        /* Message for SMTP input error */
-extern BOOL   smtp_reading_data;      /* True if "." needed before response */
 extern int    smtp_receive_timeout;   /* Applies to each received line */
 extern char  *smtp_reserve_hosts;     /* Hosts for reserved slots */
 extern BOOL   smtp_use_size;          /* Global for passed connections */
@@ -481,7 +491,7 @@ extern int    store_pool;             /* Current pool number */
 extern BOOL   strip_excess_angle_brackets; /* Surrounding route-addrs */
 extern BOOL   strip_trailing_dot;     /* Remove dots at ends of domains */
 
-/* Ditto for system filter */
+extern BOOL   timestamps_utc;         /* Use UTC for all times */
 
 extern transport_instance *transport_message_filter_directory;
 extern transport_instance *transport_message_filter_directory2;
@@ -490,9 +500,9 @@ extern transport_instance *transport_message_filter_pipe;
 extern transport_instance *transport_message_filter_reply;
 
 extern int    transport_count;        /* Count of bytes transported */
-extern char ** transport_filter_argv; /* For on-the-fly filtering */
+extern char **transport_filter_argv;  /* For on-the-fly filtering */
 extern transport_info transports_available[]; /* Vector of available transports */
-extern transport_instance *transports;  /* Chain of instantiated transports */
+extern transport_instance *transports; /* Chain of instantiated transports */
 extern transport_instance transport_defaults; /* Default values */
 
 extern int    transport_write_timeout;/* Set to time out individual writes */
