@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for finding hosts, either by gethostbyname, gethostbyaddr, or via
@@ -15,22 +15,6 @@ concerned with hosts and addresses. */
 
 #include "exim.h"
 
-/* While IPv6 is yet young: the definition of T_AAAA may not be included in
-arpa/nameser.h. Fudge it here. */
-
-#if HAVE_IPV6
-#ifndef T_AAAA
-#define T_AAAA 28
-#endif
-#endif
-
-/* Ancient systems (e.g. SunOS4) don't appear to have T_TXT defined in their
-header files. */
-
-#ifndef T_TXT
-#define T_TXT 16
-#endif
-
 
 
 
@@ -67,35 +51,60 @@ return (unsigned int)(seed >> 16) % limit;
 *************************************************/
 
 /* This function builds a chain of host items from a textual list of host
-names. It does not do any lookups.
+names. It does not do any lookups. If randomize is true, the chain is build in
+a randomized order.
 
 Arguments:
   anchor      anchor for the chain
   list        text list
+  randomize   TRUE for randomizing
 
 Returns:      nothing
 */
 
 void
-host_build_hostlist(host_item **anchor, char *list)
+host_build_hostlist(host_item **anchor, char *list, BOOL randomize)
 {
+int sep = 0;
 char *name;
 char buffer[1024];
 
 if (list == NULL) return;
 
-while ((name = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
+*anchor = NULL;
+
+while ((name = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
   {
   host_item *h = store_get(sizeof(host_item));
-  h->next = NULL;
   h->name = string_copy(name);
   h->address = NULL;
   h->mx = -1;
+  h->sort_key = randomize? random_number(100) : 0;
   h->status = hstatus_unknown;
   h->why = hwhy_unknown;
   h->last_try = 0;
-  *anchor = h;
-  anchor = &(h->next);
+
+  if (*anchor == NULL)
+    {
+    h->next = NULL;
+    *anchor = h;
+    }
+  else
+    {
+    host_item *hh = *anchor;
+    if (h->sort_key < hh->sort_key)
+      {
+      h->next = hh;
+      *anchor = h;
+      }
+    else
+      {
+      while (hh->next != NULL && h->sort_key >= (hh->next)->sort_key)
+        hh = hh->next;
+      h->next = hh->next;
+      hh->next = h;
+      }
+    }
   }
 }
 
@@ -628,6 +637,7 @@ Returns:      a chain of ip_address_items, each pointing to a textual
 ip_address_item *
 host_find_interfaces(void)
 {
+int sep = 0;
 char *s;
 char *listptr = local_interfaces;
 char buffer[64];
@@ -637,7 +647,7 @@ ip_address_item *next;
 
 if (local_interfaces == NULL) return host_find_running_interfaces();
 
-while ((s = string_nextinlist(&listptr, ':', buffer, sizeof(buffer))) != NULL)
+while ((s = string_nextinlist(&listptr, &sep, buffer, sizeof(buffer))) != NULL)
   {
   if (!string_is_ip_address(s, NULL))
     log_write(0, LOG_MAIN|LOG_PANIC_DIE, "Malformed IP address \"%s\" in "
@@ -719,8 +729,15 @@ if (type < 0)
     }
   }
 else
+  {
   yield = (char *)inet_ntop(type, arg, addr_buffer, sizeof(addr_buffer));
-#else
+  }
+
+/* If the result is a mapped IPv4 address, show it in V4 format. */
+
+if (strncmp(yield, "::ffff:", 7) == 0) yield += 7;
+
+#else  /* HAVE_IPV6 */
 
 /* The old world */
 
@@ -1355,6 +1372,7 @@ records.
 
 Arguments:
   host                  points to initial host item
+  allow_ip              if TRUE, recognize an IP address and return it
   fully_qualified_name  if not NULL, return fully qualified name here if
                           the contents are different (i.e. it must be preset
                           to something)
@@ -1365,13 +1383,24 @@ Returns:       HOST_FIND_FAILED     couldn't find A record
 */
 
 static int
-set_address_from_dns(host_item *host, char **fully_qualified_name)
+set_address_from_dns(host_item *host, BOOL allow_ip,
+  char **fully_qualified_name)
 {
 dns_record *rr;
 host_item *last = NULL;
 BOOL first_find_again = FALSE;
 int i, rc, type;
 
+/* If allow_ip is set, a name which is an IP address returns that value
+as its address. This is used for MX records when allow_mx_to_ip is set, for
+those sites that feel they have to flaunt the RFC rules. */
+
+if (allow_ip && string_is_ip_address(host->name, NULL) != 0)
+  {
+  host->address = host->name;
+  return HOST_FOUND;
+  }
+
 /* On an IPv6 system, go round the loop twice, looking for AAAA records
 the first time. */
 
@@ -1537,12 +1566,7 @@ host_find_failed_syntax = FALSE;
 if (!a_only)
   {
   rc = dns_lookup(host->name, T_MX, fully_qualified_name);
-
-/* Changed to be the same as in set_address_from_dns above */
-/*  if (rc == DNS_FAIL) return HOST_FIND_FAILED; */
-
-  if (rc == DNS_FAIL) return HOST_FIND_AGAIN;
-  if (rc == DNS_AGAIN) return HOST_FIND_AGAIN;
+  if (rc == DNS_FAIL || rc == DNS_AGAIN) return HOST_FIND_AGAIN;
   }
 
 /* If there were no MX records and mx_only is FALSE, or if a_only is TRUE,
@@ -1559,7 +1583,7 @@ if (rc != DNS_SUCCEED)
     host_item *next = host->next;
     last = host;
     host->mx = -1;
-    rc = set_address_from_dns(host, fully_qualified_name);
+    rc = set_address_from_dns(host, FALSE, fully_qualified_name);
 
     /* If one or more A records have been found, find the last one and
     check that none of them are local. Since we know the host items all
@@ -1843,7 +1867,7 @@ in case. If any lookup gives a soft error, change the default yield. */
 for (h = host; h != last->next; h = h->next)
   {
   if (h->address != NULL || h->status == hstatus_unusable) continue;
-  rc = set_address_from_dns(h, NULL);
+  rc = set_address_from_dns(h, allow_mx_to_ip, NULL);
   if (rc != HOST_FOUND)
     {
     h->status = hstatus_unusable;
@@ -1894,11 +1918,14 @@ return yield;
 
 /* Anti-spamming "realtime blocking lists" are appearing on the DNS. This
 function, given an IP address and the domain of one such list, looks up the
-address in the blocking domain.
+address in the blocking domain. If a list of IP addresses is given, the
+returned A record is compared to them, and ignored unless one of them matches.
+This allows for handling different categories of entry in some RBL lists.
 
 Arguments:
   address      the string version of the address
   domain       the rbl domain to use
+  iplist       a list of IP addreses to match against, or NULL
   buffer       somewhere to point an error message
 
 Returns:    DNS_SUCCEED   successful lookup (i.e. the address is blocked)
@@ -1908,7 +1935,7 @@ Returns:    DNS_SUCCEED   successful lookup (i.e. the address is blocked)
 */
 
 int
-host_check_rbl(char *address, char *domain, char **bufptr)
+host_check_rbl(char *address, char *domain, char *iplist, char **bufptr)
 {
 int bin[4];
 int rc;
@@ -1960,39 +1987,85 @@ else
   }
 #endif
 
+/* Now do the DNS lookup */
 
 strcpy(t, domain);
 rc = dns_basic_lookup(name, type);
 
-/* Set the buffer pointer to point to the null string, or to a TXT
-record's data if present after success. */
+/* An entry was found in the DNS; check against the IP address list if
+there is one. */
 
 if (rc == DNS_SUCCEED)
   {
+  if (iplist != NULL)
+    {
+    BOOL ok = FALSE;
+    int sep = ',';             /* fixed comma separator */
+    char *ptr = iplist;        /* save original for msg */
+    char ip[46];               /* allow for IPv6 */
+
+    while (!ok &&
+	  (string_nextinlist(&ptr, &sep, ip, sizeof(ip)) != NULL))
+      {
+      dns_record *rr;
+      for (rr = dns_next_rr(RESET_ANSWERS);
+	   rr != NULL && !ok;
+	   rr = dns_next_rr(RESET_NEXT))
+	{
+        #if HAVE_IPV6
+        if (rr->type == T_A || rr->type == T_AAAA)
+        #else
+        if (rr->type == T_A)
+        #endif
+	  ok = strcmp(dns_address_from_rr(rr), ip) == 0;
+	}
+      }
+
+    /* No IP address matched any of those in the list. In that case,
+    behave as if the RBL lookup had not succeeded. */
+
+    if (!ok)
+      {
+      HDEBUG(8)
+	{
+	debug_printf("RBL lookup for %s suceeded\n", name);
+	debug_printf("=> that means it is black listed at %s,\n", domain);
+	debug_printf("=> but we are not accepting this block class because\n");
+        debug_printf("=> there was no match for %s\n", iplist);
+	}
+      return DNS_NOMATCH;
+      }
+    }
+
+  /* Either there was no IP list, or the record matched. Set the buffer
+  pointer to point to the null string, or to a TXT record's data if present
+  after success. */
+
   *bufptr = "";
   if (dns_basic_lookup(name, T_TXT) == DNS_SUCCEED)
     {
     dns_record *rr;
-
-    for (rr = dns_next_rr(RESET_ANSWERS); rr != NULL; rr = dns_next_rr(RESET_NEXT))
+    for (rr = dns_next_rr(RESET_ANSWERS); rr != NULL;
+         rr = dns_next_rr(RESET_NEXT))
       if (rr->type == T_TXT) break;
-
     if (rr != NULL)
       {
       int len = (rr->data)[0];
       if (len > 511) len = 127;
       *bufptr = string_sprintf("%.*s", len, (const char *)(rr->data + 1));
       }
+    }
 
-    HDEBUG(8)
-      {
-      debug_printf("RBL lookup for %s succeeded\n", name);
-      debug_printf("=> that means it is black listed at %s\n", domain);
-      if (**bufptr != 0) debug_printf("%s\n", *bufptr);
-      }
+  HDEBUG(8)
+    {
+    debug_printf("RBL lookup for %s succeeded\n", name);
+    debug_printf("=> that means it is black listed at %s\n", domain);
+    if (**bufptr != 0) debug_printf("%s\n", *bufptr);
     }
   }
 
+/* No entry was found in the DNS */
+
 else HDEBUG(8)
   {
   debug_printf("RBL lookup for %s failed\n", name);
@@ -2029,6 +2102,7 @@ char buffer[256];
 primary_hostname = "";
 debug_level = 9;
 debug_file = stdout;
+debug_fd = fileno(debug_file);
 
 printf("Exim stand-alone host functions test\n");
 
