@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -161,7 +161,8 @@ for (i = 0; i < eo->en_cols.en_cols_len; i++)
   bytes. */
 
   if (value == NULL) value = ""; else
-    while (len > 0 && (value[len-1] == 0 || isspace(value[len-1]))) len--;
+    while (len > 0 && (value[len-1] == 0 || isspace((uschar)value[len-1])))
+      len--;
 
   /* Concatenate all fields if no specific one selected */
 
