@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -82,19 +82,6 @@ but if it is not set, default to EDQUOT if it exists, otherwise ENOSPC. */
 
 #define mac_max(a, b) ((a) > (b) ? (a) : (b))
 
-/* The way of finding out the maximum file descriptor various between OS.
-Most have sysconf(), but a few don't. */
-
-#ifdef _SC_OPEN_MAX
-  #define mac_maxfd (sysconf(_SC_OPEN_MAX) - 1)
-#elif defined OPEN_MAX
-  #define mac_maxfd (OPEN_MAX - 1)
-#elif defined NOFILE
-  #define mac_maxfd (NOFILE - 1)
-#else
-  #define mac_maxfd 255;    /* just in case */
-#endif
-
 /* A macro to simplify testing bits in lookup types */
 
 #define mac_islookup(a,b) ((lookup_list[a].type & (b)) != 0)
@@ -256,6 +243,17 @@ enum {
 
 enum { DNS_SUCCEED, DNS_NOMATCH, DNS_AGAIN, DNS_FAIL };
 
+/* Ending states when reading a message. The order is important. The test
+for having to swallow the rest of an SMTP message is whether the value is
+>= END_NOTENDED. */
+
+#define END_NOTSTARTED 0    /* Message not started */
+#define END_DOT        1    /* Message ended with '.' */
+#define END_EOF        2    /* Message ended with EOF (error for SMTP) */
+#define END_NOTENDED   3    /* Message reading not yet ended */
+#define END_SIZE       4    /* Reading ended because message too big */
+#define END_WERROR     5    /* Write error while reading the message */
+
 /* Private error numbers for delivery failures, set negative so as not
 to conflict with system errno values. */
 
@@ -294,6 +292,8 @@ to conflict with system errno values. */
 #define ERRNO_UNKNOWNHOST    (-33)   /* Lookup failed in smtp transport */
 #define ERRNO_FORMATUNKNOWN  (-34)   /* Can't match format in appendfile */
 #define ERRNO_BADCREATE      (-35)   /* Creation outside home in appendfile */
+#define ERRNO_LISTDEFER      (-36)   /* Can't check a list; lookup defer */
+#define ERRNO_DNSDEFER       (-37)   /* DNS lookup defer */
 
 /* These must be last, so all retry deferments can easily be identified */
 
