@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for parsing addresses */
@@ -1028,6 +1028,7 @@ DEBUG(9) debug_printf("parse_extract_addresses: %s\n", s);
 for (;;)
   {
   int len;
+  int special = 0;
   char *ss, *nexts;
   address_item *addr;
   BOOL inquote = FALSE;
@@ -1095,29 +1096,35 @@ for (;;)
   at all in this expansion is no longer an error. For :defer:, :unknown:, or
   :fail: we have to set up the error message and give up right away. */
 
-  if (allow_specials)
+  if (strncmp(s, ":blackhole:", len) == 0)
     {
-    int special = 0;
-    if (strncmp(s, ":blackhole:", len) == 0)
+    if (!allow_specials)
       {
-      optional = TRUE;
-      s = nexts;
-      continue;
+      *error = ":blackhole: not permitted";
+      return EXTRACTED_ERROR;
       }
+    optional = TRUE;
+    s = nexts;
+    continue;
+    }
 
-    if      (strncmp(s, ":defer:", 7) == 0) special = EXTRACTED_DEFER;
-    else if (strncmp(s, ":unknown:", 9) == 0) special = EXTRACTED_UNKNOWN;
-    else if (strncmp(s, ":fail:", 6) == 0) special = EXTRACTED_FAIL;
+  if      (strncmp(s, ":defer:", 7) == 0) special = EXTRACTED_DEFER;
+  else if (strncmp(s, ":unknown:", 9) == 0) special = EXTRACTED_UNKNOWN;
+  else if (strncmp(s, ":fail:", 6) == 0) special = EXTRACTED_FAIL;
 
-    if (special != 0)
+  if (special != 0)
+    {
+    char *ss = strchr(s+1, ':') + 1;
+    if (!allow_specials)
       {
-      char *ss = strchr(s+1, ':') + 1;
-      while (*ss != 0 && isspace(*ss)) ss++;
-      while (s[len] != 0 && s[len] != '\n') len++;
-      s[len] = 0;
-      *error = string_copy(ss);
-      return special;
+      *error = string_sprintf("\"%.*s\" is not permitted", len, s);
+      return EXTRACTED_ERROR;
       }
+    while (*ss != 0 && isspace((uschar)*ss)) ss++;
+    while (s[len] != 0 && s[len] != '\n') len++;
+    s[len] = 0;
+    *error = string_copy(ss);
+    return special;
     }
 
   /* If the address is of the form :include:pathname, read the file, and call
