@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions that operate on the input queue. */
@@ -12,6 +12,229 @@
 
 
 
+/*************************************************
+*             Get list of spool files            *
+*************************************************/
+
+/* Scan the spool directory and return a list of the relevant file names
+therein. Single-character sub-directories are handled as follows:
+
+  If the first argument is > 0, a sub-directory is scanned; the letter is
+  taken from the nth entry in subdirs.
+
+  If the first argument is 0, sub-directories are not scanned. However, a
+  list of them is returned.
+
+  If the first argument is < 0, sub-directories are scanned for messages,
+  and a single, unified list is created. The returned data blocks contain the
+  identifying character of the subdirectory, if any. The subdirs vector is
+  still required as an argument.
+
+If the randomize argument is TRUE, messages are returned in "randomized" order.
+Actually, the order is anything but random, but the algorithm is cheap, and the
+point is simply to ensure that the same order doesn't occur every time, in case
+a particular message is causing a remote MTA to barf - we would like to try
+other messages to that MTA first. If the argument is FALSE, sort the list
+according to the file name. This should give the order in which the messages
+arrived. It is normally used only for presentation to humans, in which case the
+insertion sort that it does is not part of the normal operational code. This
+case also has the first argument as NULL, so all messages are included.
+
+However, if the queue_run_in order is set, sorting happens, but for multiple
+directories will apply only per-directory.
+
+Arguments:
+  subdiroffset   sub-directory character offset, or 0 or -1 (see above)
+  subdirs        vector to store list of subdirchars
+  subcount       pointer to int in which to store count of subdirs
+  randomize      TRUE if the order of the list is to be unpredictable
+
+Returns:         pointer to a chain of queue name items
+*/
+
+static queue_filename *
+queue_get_spool_list(int subdiroffset, uschar *subdirs, int *subcount,
+  BOOL randomize)
+{
+int i;
+int flags = 0;
+int resetflags = -1;
+int subptr;
+queue_filename *yield = NULL;
+queue_filename *last = NULL;
+struct dirent *ent;
+DIR *dd;
+char buffer[256];
+
+/* The file names are added onto the start or end of the list according to the
+bits of the flags variable. When randomizing, get a collection of bits from the
+current time. Use the bottom 16 and just keep re-using them if necessary. */
+
+if (randomize) resetflags = time(NULL) & 0xFFFF;
+
+/* If processing the full queue, or just the top-level, start at the base
+directory, and initialize the first subdirectory name (as none). Otherwise,
+start at the sub-directory offset. */
+
+if (subdiroffset <= 0)
+  {
+  i = 0;
+  subdirs[0] = 0;
+  *subcount = 0;
+  }
+else i = subdiroffset;
+
+/* Set up prototype for the directory name. */
+
+sprintf(buffer, "%s/input", spool_directory);
+subptr = (int)strlen(buffer);
+buffer[subptr+2] = 0;               /* terminator for lengthened name */
+
+/* This loop runs at least once, for the main or given directory, and then as
+many times as necessary to scan any subdirectories encountered in the main
+directory, if they are to be scanned at this time. */
+
+for (; i <= *subcount; i++)
+  {
+  int count = 0;
+  int subdirchar = subdirs[i];      /* 0 for main directory */
+
+  if (subdirchar != 0)
+    {
+    buffer[subptr] = '/';
+    buffer[subptr+1] = subdirchar;
+    }
+
+  dd = opendir(buffer);
+  if (dd == NULL) continue;
+
+  /* Now scan the directory. */
+
+  while ((ent = readdir(dd)) != NULL)
+    {
+    char *name = ent->d_name;
+    int len = (int)strlen(name);
+
+    /* Count entries */
+
+    count++;
+
+    /* If we find a single alphameric sub-directory in the base directory,
+    add it to the list for subsequent scans. */
+
+    if (i == 0 && len == 1 && isalnum((uschar)*name))
+      {
+      *subcount = *subcount + 1;
+      subdirs[*subcount] = *name;
+      continue;
+      }
+
+    /* Otherwise, if it is a header spool file, add it to the list */
+
+    if (len == SPOOL_NAME_LENGTH &&
+        strcmp(name + SPOOL_NAME_LENGTH - 2, "-H") == 0)
+      {
+      queue_filename *next =
+        store_get(sizeof(queue_filename) + (int)strlen(name));
+      strcpy(next->text, name);
+      next->dir_char = subdirchar;
+
+      /* First item becomes the top and bottom of the list. */
+
+      if (yield == NULL)
+        {
+        next->next = NULL;
+        yield = last = next;
+        }
+
+      /* If randomizing, insert at either top or bottom, randomly. This is, I
+      argue, faster than doing a sort by allocating a random number to each item,
+      and it also saves having to store the number with each item. */
+
+      else if (randomize)
+        {
+        if (flags == 0) flags = resetflags;
+        if ((flags & 1) == 0)
+          {
+          next->next = yield;
+          yield = next;
+          }
+        else
+          {
+          next->next = NULL;
+          last->next = next;
+          last = next;
+          }
+        flags = flags >> 1;
+        }
+
+      /* Otherwise do an insertion sort based on the name. First see if
+      it should go before the first item. */
+
+      else if (strcmp(next->text, yield->text) < 0)
+        {
+        next->next = yield;
+        yield = next;
+        }
+
+      /* Otherwise find the item it should go after; check the last one
+      first, because that will often be the case. */
+
+      else
+        {
+        queue_filename *this;
+        if (strcmp(next->text, last->text) < 0)
+          {
+          for (this = yield; this != last; this = this->next)
+            if (strcmp(next->text, this->next->text) < 0) break;
+          }
+        else this = last;
+        next->next = this->next;
+        this->next = next;
+        if (this == last) last = next;
+        }
+      }
+    }
+
+  /* Finished with this directory */
+
+  closedir(dd);
+
+  /* If we have just scanned a sub-directory, and it was empty (count == 2
+  implies just "." and ".." entries), and Exim is no longer configured to
+  use sub-directories, attempt to get rid of it. At the same time, try to
+  get rid of any corresponding msglog subdirectory. These are just cosmetic
+  tidying actions, so just ignore failures. If we are scanning just a single
+  sub-directory, break the loop. */
+
+  if (i != 0)
+    {
+    if (!split_spool_directory && count <= 2)
+      {
+      rmdir(buffer);
+      sprintf(big_buffer, "%s/msglog/%c", spool_directory, subdirchar);
+      rmdir(big_buffer);
+      }
+    if (subdiroffset > 0) break;    /* Single sub-directory */
+    }
+
+  /* If we have just scanned the base directory, and subdiroffset is 0,
+  we do not want to continue scanning the sub-directories. */
+
+  else
+    {
+    if (subdiroffset == 0) break;
+    }
+  }
+
+/* Pass back the list of file items */
+
+return yield;
+}
+
+
+
+
 /*************************************************
 *              Perform a queue run               *
 *************************************************/
@@ -49,10 +272,12 @@ queue_run(char *start_id, char *stop_id, BOOL recurse)
 {
 BOOL force_delivery = queue_run_force || deliver_selectstring != NULL ||
   deliver_selectstring_sender != NULL;
-queue_filename *f;
 pcre *selectstring_regex = NULL;
 pcre *selectstring_regex_sender = NULL;
 char *log_detail = NULL;
+int subcount = 0;
+int i;
+uschar subdirs[64];
 
 /* Turn off the command line flag that causes remote processing not to happen,
 unless queue_run_local is set, in which case force it. Cancel any specific
@@ -121,19 +346,41 @@ if (deliver_selectstring_sender != NULL && deliver_selectstring_sender_regex)
   selectstring_regex_sender =
     regex_must_compile(deliver_selectstring_sender, TRUE, FALSE);
 
-/* Run the queue. If a start or finish id is given, we must take the queue in
-its natural order. Otherwise "randomize" it so we don't always do things in the
-same order. If deliver_selectstring is set, check for an undelivered address
-or a sender address that matches the string. This does mean we have to read the
-header file, and then again when actually delivering, but it's cheaper than
-forking a delivery process for each file. */
+/* If the spool is split into subdirectories, we want to process it one
+directory at a time, so as to spread out the directory scanning and the
+delivering when there are lots of messages involved, except when
+queue_run_in_order is set.
 
-for (f = queue_get_spool_list(!queue_run_in_order &&
-       start_id == NULL && stop_id == NULL);
-     f != NULL; f = f->next)
+In the random order case, this loop runs once for the main directory (handling
+any messages therein), and then repeats for any subdirectories that were found.
+When the first argument of queue_get_spool_list() is 0, it scans the top
+directory, fills in subdirs, and sets subcount. The order of the directories is
+then randomized after the first time through, before they are scanned in
+subsqeuent iterations.
+
+When the first argument of queue_get_spool_list() is -1 (for queue_run_in_
+order), it scans all directories and makes a single message list. */
+
+for (i  = (queue_run_in_order? -1 : 0);
+     i <= (queue_run_in_order? -1 : subcount);
+     i++)
   {
-  if (stop_id != NULL && strcmp(f->text, stop_id) > 0) break;
-  if (start_id == NULL || strcmp(f->text, start_id) >= 0)
+  queue_filename *f;
+  void *reset_point1 = store_get(0);
+
+  DEBUG(9)
+    {
+    if (i == 0)
+      debug_printf("queue running main directory\n");
+    else if (i == -1)
+      debug_printf("queue running combined directories\n");
+    else
+      debug_printf("queue running subdirectory '%c'\n", subdirs[i]);
+    }
+
+  for (f = queue_get_spool_list(i, subdirs, &subcount, !queue_run_in_order);
+       f != NULL;
+       f = f->next)
     {
     pid_t pid;
     int status;
@@ -141,6 +388,9 @@ for (f = queue_get_spool_list(!queue_run_in_order &&
     struct stat statbuf;
     char buffer[256];
 
+    if (stop_id != NULL && strcmp(f->text, stop_id) > 0) continue;
+    if (start_id != NULL && strcmp(f->text, start_id) < 0) continue;
+
     message_subdir[0] = f->dir_char;
     sprintf(buffer, "%s/input/%s/%s", spool_directory, message_subdir, f->text);
     if (stat(buffer, &statbuf) < 0) continue;
@@ -148,41 +398,42 @@ for (f = queue_get_spool_list(!queue_run_in_order &&
     /* Check for a matching address if deliver_selectstring[_sender} is set.
     If so, we do a fully delivery - don't want to omit other addresses since
     their routing might trigger re-writing etc. Ensure the store used in
-    reading the header file is scavenged. */
+    reading the header file is scavenged. This logic does mean we have to read
+    the header file, and then again when actually delivering, but it's cheaper
+    than forking a delivery process for each message even when they aren't all
+    going to be delivered. */
 
     if (deliver_selectstring != NULL || deliver_selectstring_sender != NULL)
       {
-      void *reset_point = store_get(0);
+      BOOL wanted = TRUE;
+      void *reset_point2 = store_get(0);
+
       if (spool_read_header(f->text, FALSE, TRUE) != spool_read_OK) continue;
 
       if (deliver_freeze && !deliver_force_thaw)
         {
         log_write(5, LOG_MAIN, "Message is frozen");
-        store_reset(reset_point);
-        continue;  /* With next message */
+        wanted = FALSE;
         }
 
       /* Sender matching */
 
-      if (deliver_selectstring_sender != NULL)
+      else if (deliver_selectstring_sender != NULL &&
+              !(deliver_selectstring_sender_regex?
+                (pcre_exec(selectstring_regex_sender, NULL, sender_address,
+                  (int)strlen(sender_address), 0, PCRE_EOPT, NULL, 0) >= 0)
+                :
+                (strstric(sender_address, deliver_selectstring_sender, FALSE)
+                  != NULL)))
         {
-        if (!(deliver_selectstring_sender_regex?
-              (pcre_exec(selectstring_regex_sender, NULL, sender_address,
-                (int)strlen(sender_address), 0, PCRE_EOPT, NULL, 0) >= 0)
-              :
-              (strstric(sender_address, deliver_selectstring_sender, FALSE) !=
-                NULL)))
-          {
-          DEBUG(9) debug_printf("%s: sender address did not match %s\n",
-            f->text, deliver_selectstring_sender);
-          store_reset(reset_point);
-          continue;     /* With next message */
-          }
+        DEBUG(9) debug_printf("%s: sender address did not match %s\n",
+          f->text, deliver_selectstring_sender);
+        wanted = FALSE;
         }
 
       /* Recipient matching */
 
-      if (deliver_selectstring != NULL)
+      else if (deliver_selectstring != NULL)
         {
         int i;
         for (i = 0; i < recipients_count; i++)
@@ -202,14 +453,12 @@ for (f = queue_get_spool_list(!queue_run_in_order &&
           {
           DEBUG(9) debug_printf("%s: no recipient address matched %s\n",
             f->text, deliver_selectstring);
-          store_reset(reset_point);
-          continue;      /* With next message */
+          wanted = FALSE;
           }
         }
 
-      /* Message matches -R and -S (whichever are set) */
-
-      store_reset(reset_point);
+      store_reset(reset_point2);
+      if (!wanted) continue;      /* With next message */
       }
 
     /* OK, got a message we want to deliver. Create a pipe which will
@@ -287,8 +536,29 @@ for (f = queue_get_spool_list(!queue_run_in_order &&
     (void)read(pfd[pipe_read], buffer, sizeof(buffer));
     (void)close(pfd[pipe_read]);
     set_process_info("running queue");
+    }                                  /* End loop for list of messages */
+
+  store_reset(reset_point1);           /* Scavenge list of messages */
+
+  /* If this was the first time through for random order processing, and
+  sub-directories have been found, randomize their order if necessary. */
+
+  if (i == 0 && subcount > 1 && !queue_run_in_order)
+    {
+    int j;
+    for (j = 1; j <= subcount; j++)
+      {
+      int r = random_number(100);
+      if (r >= 50)
+        {
+        int k = (r % subcount) + 1;
+        int x = subdirs[j];
+        subdirs[j] = subdirs[k];
+        subdirs[k] = x;
+        }
+      }
     }
-  }
+  }                                    /* End loop for multiple directories */
 
 /* If queue_2stage is true, we do it all again, with the 2stage flag
 turned off. */
@@ -312,180 +582,6 @@ if (!recurse)
 
 
 
-/*************************************************
-*             Get list of spool files            *
-*************************************************/
-
-/* Scan the spool directory and return a list of the relevant file names
-therein. If the argument is TRUE, they are returned in "randomized" order.
-Actually, the order is anything but random, but the algorithm is cheap, and
-the point is simply to ensure that the same order doesn't occur every time, in
-case a particular message is causing a remote MTA to barf - we would like to
-try other messages to that MTA first. If the argument is FALSE, sort the list
-according to the file name. This should give the order in which the messages
-arrived. It is normally used only for presentation to humans, in which case the
-insertion sort that it does is not part of the normal operational code.
-However, if the queue_run_in order is set, or if a -q option specifies starting
-and stopping messages, the sort is required.
-
-If any single-character sub-directories are encountered in the initial
-scan, they are subsequently scanned for message files too. The returned
-data blocks contain the identifying character of the subdirectory, if any.
-
-Argument:   TRUE if the order of the list is to be unpredictable
-Returns:    pointer to a chain of queue name items
-*/
-
-queue_filename *
-queue_get_spool_list(BOOL randomize)
-{
-int i;
-int subdir_max = 1;
-int flags = 0;
-int resetflags = -1;
-int subptr;
-queue_filename *yield = NULL;
-queue_filename *last = NULL;
-struct dirent *ent;
-DIR *dd;
-char subdirs[64];
-char buffer[256];
-
-/* The file names are added onto the start or end of the list according to the
-bits of the flags variable. When randomizing, get a collection of bits from the
-current time. Use the bottom 16 and just keep re-using them if necessary. */
-
-if (randomize) resetflags = time(NULL) & 0xFFFF;
-
-/* This loop runs at least once, for the main directory, and then as many
-times as necessary to scan any subdirectories encountered in the main
-directory. */
-
-subdirs[0] = 0;
-sprintf(buffer, "%s/input", spool_directory);
-subptr = (int)strlen(buffer);
-buffer[subptr+2] = 0;               /* terminator for lengthened name */
-
-for (i = 0; i < subdir_max; i++)
-  {
-  int count = 0;
-  int subdirchar = subdirs[i];      /* 0 for main directory */
-  if (subdirchar != 0)
-    {
-    buffer[subptr] = '/';
-    buffer[subptr+1] = subdirchar;
-    }
-  dd = opendir(buffer);
-  if (dd == NULL) continue;
-
-  /* Now scan the directory. */
-
-  while ((ent = readdir(dd)) != NULL)
-    {
-    char *name = ent->d_name;
-    int len = (int)strlen(name);
-
-    /* Count entries */
-
-    count++;
-
-    /* If we find a single alphameric sub-directory on the first
-    pass, add it to the list for subsequent scans. */
-
-    if (i == 0 && len == 1 && isalnum(*name))
-      {
-      subdirs[subdir_max++] = *name;
-      continue;
-      }
-
-    /* Otherwise, if it is a header spool file, add it to the list */
-
-    if (len == SPOOL_NAME_LENGTH &&
-        strcmp(name + SPOOL_NAME_LENGTH - 2, "-H") == 0)
-      {
-      queue_filename *next =
-        store_get(sizeof(queue_filename) + (int)strlen(name));
-      strcpy(next->text, name);
-      next->dir_char = subdirchar;
-
-      /* First item becomes the top and bottom of the list. */
-
-      if (yield == NULL)
-        {
-        next->next = NULL;
-        yield = last = next;
-        }
-
-      /* If randomizing, insert at either top or bottom, randomly. This is, I
-      argue, faster than doing a sort by allocating a random number to each item,
-      and it also saves having to store the number with each item. */
-
-      else if (randomize)
-        {
-        if (flags == 0) flags = resetflags;
-        if ((flags & 1) == 0)
-          {
-          next->next = yield;
-          yield = next;
-          }
-        else
-          {
-          next->next = NULL;
-          last->next = next;
-          last = next;
-          }
-        flags = flags >> 1;
-        }
-
-      /* Otherwise do an insertion sort based on the name. First see if
-      it should go before the first item. */
-
-      else if (strcmp(next->text, yield->text) < 0)
-        {
-        next->next = yield;
-        yield = next;
-        }
-
-      /* Otherwise find the item it should go after; check the last one
-      first, because that will often be the case. */
-
-      else
-        {
-        queue_filename *this;
-        if (strcmp(next->text, last->text) < 0)
-          {
-          for (this = yield; this != last; this = this->next)
-            if (strcmp(next->text, this->next->text) < 0) break;
-          }
-        else this = last;
-        next->next = this->next;
-        this->next = next;
-        if (this == last) last = next;
-        }
-      }
-    }
-
-  /* Close the directory and, it was empty (count == 2 implies just "." and
-  ".." entries) and it was a subdirectory, and Exim is no longer configured
-  to use subdirectories, attempt to get rid of it. At the same time, try
-  to get rid of any corresponding msglog subdirectory. These are just
-  cosmetic tidying actions, so just ignore failures. */
-
-  closedir(dd);
-  if (i != 0 && !split_spool_directory && count <= 2)
-    {
-    rmdir(buffer);
-    sprintf(big_buffer, "%s/msglog/%c", spool_directory, subdirchar);
-    rmdir(big_buffer);
-    }
-  }
-
-return yield;
-}
-
-
-
-
 /************************************************
 *          List extra deliveries                *
 ************************************************/
@@ -536,9 +632,11 @@ void
 queue_list(int option, char **list, int count)
 {
 int i;
+int subcount;
 int now = (int)time(NULL);
 void *reset_point;
 queue_filename *f = NULL;
+uschar subdirs[64];
 
 /* If given a list of messages, build a chain containing their ids. */
 
@@ -559,7 +657,12 @@ if (count > 0)
 
 /* Otherwise get a list of the entire queue, in order if necessary. */
 
-else f = queue_get_spool_list(option >= 8);
+else
+  f = queue_get_spool_list(
+          -1,             /* entire queue */
+          subdirs,        /* for holding sub list */
+          &subcount,      /* for subcount */
+          option >= 8);   /* randomize if required */
 
 if (option >= 8) option -= 8;
 
@@ -765,6 +868,7 @@ if (action >= MSG_SHOW_BODY)
   while((rc = read(fd, big_buffer, big_buffer_size)) > 0)
     write(fileno(stdout), big_buffer, rc);
 
+  close(fd);
   return TRUE;
   }
 
@@ -1181,6 +1285,7 @@ void
 queue_check_only(void)
 {
 BOOL *set;
+int sep = 0;
 struct stat statbuf;
 char *s, *ss, *name;
 char buffer[1024];
@@ -1188,7 +1293,7 @@ char buffer[1024];
 if (queue_only_file == NULL) return;
 
 s = queue_only_file;
-while ((ss = string_nextinlist(&s, ':', buffer, sizeof(buffer))) != NULL)
+while ((ss = string_nextinlist(&s, &sep, buffer, sizeof(buffer))) != NULL)
   {
   if (strncmp(ss, "remote", 6) == 0)
     {
