@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for reading the configuration file, and for displaying
@@ -35,6 +35,7 @@ static optionlist optionlist_config[] = {
   { "accept_8bitmime",          opt_bool,        &accept_8bitmime },
   { "accept_timeout",           opt_time,        &accept_timeout },
   { "admin_groups",             opt_gidlist,     &admin_groups },
+  { "allow_mx_to_ip",           opt_bool,        &allow_mx_to_ip },
   { "always_bcc",               opt_bool,        &always_bcc },
 #ifdef HAVE_AUTH
   { "auth_always_advertise",    opt_bool,        &auth_always_advertise },
@@ -106,6 +107,9 @@ static optionlist optionlist_config[] = {
   { "local_domains",            opt_stringptr,   &local_domains },
   { "local_domains_include_host", opt_bool,      &local_domains_include_host },
   { "local_domains_include_host_literals", opt_bool, &local_domains_include_host_literals },
+  { "local_from_check",         opt_bool,        &local_from_check },
+  { "local_from_prefix",        opt_stringptr,   &local_from_prefix },
+  { "local_from_suffix",        opt_stringptr,   &local_from_suffix },
   { "local_interfaces",         opt_stringptr,   &local_interfaces },
   { "localhost_number",         opt_stringptr,   &host_number_string },
   { "locally_caseless",         opt_bool,        &locally_caseless },
@@ -150,6 +154,9 @@ static optionlist optionlist_config[] = {
 #ifdef EXIM_PERL
   { "perl_at_start",            opt_bool,        &opt_perl_at_start },
   { "perl_startup",             opt_stringptr,   &opt_perl_startup },
+#endif
+#ifdef LOOKUP_PGSQL
+  { "pgsql_servers",            opt_stringptr,   &pgsql_servers },
 #endif
   { "pid_file_path",            opt_stringptr,   &pid_file_path },
   { "preserve_message_logs",    opt_bool,        &preserve_message_logs },
@@ -199,6 +206,7 @@ static optionlist optionlist_config[] = {
   { "rfc1413_query_timeout",    opt_time,        &rfc1413_query_timeout },
   { "security",                 opt_stringptr,   &security_type },
   { "sender_address_relay",     opt_stringptr,   &sender_address_relay },
+  { "sender_address_relay_hosts", opt_stringptr, &sender_address_relay_hosts },
   { "sender_reject",            opt_stringptr,   &sender_reject },
   { "sender_reject_recipients", opt_stringptr,   &sender_reject_recipients },
   { "sender_try_verify",        opt_bool,        &sender_try_verify },
@@ -230,6 +238,7 @@ static optionlist optionlist_config[] = {
   { "spool_directory",          opt_stringptr,   &spool_directory },
   { "strip_excess_angle_brackets", opt_bool,     &strip_excess_angle_brackets },
   { "strip_trailing_dot",       opt_bool,        &strip_trailing_dot },
+  { "timestamps_utc",           opt_bool,        &timestamps_utc },
   { "trusted_groups",           opt_gidlist,     &trusted_groups },
   { "trusted_users",            opt_uidlist,     &trusted_users },
   { "unknown_login",            opt_stringptr,   &unknown_login },
@@ -276,127 +285,119 @@ return "";
 *            Read configuration line             *
 *************************************************/
 
-/* A line of text is read from the configuration file into the big
-buffer. Leading and trailing spaces are removed and the line is scanned
-for macros, which are replaced. The size of big_buffer is increased if
-necessary. The count of configuration lines is maintained. Comment lines are
-always ignored. Continuation lines are handled if that is wanted.
+/* A logical line of text is read from the configuration file into the big
+buffer, taking account of continuations. The size of big_buffer is increased if
+necessary. The count of configuration lines is maintained. Physical input lines
+starting with # (ignoring leading white space) and empty logical lines are
+always ignored. Leading and trailing spaces are removed and the line is scanned
+for macros, which are replaced.
 
-Arguments:
-  ignore_blank      if TRUE, blank lines are ignored
-  get_continued     if TRUE and the line ends with '\', join the next one
-                    onto it
-  maybe_macro       if TRUE and the line starts with an upper case letter and
-                    ends with '\', join the next one onto it
-  continued_string  if TRUE, this is a string continuation line, which cannot
-                    start with a macro definition
+Arguments:      none
 
-Returns:            a pointer to the first non-blank in the line,
-                    or NULL if eof is reached
+Returns:        a pointer to the first non-blank in the line,
+                or NULL if eof is reached
 */
 
 static char *
-get_config_line(BOOL ignore_blank, BOOL get_continued, BOOL maybe_macro,
-  BOOL continued_string)
+get_config_line(void)
 {
-int n;
+int len = 0;
+int startoffset = 0;
+int newlen;
 char *s, *ss;
 macro_item *m;
 
-/* Loop for skipping blanks lines and comments if required */
+/* Loop for handling continuation lines and skipping comments. */
 
 for (;;)
   {
-  if (fgets(big_buffer, big_buffer_size, config_file) == NULL) return NULL;
-  config_lineno++;
-  n = (int)strlen(big_buffer);
-
-  /* Handle pathologically long lines - yes, it did happen - by extending
-  big_buffer. */
-
-  while (n == big_buffer_size - 1 && big_buffer[n-1] != '\n')
+  if (fgets(big_buffer+len, big_buffer_size-len, config_file) == NULL)
     {
-    char *newbuffer = store_malloc(big_buffer_size += BIG_BUFFER_SIZE);
+    if (len == 0) return NULL;  /* EOF at start of logical line */
+    break;                      /* EOF after continuation */
+    }
+
+  config_lineno++;
+  newlen = len + (int)strlen(big_buffer + len);
+
+  /* Handle pathologically long physical lines - yes, it did happen - by
+  extending big_buffer at this point. The code also copes with very long
+  logical lines. */
+
+  while (newlen == big_buffer_size - 1 && big_buffer[newlen - 1] != '\n')
+    {
+    char *newbuffer;
+    big_buffer_size += BIG_BUFFER_SIZE;
+    newbuffer = store_malloc(big_buffer_size);
     strcpy(newbuffer, big_buffer);
-    n = (int)strlen(newbuffer);
     store_free(big_buffer);
     big_buffer = newbuffer;
-    if (fgets(big_buffer+n, big_buffer_size-n, config_file) == NULL) break;
-    n = (int)strlen(big_buffer);
+    if (fgets(big_buffer+newlen, big_buffer_size-newlen, config_file) == NULL)
+      break;
+    newlen += (int)strlen(big_buffer + newlen);
     }
 
-  /* Remove trailing whitespace and find genuine start of line */
+  /* Find the true start of the physical line - leading spaces are always
+  ignored. */
 
-  while (n > 0 && isspace(big_buffer[n-1])) n--;
-  big_buffer[n] = 0;
-  s = big_buffer;
-  while (isspace(*s)) s++;
-  if (*s != '#' && (*s != 0 || !ignore_blank)) break;
-  }
+  s = ss = big_buffer + len;
+  while (isspace((uschar)*ss)) ss++;
 
-/* Handle continuation lines if requested to do so - this isn't used for
-strings, which handle continuation themselves. Some of this repeats code above,
-but I can't see how to pack them up tidily, and we do want this to be
-efficient, since Exim is forever reading its configuration file. */
+  /* Check for comment lines - these are physical lines. */
 
-if (get_continued || (maybe_macro && isupper(*s)))
-  {
-  while (big_buffer[n-1] == '\\')
+  if (*ss == '#') continue;
+
+  /* If this is the start of the logical line, remember where the non-blank
+  data starts. Otherwise shuffle down continuation lines to remove leading
+  white space */
+
+  if (len == 0)
+    startoffset = ss - big_buffer;
+  else
     {
-    int c = fgetc(config_file);
-    while (c == ' ' || c == '\t') c = fgetc(config_file);
-    if (c == '\n' || c == EOF) { config_lineno++; break; }
-    big_buffer[n-1] = c;
-
-    if (big_buffer_size - n < 256)
+    if (ss != s)
       {
-      char *newbuffer = store_malloc(big_buffer_size += BIG_BUFFER_SIZE);
-      strcpy(newbuffer, s);
-      s = newbuffer;
-      n = (int)strlen(newbuffer);
-      store_free(big_buffer);
-      big_buffer = newbuffer;
+      memmove(s, ss, big_buffer + newlen - ss);
+      newlen -= ss - s;
       }
-
-    if (fgets(big_buffer+n, big_buffer_size-n, config_file) == NULL) break;
-    config_lineno++;
-    n = (int)strlen(big_buffer);
-
-    while (n == big_buffer_size - 1 && big_buffer[n-1] != '\n')
-      {
-      char *newbuffer = store_malloc(big_buffer_size += BIG_BUFFER_SIZE);
-      strcpy(newbuffer, big_buffer);
-      n = (int)strlen(newbuffer);
-      store_free(big_buffer);
-      s = big_buffer = newbuffer;
-      if (fgets(big_buffer+n, big_buffer_size-n, config_file) == NULL) break;
-      n = (int)strlen(big_buffer);
-      }
-
-    if (c == '#') continue;
-    while (n > 0 && isspace(big_buffer[n-1])) n--;
-    big_buffer[n] = 0;
     }
-  }
+
+  /* Accept the new addition to the line. Remove trailing white space. */
+
+  len = newlen;
+  while (len > 0 && isspace((uschar)big_buffer[len-1])) len--;
+  big_buffer[len] = 0;
+
+  /* We are done if the line does not end in backslash and contains some data.
+  Empty logical lines are ignored. For continuations, remove the backslash and
+  go round the loop to read the continuation line. */
+
+  if (len > 0)
+    {
+    if (big_buffer[len-1] != '\\') break;
+    big_buffer[--len] = 0;
+    }
+  }     /* Loop for reading physical lines */
 
 /* Handle macro expansion. This is done very simply. However, we must take care
-not to expand a macro name at the start of a definition, but we must do so at
-the start of a continuation line inside a string. */
+not to expand a macro name at the start of a definition. */
+
+s = big_buffer + startoffset;
+ss = big_buffer + len;
 
-ss = big_buffer + n;
 for (m = macros; m != NULL; m = m->next)
   {
   char *p, *pp;
   char *t = s;
 
-  /* If not a continued string, skip over any line that starts with a sequence
-  of upper case characters followed by an equals sign, because that is the
+  /* Skip over any line that starts with an upper case character followed by
+  a sequence of name characters and an equals sign, because that is the
   definition of a new macro. */
 
-  if (!continued_string && isupper(*t))
+  if (isupper((uschar)*t))
     {
-    while (isalnum(*t) || *t == '_') t++;
-    while (isspace(*t)) t++;
+    while (isalnum((uschar)*t) || *t == '_') t++;
+    while (isspace((uschar)*t)) t++;
     if (*t != '=') t = s;
     }
 
@@ -464,17 +465,17 @@ static char *
 readconf_readname(char *name, int len, char *s)
 {
 int p = 0;
-while (isspace(*s)) s++;
-if (isalpha(*s))
+while (isspace((uschar)*s)) s++;
+if (isalpha((uschar)*s))
   {
-  while (isalnum(*s) || *s == '_')
+  while (isalnum((uschar)*s) || *s == '_')
     {
     if (p < len-1) name[p++] = *s;
     s++;
     }
   }
 name[p] = 0;
-while (isspace(*s)) s++;
+while (isspace((uschar)*s)) s++;
 return s;
 }
 
@@ -508,7 +509,7 @@ int yield = 0;
 for (;;)
   {
   int value, count;
-  if (!isdigit(*s)) return -1;
+  if (!isdigit((uschar)*s)) return -1;
   (void)sscanf(s, "%d%n", &value, &count);
   s += count;
   switch (*s)
@@ -548,14 +549,14 @@ readconf_readfixed(char *s, int terminator)
 {
 int yield = 0;
 int value, count;
-if (!isdigit(*s)) return -1;
+if (!isdigit((uschar)*s)) return -1;
 (void)sscanf(s, "%d%n", &value, &count);
 s += count;
 yield = value * 1000;
 if (*s == '.')
   {
   int m = 100;
-  while (isdigit(*(++s)))
+  while (isdigit((uschar)(*(++s))))
     {
     yield += (*s - '0') * m;
     m /= 10;
@@ -700,17 +701,17 @@ char name2[64];
 
 /* There may be leading spaces */
 
-while (isspace(*s)) s++;
+while (isspace((uschar)*s)) s++;
 
 /* Read the name of the option, and skip any subsequent white space. */
 
-while (isalnum(*s) || *s == '_')
+while (isalnum((uschar)*s) || *s == '_')
   {
   if (ptr < sizeof(name)-1) name[ptr++] = *s;
   s++;
   }
 name[ptr] = 0;
-while (isspace(*s)) s++;
+while (isspace((uschar)*s)) s++;
 
 /* Deal with "no_" or "not_" here for booleans */
 
@@ -788,7 +789,7 @@ if (data_block != NULL && (ol->type & opt_public) == 0)
 
 /* Skip white space after = */
 
-while (isspace(*(++s)));
+while (isspace((uschar)(*(++s))));
 
 /* Now get the data according to the type. */
 
@@ -833,8 +834,13 @@ switch (type)
     {
     int count = 0;
 
-    while (*(++s) != 0)
+    for(;;)
       {
+      if (*(++s) == 0)
+        log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
+          "missing quote at end of string value for %s in line %d",
+          name, config_lineno);
+
       if (*s == '\"')
         {
         if (*(++s) == 0) break;
@@ -843,25 +849,13 @@ switch (type)
           "for %s in line %d", name, config_lineno);
         }
 
-      /* Still in the string. Handle escape characters. */
+      /* Still in the string. Handle escape characters. If nothing follows
+      the \ we do nothing. The main loop then discovers an unexpected eol
+      and fails. */
 
       else if (*s == '\\')
         {
-        if (s[1] == 0)
-          {
-          char *ss = get_config_line(FALSE, FALSE, FALSE, TRUE);
-
-          /* If we hit eof, nothing is done here, the main loop then
-          retries and finds *(++s) == 0 and so exits. Otherwise, point
-          s at the char before the start of the new line. Note that we
-          have skipped comments. This is a feature, not a bug! */
-
-          if (ss != NULL) s = ss - 1;
-          }
-
-        /* Handle '\' not at end of line */
-
-        else
+        if (s[1] != 0)
           {
           char ch = string_interpret_escape(&s);
           sptr = string_cat(sptr, &n, &count, &ch, 1);
@@ -1067,7 +1061,8 @@ switch (type)
       p = sptr;
       while (count-- > 1)
         {
-        (void)string_nextinlist(&p, ':', big_buffer, BIG_BUFFER_SIZE);
+        int sep = 0;
+        (void)string_nextinlist(&p, &sep, big_buffer, BIG_BUFFER_SIZE);
         if (!direct_finduser(big_buffer, NULL, &uid))
           log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
             "user %s, referred to in line %d, "
@@ -1100,7 +1095,8 @@ switch (type)
       p = sptr;
       while (count-- > 1)
         {
-        (void)string_nextinlist(&p, ':', big_buffer, BIG_BUFFER_SIZE);
+        int sep = 0;
+        (void)string_nextinlist(&p, &sep, big_buffer, BIG_BUFFER_SIZE);
         if (!direct_findgroup(big_buffer, &gid))
           log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
             "group %s, referred to in line %d, "
@@ -1127,10 +1123,10 @@ switch (type)
       if (strncmp(sp, "partial", 7) == 0)
         {
         sp += 7;
-        if (isdigit (*sp))
+        if (isdigit ((uschar)*sp))
           {
           pv = 0;
-          while (isdigit(*sp)) pv = pv*10 + *sp++ - '0';
+          while (isdigit((uschar)*sp)) pv = pv*10 + *sp++ - '0';
           }
         else pv = 2;
 
@@ -1296,7 +1292,7 @@ switch (type)
   if (tolower(s[count]) == 'k') { value *= 1024; count++; }
   if (tolower(s[count]) == 'm') { value *= 1024*1024; count++; }
 
-  while (isspace(s[count])) count++;
+  while (isspace((uschar)s[count])) count++;
 
   if (s[count] != 0)
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
@@ -1320,14 +1316,14 @@ switch (type)
   if (s[count] == '.')
     {
     int d = 100;
-    while (isdigit(s[++count]))
+    while (isdigit((uschar)s[++count]))
       {
       value += (s[count] - '0') * d;
       d /= 10;
       }
     }
 
-  while (isspace(s[count])) count++;
+  while (isspace((uschar)s[count])) count++;
 
   if (s[count] != 0)
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
@@ -1371,7 +1367,7 @@ switch (type)
       if (snext != NULL)
         {
         char *ss = snext;
-        while (ss > s && isspace(ss[-1])) ss--;
+        while (ss > s && isspace((uschar)ss[-1])) ss--;
         terminator = *ss;
         }
       value = readconf_readtime(s, terminator);
@@ -1384,7 +1380,7 @@ switch (type)
       list[count+1] = value;
       if (snext == NULL) break;
       s = snext + 1;
-      while (isspace(*s)) s++;
+      while (isspace((uschar)*s)) s++;
       }
 
     if (count > list[0] - 2)
@@ -2017,27 +2013,26 @@ if (!config_changed)
 letter. If we see something starting with an upper case letter, it is taken as
 a macro definition. */
 
-while ((s = get_config_line(TRUE, FALSE, TRUE, FALSE)) != NULL &&
-        strcmpic(s, "end") != 0)
+while ((s = get_config_line()) != NULL && strcmpic(s, "end") != 0)
   {
-  if (isupper(s[0]))
+  if (isupper((uschar)s[0]))
     {
     macro_item *m;
     macro_item *mlast = NULL;
     char name[24];
     int namelen = 0;
 
-    while (isalnum(*s) || *s == '_')
+    while (isalnum((uschar)*s) || *s == '_')
       {
       if (namelen < sizeof(name)-1) name[namelen++] = *s;
       s++;
       }
     name[namelen] = 0;
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (*s++ != '=')
       log_write(0, LOG_CONFIG|LOG_PANIC_DIE, "malformed macro definition in "
         "line %d", config_lineno);
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
 
     /* If an existing macro of the same name was defined on the command line,
     we just skip this definition. Otherwise it's an error to attempt to
@@ -2131,13 +2126,14 @@ the null string or "syslog". */
 if (*log_file_path != 0)
   {
   char *ss, *sss;
+  int sep = ':';                       /* Fixed for log file path */
   s = expand_string(log_file_path);
   if (s == NULL)
     log_write(0, LOG_PANIC_DIE, "failed to expand log_file_path=\"%s\"",
       log_file_path);
 
   ss = s;
-  while ((sss = string_nextinlist(&ss,':',big_buffer,big_buffer_size)) != NULL)
+  while ((sss = string_nextinlist(&ss,&sep,big_buffer,big_buffer_size)) != NULL)
     {
     if (sss[0] == 0 || strcmp(sss, "syslog") == 0) continue;
     if (strstr(sss, "%s") == NULL)
@@ -2172,7 +2168,7 @@ port number from it. */
 
 if (daemon_listen && smtp_port < 0 && configured_smtp_service != NULL)
   {
-  if (isdigit(*configured_smtp_service))
+  if (isdigit((uschar)*configured_smtp_service))
     {
     char *end;
     smtp_port = strtol(configured_smtp_service, &end, 0);
@@ -2338,6 +2334,7 @@ if (exim_uid_set)
   if (strcmp(security_type, "setuid") == 0) security_level = 2;
   else if (strcmp(security_type, "seteuid") == 0) security_level = 1;
   else if (strcmp(security_type, "setuid+seteuid") == 0) security_level = 3;
+  else if (strcmp(security_type, "unprivileged") == 0) security_level = 4;
   else log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
     "Unknown security setting: %s", security_type);
 
@@ -2490,8 +2487,7 @@ char *buffer;
 
 /* Now process the configuration lines */
 
-while ((buffer = get_config_line(TRUE, FALSE, FALSE, FALSE)) != NULL &&
-        strcmpic(buffer, "end") != 0)
+while ((buffer = get_config_line()) != NULL && strcmpic(buffer, "end") != 0)
   {
   char name[64];
 
@@ -2535,7 +2531,7 @@ while ((buffer = get_config_line(TRUE, FALSE, FALSE, FALSE)) != NULL &&
 
     /* If nothing more on this line, do the next loop iteration. */
 
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (*s != 0)
       log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
         "extraneous characters after driver name \"%s\" in line %d",
@@ -2623,7 +2619,7 @@ for (ol = d->info->options; ol < d->info->options + count; ol++)
   if (value != NULL && (ss = strstr(value, s)) != NULL)
     {
     if (ss <= value || (ss[-1] != '$' && ss[-1] != '{') ||
-      isalnum(ss[(int)strlen(s)])) continue;
+      isalnum((uschar)ss[(int)strlen(s)])) continue;
     DEBUG(9) debug_printf("driver %s option %s depends on %s\n",
       d->name, ol->name, s);
     return TRUE;
@@ -2736,11 +2732,11 @@ if (*p++ != ',')
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
     "comma expected in line %d", config_lineno);
 
-while (isspace(*p)) p++;
+while (isspace((uschar)*p)) p++;
 pp = p;
-while (isalnum(*p)|| (type == 1 && *p == '.')) p++;
+while (isalnum((uschar)*p)|| (type == 1 && *p == '.')) p++;
 
-if (*p != 0 && !isspace(*p) && *p != ',' && *p != ';')
+if (*p != 0 && !isspace((uschar)*p) && *p != ',' && *p != ';')
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
     "comma or semicolon expected in line %d", config_lineno);
 
@@ -2764,8 +2760,7 @@ retry_config **chain = &retries;
 retry_config *next;
 char *p;
 
-while ((p = get_config_line(TRUE, FALSE, FALSE, FALSE)) != NULL &&
-        strcmpic(p, "end") != 0)
+while ((p = get_config_line()) != NULL && strcmpic(p, "end") != 0)
   {
   retry_rule **rchain;
   char *pp, *error;
@@ -2797,7 +2792,7 @@ while ((p = get_config_line(TRUE, FALSE, FALSE, FALSE)) != NULL &&
       }
     }
 
-  while (isspace(*p)) p++;
+  while (isspace((uschar)*p)) p++;
   pp = p;
   while (mac_isgraph(*p)) p++;
   if (p - pp <= 0)
@@ -2813,7 +2808,7 @@ while ((p = get_config_line(TRUE, FALSE, FALSE, FALSE)) != NULL &&
 
   /* Now the retry rules. Keep the maximum timeout encountered. */
 
-  while (isspace(*p)) p++;
+  while (isspace((uschar)*p)) p++;
   while (*p != 0)
     {
     retry_rule *rule = store_get(sizeof(retry_rule));
@@ -2847,11 +2842,11 @@ while ((p = get_config_line(TRUE, FALSE, FALSE, FALSE)) != NULL &&
       break;
       }
 
-    while (isspace(*p)) p++;
+    while (isspace((uschar)*p)) p++;
     if (*p == ';')
       {
       p++;
-      while (isspace(*p)) p++;
+      while (isspace((uschar)*p)) p++;
       }
     else if (*p != 0)
       log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
@@ -2886,8 +2881,7 @@ rewrite_rule **chain = &rewrite_rules;
 rewrite_rule *next;
 char *p, *pp;
 
-while ((p = get_config_line(TRUE, TRUE, FALSE, FALSE)) != NULL &&
-        strcmpic(p, "end") != 0)
+while ((p = get_config_line()) != NULL && strcmpic(p, "end") != 0)
   {
   next = store_get(sizeof(rewrite_rule));
   next->next = NULL;
@@ -2895,7 +2889,7 @@ while ((p = get_config_line(TRUE, TRUE, FALSE, FALSE)) != NULL &&
   chain = &(next->next);
 
   pp = p;
-  while (*p != 0 && !isspace(*p)) p++;
+  while (*p != 0 && !isspace((uschar)*p)) p++;
 
   next->key = string_copyn(pp, p-pp);
 
@@ -2911,7 +2905,7 @@ while ((p = get_config_line(TRUE, TRUE, FALSE, FALSE)) != NULL &&
           next->key, config_lineno);
     }
 
-  while (isspace(*p)) p++;
+  while (isspace((uschar)*p)) p++;
   if (*p == 0)
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
       "missing rewrite replacement string in line %d", config_lineno);
@@ -2948,7 +2942,7 @@ while ((p = get_config_line(TRUE, TRUE, FALSE, FALSE)) != NULL &&
   else
     {
     pp = p;
-    while (*p != 0 && !isspace(*p)) p++;
+    while (*p != 0 && !isspace((uschar)*p)) p++;
     next->replacement = string_copyn(pp, p-pp);
     }
 
