@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with retrying unsuccessful deliveries. */
@@ -21,8 +21,9 @@ IP address string. Its job is to set the status of the address if it is not
 already set (indicated by hstatus_unknown). The possible values are:
 
    hstatus_usable    the address is not listed in the unusable tree, and does
-                     not have a retry record, or the time is past the next
-                     try time.
+                     not have a retry record, OR the time is past the next
+                     try time, OR the message has been on the queue for more
+                     than the maximum retry time for a failing host
 
    hstatus_unusable  the address is listed in the unusable tree, or does have
                      a retry record, and the time is not yet at the next retry
@@ -31,12 +32,18 @@ already set (indicated by hstatus_unknown). The possible values are:
    hstatus_unusable_expired  as above, but also the retry time has expired
                      for this address.
 
+The reason a delivery is permitted when a message has been around for a very
+long time is to allow the ultimate address timeout to operate after a delivery
+failure. Otherwise some messages may stick around without being tried for too
+long.
+
 If a host retry record is retrieved from the hints database, the time of last
 trying is filled into the last_try field of the host block. If a host is
 generally usable, a check is made to see if there is a retry delay on this
 specific message at this host.
 
 Arguments:
+  domain              the address domain
   host                pointer to a host item
   include_ip_address  TRUE to include the address in the key - this is
                         usual, but sometimes is not wanted
@@ -50,7 +57,7 @@ Returns:    TRUE if the host has expired but is usable because
 */
 
 BOOL
-retry_check_address(host_item *host, BOOL include_ip_address,
+retry_check_address(char *domain, host_item *host, BOOL include_ip_address,
   char **retry_host_key, char **retry_message_key)
 {
 BOOL yield = FALSE;
@@ -128,20 +135,63 @@ else if (now - message_retry_record->time_stamp > retry_data_expire)
   }
 
 /* If there's a host-specific retry record, check for reaching the retry
-time (or forcing). If not, set the appropriate unusable flag and return FALSE.
-Otherwise arrange to return TRUE if this is an expired host. */
+time (or forcing). If not, and the host is not expired, check for the message
+having been around for longer than the maximum retry time for this host or
+address. Allow the delivery if it has. Otherwise set the appropriate unusable
+flag and return FALSE. Otherwise arrange to return TRUE if this is an expired
+host. */
 
 if (host_retry_record != NULL)
   {
+  time_t now = time(NULL);
+
   *retry_host_key = host_key;
-  if (time(NULL) < host_retry_record->next_try && !deliver_force)
+
+  if (now < host_retry_record->next_try && !deliver_force)
     {
+    /* Check for ultimate addres timeout if host not expired */
+
+    if (!host_retry_record->expired)
+      {
+      BOOL address_timeout = FALSE;
+
+      retry_config *retry =
+        retry_find_config(host_key+2, domain,
+          host_retry_record->basic_errno,
+          host_retry_record->more_errno);
+
+      DEBUG(9) debug_printf("retry time not reached: checking ultimate "
+        "address timeout\n");
+
+      if (retry != NULL)
+        {
+        retry_rule *last_rule;
+        for (last_rule = retry->rules;
+             last_rule->next != NULL;
+             last_rule = last_rule->next);
+        if (now - received_time > last_rule->timeout) address_timeout = TRUE;
+        }
+      else address_timeout = TRUE;    /* No rule => timed out */
+
+      if (address_timeout)
+        {
+        DEBUG(5) debug_printf("on queue longer than maximum retry for "
+          "address - allowing delivery\n");
+        return FALSE;
+        }
+      }
+
+    /* We have not hit the ultimate address timeout; host is unusable. */
+
     host->status = (host_retry_record->expired)?
       hstatus_unusable_expired : hstatus_unusable;
     host->why = hwhy_retry;
     host->last_try = host_retry_record->last_try;
     return FALSE;
     }
+
+  /* Host is usable; set return TRUE if expired. */
+
   yield = host_retry_record->expired;
   }
 
@@ -242,6 +292,7 @@ retry_find_config(char *destination, char *alternate, int basic_errno,
 {
 int domain = 0;
 int replace = ':';
+int sep = UCHAR_MAX + 1;
 char *colon = strchr(destination, ':');
 retry_config *yield;
 
@@ -292,13 +343,13 @@ for (yield = retries; yield != NULL; yield = yield->next)
   otherwise it's a remote host name. Either way, we can persuade the
   match_address function to handle it. */
 
-  if (match_address_list(destination, domain, &dlist, -1, 0, FALSE)) break;
+  if (match_address_list(destination, domain, &dlist, -1, &sep, FALSE)) break;
 
   /* If the match failed, have a go at matching on the alternative destination,
   which is normally the domain as opposed to the host for a remote delivery. */
 
   if (alternate != NULL &&
-      match_address_list(alternate, 0, &dlist, -1, 0, FALSE)) break;
+      match_address_list(alternate, 0, &dlist, -1, &sep, FALSE)) break;
   }
 
 *colon = replace;
