@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with rewriting headers */
@@ -87,6 +87,7 @@ BOOL done = FALSE;
 uid_t saved_euid = 0;
 gid_t saved_egid = 0;
 int rule_number = 1;
+int sep = UCHAR_MAX + 1;
 int yield_start = 0, yield_end = 0;
 
 if (whole != NULL) *whole = FALSE;
@@ -132,7 +133,7 @@ for (rule = rewrite_rules;
     /* Use the general function for matching an address against a list (here
     just one item, so set the separator to 0). */
 
-    if (!match_address_list(subject, domain - subject, &(rule->key), 0, 0, FALSE))
+    if (!match_address_list(subject,domain-subject,&(rule->key),0,&sep,FALSE))
       continue;
 
     /* The source address matches, and numerical variables have been
@@ -390,7 +391,7 @@ rewrite_one_header(header_line *h, int flag, char *routed_old, char *routed_new)
 int lastnewline = 0;
 header_line *newh = NULL;
 char *s = strchr(h->text, ':') + 1;
-while (isspace(*s)) s++;
+while (isspace((uschar)*s)) s++;
 
 DEBUG(5) debug_printf("rewrite_one_header: type=%c:\n  %s", h->type, h->text);
 
@@ -420,7 +421,7 @@ while (*s != 0)
   *ss = terminator;
   sprev = s;
   s = ss + (terminator? 1:0);
-  while (isspace(*s)) s++;
+  while (isspace((uschar)*s)) s++;
 
   /* There isn't much we can do for syntactic disasters at this stage.
   Pro tem (possibly for ever) ignore them. */
