@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with routing, and the list of generic router options. */
@@ -133,12 +133,12 @@ for (r = routers; r != NULL; r = r->next)
   else if (strncmp(s, "reroute:", 8) == 0)
     {
     s += 8;
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     if (strncmp(s, "rewrite:", 8) == 0)
       {
       r->self_rewrite = TRUE;
       s += 8;
-      while (isspace(*s)) s++;
+      while (isspace((uschar)*s)) s++;
       }
     r->self = s;
     r->self_code = self_reroute;
@@ -148,7 +148,7 @@ for (r = routers; r != NULL; r = r->next)
 
   /* Build a host list if fallback hosts is set */
 
-  host_build_hostlist(&(r->fallback_hostlist), r->fallback_hosts);
+  host_build_hostlist(&(r->fallback_hostlist), r->fallback_hosts, FALSE);
   }
 }
 
@@ -194,21 +194,30 @@ Arguments:
   addr_local    the local transport chain
   addr_remote   the remote transport chain,
   rblock        the router block
+  ugidptr       pointer to ugid block, or NULL if not yet set up
   type          "director" or "router"
 
-Returns:        FALSE on error
+Returns:        FALSE on error; the only case is failing to get a uid/gid
 */
 
 BOOL
 route_queue(address_item *addr, address_item **addr_local,
-  address_item **addr_remote, router_instance *rblock, char *type)
+  address_item **addr_remote, router_instance *rblock, ugid_block *ugidptr,
+  char *type)
 {
+addr->domain_data = domain_data;            /* Save these values for use */
+addr->local_part_data = local_part_data;    /* in the transport */
+
 if (addr->transport != NULL && addr->transport->info->local)
   {
   ugid_block ugid;
-  if (!direct_get_ugid((director_instance *)rblock, type, addr, &ugid))
-   return FALSE;
-  direct_set_ugid(addr, &ugid);
+  if (ugidptr == NULL)
+    {
+    if (!direct_get_ugid((director_instance *)rblock, type, addr, &ugid))
+      return FALSE;
+    ugidptr = &ugid;
+    }
+  direct_set_ugid(addr, ugidptr);
   addr->next = *addr_local;
   *addr_local = addr;
   }
@@ -219,9 +228,14 @@ else
   addr->fallback_hosts = rblock->fallback_hostlist;
   }
 
-DEBUG(2) debug_printf("queued for %s transport: local_part=%s domain=%s\n",
-  (addr->transport == NULL)? "<unset>" : addr->transport->name,
-  addr->local_part, addr->domain);
+DEBUG(2)
+  {
+  debug_printf("queued for %s transport: local_part=%s domain=%s\n",
+    (addr->transport == NULL)? "<unset>" : addr->transport->name,
+    addr->local_part, addr->domain);
+  debug_printf("  domain_data=%s local_part_data=%s\n", domain_data,
+    local_part_data);
+  }
 
 return TRUE;
 }
@@ -317,7 +331,8 @@ Arguments:
   pmore        address of the "more" flag
   driver_type  either "router" or "director"
 
-Returns:       TRUE if this driver is to be skipped
+Returns:       TRUE if this driver is to be skipped, but if search_find_defer
+               is set, it means a domains or local parts lookup deferred
 */
 
 
@@ -330,6 +345,10 @@ route_skip_driver(router_instance *dr, char *local_part, char *domain,
 domain_data = NULL;
 local_part_data = NULL;
 
+/* Unset this in case there aren't any actual lookups. */
+
+search_find_defer = FALSE;
+
 /* Skip this driver if not verifying and it has verify_only set */
 
 if ((verify == v_none || verify == v_expn) && dr->verify_only)
@@ -536,11 +555,18 @@ for (r = routers; more && r != NULL; r = r->next)
   int rc;
 
   /* Check the conditions that are common to both routers and directors. We
-  need the expansions set in case anything therein is expanded. */
+  need the expansions set in case anything therein is expanded. If there was
+  a lookup defer while checking, defer this address. */
 
   deliver_set_expansions(addr);
   if (route_skip_driver(r, addr->local_part, addr->route_domain,
-    verify, &more, "router")) continue;
+      verify, &more, "router"))
+    {
+    if (!search_find_defer) continue;
+    addr->message = "lookup defer for domains or local_parts";
+    addr->router = r;
+    return DEFER;
+    }
 
   /* Loop protection: If this address has a parent with the same address that
   was routed by this router, we skip this router. This prevents a looping
@@ -594,21 +620,32 @@ for (r = routers; more && r != NULL; r = r->next)
       {
       if (rc == DEFER)
         {
-        addr->message = string_sprintf("sender check defer in %s router",
-          r->name);
+        addr->message = "sender check lookup defer";
         yield = DEFER;
         }
-      DEBUG(9) debug_printf("%s router: sender match failure\n",
-        r->name);
-      yield = FAIL;
+      else
+        {
+        DEBUG(9) debug_printf("%s router: sender match failure\n",
+          r->name);
+        yield = FAIL;
+        }
       }
     else if (r->condition != NULL &&
           !expand_check_condition(r->condition, r->name, "router"))
       {
-      DEBUG(9) debug_printf("%s router: condition failure; more set TRUE\n",
-        r->name);
-      yield = FAIL;
-      more = TRUE;      /* Condition tests are logically before "more" */
+      if (search_find_defer)
+        {
+        addr->message = "condition check lookup defer";
+        yield = DEFER;
+        DEBUG(9) debug_printf("%s\n", addr->message);
+        }
+      else
+        {
+        DEBUG(9) debug_printf("%s router: condition failure; more set TRUE\n",
+          r->name);
+        yield = FAIL;
+        more = TRUE;      /* Condition tests are logically before "more" */
+        }
       }
     else
       {
