@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -24,29 +24,31 @@ not set explicitly, but as a side-effect of search_type. An entry in this
 table allows the option-reading function to find where to put the value. */
 
 optionlist domainlist_router_options[] = {
-  { "*partial_match", opt_int | opt_hidden,
+  { "*partial_match",   opt_int | opt_hidden,
       (void *)(offsetof(domainlist_router_options_block, partial_match)) },
   { "host_find_failed", opt_stringptr,
       (void *)(offsetof(domainlist_router_options_block, host_find_failed)) },
-  { "modemask",       opt_octint,
+  { "hosts_randomize",  opt_bool,
+      (void *)(offsetof(domainlist_router_options_block, hosts_randomize)) },
+  { "modemask",         opt_octint,
       (void *)(offsetof(domainlist_router_options_block, modemask)) },
-  { "owners",         opt_uidlist,
+  { "owners",           opt_uidlist,
       (void *)(offsetof(domainlist_router_options_block, owners)) },
-  { "owngroups",      opt_gidlist,
+  { "owngroups",        opt_gidlist,
       (void *)(offsetof(domainlist_router_options_block, owngroups)) },
-  { "qualify_single",     opt_bool,
+  { "qualify_single",   opt_bool,
       (void *)(offsetof(domainlist_router_options_block, qualify_single)) },
-  { "route_file",     opt_stringptr,
+  { "route_file",       opt_stringptr,
       (void *)(offsetof(domainlist_router_options_block, route_file)) },
-  { "route_list",     opt_stringptr,
+  { "route_list",       opt_stringptr,
       (void *)(offsetof(domainlist_router_options_block, route_list)) },
-  { "route_queries",  opt_stringptr,
+  { "route_queries",    opt_stringptr,
       (void *)(offsetof(domainlist_router_options_block, route_queries)) },
-  { "route_query",    opt_stringptr,
+  { "route_query",      opt_stringptr,
       (void *)(offsetof(domainlist_router_options_block, route_query)) },
-  { "search_parents",     opt_bool,
+  { "search_parents",   opt_bool,
       (void *)(offsetof(domainlist_router_options_block, search_parents)) },
-  { "search_type",    opt_searchtype,
+  { "search_type",      opt_searchtype,
       (void *)(offsetof(domainlist_router_options_block, search_type)) }
 };
 
@@ -65,6 +67,7 @@ domainlist_router_options_block domainlist_router_option_defaults = {
   hff_freeze,   /* host_find_failed code */
   NULL,         /* owners */
   NULL,         /* owngroups */
+  FALSE,        /* hosts_randomize */
   TRUE,         /* qualify_single */
   FALSE,        /* search_parents */
   "freeze",     /* host_find_failed */
@@ -179,7 +182,7 @@ else
 if obtained from a configuration string. The host list may be enclosed in
 single or double quotes. When read from a file, the domain isn't present (it's
 the key used for the lookup). Point the arguments to the various parts of the
-line, and insert zeroes to terminate the strings. If domain == NULL, then
+line, and insert zeros to terminate the strings. If domain == NULL, then
 there's no domain at the start.
 
 Arguments:
@@ -197,14 +200,14 @@ parse_routelist_item(char *s, char **domain, char **hostlist, char **options)
 {
 if (domain != NULL)
   {
-  while (*s != 0 && isspace(*s)) s++;
+  while (*s != 0 && isspace((uschar)*s)) s++;
   if (*s == 0) return FALSE;            /* missing data */
   *domain = s;
-  while (*s != 0 && !isspace(*s)) s++;
+  while (*s != 0 && !isspace((uschar)*s)) s++;
   if (*s != 0) *s++ = 0;
   }
 
-while (*s != 0 && isspace(*s)) s++;
+while (*s != 0 && isspace((uschar)*s)) s++;
 
 if (*s == '\"' || *s == '\'')
   {
@@ -214,13 +217,13 @@ if (*s == '\"' || *s == '\'')
 else
   {
   *hostlist = s;
-  while (*s != 0 && !isspace(*s)) s++;
+  while (*s != 0 && !isspace((uschar)*s)) s++;
   }
 
 if (*s != 0)
   {
   *s++ = 0;
-  while (*s != 0 && isspace(*s)) s++;
+  while (*s != 0 && isspace((uschar)*s)) s++;
   }
 
 *options = s;
@@ -256,7 +259,7 @@ domainlist_router_entry(
 {
 BOOL matched = FALSE;
 BOOL individual_transport_set = FALSE;
-int lookup_type, rc;
+int lookup_type, rc, host_sep, host_count;
 host_item *h, *next_h, *prev;
 char *routelist_item = NULL;
 char *domain, *hostlist, *newhostlist, *options, *listptr;
@@ -264,6 +267,7 @@ domainlist_router_options_block *ob =
   (domainlist_router_options_block *)(rblock->options_block);
 transport_instance *transport;
 char route_buffer[1024];
+char buffer[128];
 
 DEBUG(2) debug_printf("%s router called for %s\n  route_domain = %s\n",
   rblock->name, addr->orig, addr->route_domain);
@@ -273,8 +277,9 @@ both are specified. The list takes precedence over the file. */
 
 if (ob->route_list != NULL)
   {
+  int sep = -(';');             /* Default is semicolon */
   listptr = ob->route_list;
-  while ((routelist_item = string_nextinlist(&listptr, ';', route_buffer,
+  while ((routelist_item = string_nextinlist(&listptr, &sep, route_buffer,
          sizeof(route_buffer))) != NULL)
     {
     DEBUG(9) debug_printf("routelist_item = %s\n", routelist_item);
@@ -403,7 +408,8 @@ if (!matched && (ob->route_file != NULL || ob->route_query != NULL ||
   if (ob->route_queries != NULL)
     {
     char *onequery;
-    while ((onequery = string_nextinlist(&query, ':', big_buffer,
+    int sep = 0;
+    while ((onequery = string_nextinlist(&query, &sep, big_buffer,
            BIG_BUFFER_SIZE)) != NULL)
       {
       routelist_item = search_find(handle, filename, onequery,
@@ -473,6 +479,16 @@ else hostlist = newhostlist;
 DEBUG(9) debug_printf("expanded hostlist=\"%s\" options=%s\n", hostlist,
   options);
 
+/* Some paths below require only a single item in the host list. The only safe
+way to check this is to read a couple of items from the list. Checking for the
+separator character alone is not right - it may appear doubled. Pointless
+reproducing the code again here. */
+
+host_sep = host_count = 0;
+listptr = hostlist;
+while (string_nextinlist(&listptr,&host_sep,buffer,(int)sizeof(buffer)) != NULL)
+  host_count++;
+
 /* Set default lookup type and scan the options */
 
 lookup_type = lk_pass;
@@ -481,7 +497,7 @@ while (*options != 0)
   {
   int term;
   char *s = options;
-  while (*options != 0 && !isspace(*options)) options++;
+  while (*options != 0 && !isspace((uschar)*options)) options++;
   term = *options;
   *options = 0;
 
@@ -514,7 +530,7 @@ while (*options != 0)
   if (term != 0)
     {
     options++;
-    while (*options != 0 && isspace(*options)) options++;
+    while (*options != 0 && isspace((uschar)*options)) options++;
     }
   }
 
@@ -549,7 +565,7 @@ if (transport != NULL && transport->info->local)
     {
     host_item *h;
 
-    if (strchr(hostlist, ':') != NULL)
+    if (host_count > 1)
       {
       log_write(0, LOG_MAIN, "Error in %s router: attempt to pass more than one "
         "host name to a local transport", rblock->name);
@@ -574,7 +590,7 @@ if (transport != NULL && transport->info->local)
   route_queue() function. */
 
   addr->transport = transport;
-  return route_queue(addr, addr_local, addr_remote, rblock, "router")?
+  return route_queue(addr, addr_local, addr_remote, rblock, NULL, "router")?
     OK : ERROR;
   }
 
@@ -599,7 +615,7 @@ host name. */
 
 if (lookup_type == lk_pass)
   {
-  if (strchr(hostlist, ':') != NULL || individual_transport_set)
+  if (host_count > 1 || individual_transport_set)
     {
     char *s = individual_transport_set?
       "transport specified when passing domain on to next router" :
@@ -619,8 +635,7 @@ if (lookup_type == lk_pass)
 for the list of configured hosts, and then finding their addresses. Typically
 there is only one host. */
 
-host_build_hostlist(&(addr->host_list), hostlist);
-
+host_build_hostlist(&(addr->host_list), hostlist, ob->hosts_randomize);
 
 /* Look up each host address. A lookup may add additional items into the chain
 if there are multiple addresses. Hence the use of next_h to start each cycle of
@@ -655,6 +670,9 @@ for (h = addr->host_list; h != NULL; prev = h, h = next_h)
         rblock->name);
       return FAIL;
       }
+    addr->message = string_sprintf("host lookup for %s did not complete "
+      "(DNS timeout?)", h->name);
+    addr->basic_errno = ERRNO_DNSDEFER;
     return DEFER;
     }
 
