@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -145,7 +145,7 @@ if (!route_get_transport(rblock->transport, rblock->expand_transport,
       addr, &(addr->transport), rblock->name, NULL))
   return ERROR;
 
-return route_queue(addr, addr_local, addr_remote, rblock, "router")?
+return route_queue(addr, addr_local, addr_remote, rblock, NULL, "router")?
   OK : ERROR;
 }
 
