@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -94,6 +94,7 @@ lookuphost_router_entry(
 {
 host_item h;
 int rc;
+int widen_sep = 0;
 lookuphost_router_options_block *ob =
   (lookuphost_router_options_block *)(rblock->options_block);
 char *widen = NULL;
@@ -121,7 +122,8 @@ with some conditions under which it is done, it can be selected here. */
 if (ob->widen_domains != NULL)
   {
   listptr = ob->widen_domains;
-  widen = string_nextinlist(&listptr, ':', widen_buffer, sizeof(widen_buffer));
+  widen = string_nextinlist(&listptr, &widen_sep, widen_buffer,
+    sizeof(widen_buffer));
 
 /****
   if (some condition requiring pre-widening)
@@ -147,7 +149,8 @@ for (;;)
   else if (widen != NULL)
     {
     h.name = string_sprintf("%s.%s", addr->route_domain, widen);
-    widen = string_nextinlist(&listptr,':',widen_buffer,sizeof(widen_buffer));
+    widen = string_nextinlist(&listptr, &widen_sep, widen_buffer,
+      sizeof(widen_buffer));
     DEBUG(2) debug_printf("%s router widened %s to %s\n", rblock->name,
       addr->route_domain, h.name);
     }
@@ -187,17 +190,36 @@ for (;;)
     rc = host_find_bydns(&h, FALSE, FALSE, ob->qualify_single,
       ob->search_parents, &fully_qualified_name, &(addr->local_host_removed));
 
-    if ((rc == HOST_FOUND || rc == HOST_FOUND_LOCAL) &&
-        h.mx < 0 &&                            /* Found A records only */
-        (ob->mx_domains != NULL &&             /* In mx-domains list */
-        match_isinlist(fully_qualified_name, &(ob->mx_domains), TRUE, TRUE,
-          NULL)) &&
-        !match_isinlist(fully_qualified_name,  /* Not turned into a local */
-          &local_domains, TRUE, TRUE, NULL))   /* domain */
+    /* If host found with only A records, test for the domain's being in the
+    mx_domains list, and not turned into a local domain. */
+
+    if ((rc == HOST_FOUND || rc == HOST_FOUND_LOCAL) && h.mx < 0)
       {
-      DEBUG(2) debug_printf("%s router rejected %s: no MX record(s)\n",
-        rblock->name, fully_qualified_name);
-      continue;
+      BOOL become_local;
+      BOOL in_mx_domains = ob->mx_domains != NULL &&
+        match_isinlist(fully_qualified_name, &(ob->mx_domains),TRUE,TRUE,NULL);
+
+      if (search_find_defer)
+        {
+        addr->message = "lookup defer for mx_domains";
+        return DEFER;
+        }
+
+      become_local =
+        match_isinlist(fully_qualified_name, &local_domains,TRUE,TRUE,NULL);
+
+      if (search_find_defer)
+        {
+        addr->message = "lookup defer for local_domains";
+        return DEFER;
+        }
+
+      if (in_mx_domains && !become_local)
+        {
+        DEBUG(2) debug_printf("%s router rejected %s: no MX record(s)\n",
+          rblock->name, fully_qualified_name);
+        continue;
+        }
       }
     }
 
@@ -255,12 +277,18 @@ if (strcmp(addr->route_domain, fully_qualified_name) != 0)
   {
   addr->route_domain = fully_qualified_name;
   addr->rewrite_headers = ob->rewrite_headers;
+  search_find_defer = FALSE;
   if (rc == HOST_FOUND_LOCAL ||
       match_isinlist(fully_qualified_name, &local_domains, TRUE, TRUE, NULL))
     {
     addr->rewrite_headers = TRUE;
     return ISLOCAL;
     }
+  else if (search_find_defer)
+    {
+    addr->message = "lookup defer for local_domains";
+    return DEFER;
+    }
   }
 
 
@@ -325,7 +353,7 @@ if (!route_get_transport(rblock->transport, rblock->expand_transport,
       addr, &(addr->transport), rblock->name, NULL))
   return ERROR;
 
-return route_queue(addr, addr_local, addr_remote, rblock, "router")?
+return route_queue(addr, addr_local, addr_remote, rblock, NULL, "router")?
   OK : ERROR;
 }
 
