@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -91,8 +91,8 @@ queryprogram_router_entry(
   address_item **addr_remote,     /* add it to this if successful & remote */
   BOOL verify)                    /* TRUE if verifying */
 {
-int pfd[2];
-int pid, fd_in, i, len, rc;
+int fd_in, fd_out, i, len, rc;
+pid_t pid;
 uid_t uid;
 gid_t gid;
 char *s, *expanded;
@@ -143,17 +143,6 @@ gid = (ob->gid_set)? ob->gid : nobody_gid;
 DEBUG(2) debug_printf("uid=%ld gid=%ld current_directory=%s\n",
   (long int)uid, (long int)gid, current_directory);
 
-/* Create a pipe to read the standard output of the subprocess. */
-
-if (pipe(pfd) != 0)
-  {
-  log_write(0, LOG_MAIN, "%s router: couldn't create pipe", rblock-> name);
-  addr->message = string_sprintf("%s router couldn't create pipe",
-    rblock->name);
-  return DEFER;
-  }
-
-
 /* Expand the command string, making the domain available as a variable,
 and ensuring a copy of the text is made. Then set up the arguments for the
 command. */
@@ -175,17 +164,17 @@ if (expanded == NULL)
 
 for (i = 0; *s != 0 && i < 19; i++)
   {
-  while (isspace(*s)) s++;
+  while (isspace((uschar)*s)) s++;
   argv[i] = s;
-  while (*s != 0 && !isspace(*s)) s++;
+  while (*s != 0 && !isspace((uschar)*s)) s++;
   if (*s != 0) *s++ = 0;
   }
 argv[i] = NULL;
 
 
-/* Create the child process and make it a group leader */
+/* Create the child process, making it a group leader. */
 
-pid = child_open(argv, NULL, 0077, &uid, &gid, &fd_in, pfd[pipe_write],
+pid = child_open(argv, NULL, 0077, &uid, &gid, &fd_in, &fd_out,
   current_directory, TRUE);
 
 if (pid < 0)
@@ -196,6 +185,9 @@ if (pid < 0)
   return DEFER;
   }
 
+/* Nothing is written to the standard input. */
+
+close(fd_in);
 
 /* Wait for the process to finish, applying the timeout, and inspect its return
 code. */
@@ -203,8 +195,8 @@ code. */
 if ((rc = child_close(pid, ob->timeout)) != 0)
   {
   if (rc > 0)
-    addr->message = string_sprintf("%s router: command returned non-zero code %d",
-      rblock->name, rc);
+    addr->message = string_sprintf("%s router: command returned non-zero "
+      "code %d", rblock->name, rc);
 
   else if (rc == -256)
     {
@@ -234,13 +226,10 @@ if ((rc = child_close(pid, ob->timeout)) != 0)
   }
 
 
-/* Read the pipe to get the command's output. Our copy of the writing end must
-be closed first, as otherwise read() won't return zero on an empty pipe.
-Afterwards, close the reading end. */
+/* Read the pipe to get the command's output, and then close it. */
 
-close(pfd[pipe_write]);
-len = read(pfd[pipe_read], buffer, sizeof(buffer));
-close(pfd[pipe_read]);
+len = read(fd_out, buffer, sizeof(buffer));
+close(fd_out);
 
 
 /* Failure to return any data is an error. */
@@ -258,7 +247,7 @@ if (len <= 0)
 The final field goes on to the end of the string and may contain white space.
 Ignore leading and trailing white space. */
 
-while (len > 0 && isspace(buffer[len-1])) len--;
+while (len > 0 && isspace((uschar)buffer[len-1])) len--;
 buffer[len] = 0;
 
 DEBUG(2) debug_printf("command wrote: %s\n", buffer);
@@ -266,9 +255,9 @@ DEBUG(2) debug_printf("command wrote: %s\n", buffer);
 s = buffer;
 for (i = 0; i < 4; i++)
   {
-  while (isspace(*s)) s++;
+  while (isspace((uschar)*s)) s++;
   returned[i] = s;
-  while (*s != 0 && !isspace(*s)) s++;
+  while (*s != 0 && !isspace((uschar)*s)) s++;
   if (*s != 0) *s++ = 0;
   }
 returned[4] = s;
@@ -433,7 +422,7 @@ route_queue() function. */
 
 addr->transport = transport;
 if (returned[4][0] != 0) addr->route_option = string_copy(returned[4]);
-return route_queue(addr, addr_local, addr_remote, rblock, "router")?
+return route_queue(addr, addr_local, addr_remote, rblock, NULL, "router")?
   OK : ERROR;
 }
 
