@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for reading spool files. */
@@ -63,7 +63,11 @@ for (i = 0; i < 2; i++)
   return FALSE;
   }
 
-/* File is open and message_subdir is set. Lock the file. */
+/* File is open and message_subdir is set. Set the close-on-exec flag, and lock
+the file. */
+
+fcntl(deliver_datafile, F_SETFD, fcntl(deliver_datafile, F_GETFD) |
+  FD_CLOEXEC);
 
 lock_data.l_type = F_WRLCK;
 lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
@@ -425,7 +429,7 @@ for (n = 0; n < recipients_count; n++)
   left here just in case. Nothing is documented. */
 
   p = big_buffer + nn - 2;
-  while (isdigit(*p) || *p == ',') p--;
+  while (isdigit((uschar)*p) || *p == ',') p--;
   if (*p == ' ')
     {
     int orc;
@@ -506,9 +510,9 @@ while ((n = fgetc(f)) != EOF)
     if (header_checkname(h, "Precedence", 10))
       {
       char *s = h->text + 10;
-      while (isspace(*s)) s++;
+      while (isspace((uschar)*s)) s++;
       s++;
-      while (isspace(*s)) s++;
+      while (isspace((uschar)*s)) s++;
       message_precedence = string_copy(s);
       }
     }
