@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for writing spool files, and moving them about. */
@@ -140,15 +140,17 @@ fprintf(f, "%.63s %ld %ld\n", originator_login, (long int)originator_uid,
 fprintf(f, "<%s>\n", sender_address);
 fprintf(f, "%d %d\n", received_time, warning_count);
 
-/* If there is information about a sending host, remember it. */
+/* If there is information about a sending host, remember it. The HELO
+data can be set for local SMTP as well as remote. */
+
+if (sender_helo_name != NULL)
+  fprintf(f, "-helo_name %s\n", sender_helo_name);
 
 if (sender_host_address != NULL)
   {
   fprintf(f, "-host_address %s\n", sender_host_address);
   if (sender_host_name != NULL)
     fprintf(f, "-host_name %s\n", sender_host_name);
-  if (sender_helo_name != NULL)
-    fprintf(f, "-helo_name %s\n", sender_helo_name);
   if (sender_host_authenticated != NULL)
     fprintf(f, "-host_auth %s\n", sender_host_authenticated);
   }
