@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Miscellaneous string-handling functions */
@@ -46,9 +46,9 @@ if (maskptr != NULL)
   {
   char *ss = s + (int)strlen(s);
   *maskptr = 0;
-  if (s != ss && isdigit(*(--ss)))
+  if (s != ss && isdigit((uschar)*(--ss)))
     {
-    while (ss > s && isdigit(ss[-1])) ss--;
+    while (ss > s && isdigit((uschar)ss[-1])) ss--;
     if (ss > s && *(--ss) == '/')
       {
       *ss = 0;
@@ -139,8 +139,8 @@ if (strchr(s, ':') != NULL)
 for (i = 0; i < 4; i++)
   {
   if (i != 0 && *s++ != '.') return 0;
-  if (!isdigit(*s++)) return 0;
-  if (isdigit(*s) && isdigit(*(++s))) s++;
+  if (!isdigit((uschar)*s++)) return 0;
+  if (isdigit((uschar)*s) && isdigit((uschar)*(++s))) s++;
   }
 
 return (*s == 0)? yield : 0;
@@ -227,15 +227,16 @@ Returns:   the value of the character escape
 int
 string_interpret_escape(char **pp)
 {
+int ch;
 char *p = *pp;
-int ch = *(++p);
+ch = *(++p);
 if (isdigit(ch) && ch != '8' && ch != '9')
   {
   ch -= '0';
-  if (isdigit(p[1]) && p[1] != '8' && p[1] != '9')
+  if (isdigit((uschar)p[1]) && p[1] != '8' && p[1] != '9')
     {
     ch = ch * 8 + *(++p) - '0';
-    if (isdigit(p[1]) && p[1] != '8' && p[1] != '9')
+    if (isdigit((uschar)p[1]) && p[1] != '8' && p[1] != '9')
       ch = ch * 8 + *(++p) - '0';
     }
   }
@@ -246,12 +247,12 @@ else switch(ch)
   case 't':  ch = '\t'; break;
   case 'x':
   ch = 0;
-  if (isxdigit(p[1]))
+  if (isxdigit((uschar)p[1]))
     {
     ch = ch * 16 +
-      strchr(hex_digits, tolower(*(++p))) - hex_digits;
-    if (isxdigit(p[1])) ch = ch * 16 +
-      strchr(hex_digits, tolower(*(++p))) - hex_digits;
+      strchr(hex_digits, tolower((uschar)*(++p))) - hex_digits;
+    if (isxdigit((uschar)p[1])) ch = ch * 16 +
+      strchr(hex_digits, tolower((uschar)*(++p))) - hex_digits;
     }
   break;
   }
@@ -463,7 +464,7 @@ char *t, *yield;
 
 if (*s != '\"')
   {
-  while (*s != 0 && !isspace(*s)) s++;
+  while (*s != 0 && !isspace((uschar)*s)) s++;
   }
 else
   {
@@ -485,7 +486,7 @@ s = *sptr;
 
 if (*s != '\"')
   {
-  while (*s != 0 && !isspace(*s)) *t++ = *s++;
+  while (*s != 0 && !isspace((uschar)*s)) *t++ = *s++;
   }
 else
   {
@@ -642,13 +643,26 @@ return NULL;
 *       Get next string from separated list      *
 *************************************************/
 
-/* Leading and trailing space is removed from each item.
+/* Leading and trailing space is removed from each item. The separator in the
+list is controlled by the int pointed to by the separator argument as follows:
+
+  If its value is > 0 it is used as the delimiter.
+    (If its value is actually > UCHAR_MAX there is only one item in the list.
+    This is used for some cases when called via match_isinlist.)
+  If its value is <= 0, the string is inspected for a leading <x, where
+    x is an ispunct() value. If found, it is used as the delimiter. If not
+    found: (a) if separator == 0, ':' is used
+           (b) if separator <0, then -separator is used
+    In all cases the value of the separator that is used is written back to
+      the int so that it is used on subsequent calls as we progress through
+      the list.
+
+The separator can always be represented in the string by doubling.
 
 Arguments:
   listptr    points to a pointer to the current start of the list; the
              pointer gets updated to point after the end of the next item
-  separator  the separator character; it can be represented in the
-             list by duplication
+  separator  a pointer to the separator character in an int (see above)
   buffer     where to put a copy of the next string in the list
   buflen     the size of buffer
 
@@ -657,22 +671,39 @@ Returns:     pointer to buffer, containing the next substring,
 */
 
 char *
-string_nextinlist(char **listptr, int separator, char *buffer, int buflen)
+string_nextinlist(char **listptr, int *separator, char *buffer, int buflen)
 {
-int p = 0;
-unsigned char *s = (unsigned char *)(*listptr);
+register int p = 0;
+register int sep = *separator;
+register uschar *s = (uschar *)(*listptr);
 
 if (s == NULL) return NULL;
 while (isspace(*s)) s++;
 if (*s == 0) return NULL;
 
+if (sep <= 0)
+  {
+  if (*s == '<' && ispunct(s[1]))
+    {
+    sep = s[1];
+    s += 2;
+    while (isspace(*s)) s++;
+    if (*s == 0) return NULL;
+    }
+  else
+    {
+    sep = (sep == 0)? ':' : -sep;
+    }
+  *separator = sep;
+  }
+
 for (; *s != 0; s++)
   {
-  if (*s == separator && *(++s) != separator) break;
+  if (*s == sep && *(++s) != sep) break;
   if (p < buflen - 1) buffer[p++] = *s;
   }
 
-while (p > 0 && isspace(buffer[p-1])) p--;
+while (p > 0 && isspace((uschar)buffer[p-1])) p--;
 buffer[p] = 0;
 *listptr = (char *)s;
 return buffer;
@@ -791,10 +822,10 @@ while (*format != 0)
 
   if (strchr("-+ #0", *(++format)) != NULL) format++;
 
-  if (isdigit(*format))
+  if (isdigit((uschar)*format))
     {
     width = *format++ - '0';
-    while (isdigit(*format)) width = width * 10 + *format++ - '0';
+    while (isdigit((uschar)*format)) width = width * 10 + *format++ - '0';
     }
   else if (*format == '*')
     {
@@ -812,7 +843,8 @@ while (*format != 0)
     else
       {
       precision = 0;
-      while (isdigit(*format)) precision = precision*10 + *format++ - '0';
+      while (isdigit((uschar)*format))
+        precision = precision*10 + *format++ - '0';
       }
     }
 
