@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* General functions concerned with transportation, and generic options for all
@@ -171,8 +171,8 @@ int i, rc, save_errno;
 for (i = 0; i < 100; i++)
   {
   DEBUG(9)
-    debug_printf("writing data block size=%d timeout=%d\n",
-      len, transport_write_timeout);
+    debug_printf("writing data block fd=%d size=%d timeout=%d\n",
+      fd, len, transport_write_timeout);
   if (transport_write_timeout > 0) alarm(transport_write_timeout);
   rc = write(fd, block, len);
   save_errno = errno;
@@ -213,8 +213,20 @@ for (i = 0; i < 100; i++)
     continue;
     }
 
+  /* A response of EAGAIN from write() is likely only in the case of writing
+  to a FIFO that is not swallowing the data as fast as Exim is writing it. */
+
+  if (save_errno == EAGAIN)
+    {
+    DEBUG(9) debug_printf("write temporarily locked out, waiting 1 sec\n");
+    sleep(1);
+    continue;
+    }
+
   /* Otherwise there's been an error */
 
+  DEBUG(9) debug_printf("writing error %d: %s\n", save_errno,
+    strerror(save_errno));
   errno = save_errno;
   return FALSE;
   }
@@ -587,9 +599,10 @@ if ((options & topt_no_headers) == 0)
       {
       if (list != NULL)
         {
+        int sep = 0;
         char *s, *ss;
         char buffer[128];
-        while ((s = string_nextinlist(&list, ':', buffer, sizeof(buffer)))
+        while ((s = string_nextinlist(&list, &sep, buffer, sizeof(buffer)))
                 != NULL)
           {
           int len = (int)strlen(s);
@@ -714,10 +727,8 @@ transport_write_message(address_item *addr, int fd, int options,
   char *escape_string)
 {
 BOOL use_crlf;
-int rc;
+int rc, len, yield, fd_read, fd_write, save_errno;
 int pfd[2];
-int len, fd_read, fd_write, yield;
-int save_errno = 0;
 pid_t filter_pid, write_pid;
 
 /* If there is no filter command set up, call the internal function that does
@@ -727,32 +738,33 @@ if (transport_filter_argv == NULL)
   return internal_transport_write_message(addr, fd, options, size_limit,
     add_headers, remove_headers, check_string, escape_string);
 
-/* Otherwise the message must be written through a filter. Create a pipe for
-stdout, saving the reading end. */
-
-if (pipe(pfd) != 0) return FALSE;
-fd_read = pfd[pipe_read];
-
-/* Start up a subprocess to run the command, putting the standard input input
-in fd_write, and passing it the other pipe for the standard output. Then we can
-close the writing end of the pipe in this process. If the process creation
-failed, give an error return. */
+/* Otherwise the message must be written through a filter. Start up a
+subprocess to run the command. First ensure that our main fd will be closed in
+the subprocess. We get back stdin/stdout file descriptors. If the process
+creation failed, give an error return. */
 
+fd_read = -1;
+fd_write = -1;
+save_errno = 0;
 yield = FALSE;
 write_pid = (pid_t)(-1);
-fd_write = -1;
+
+fcntl(fd, F_SETFD, fcntl(fd, F_GETFD) | FD_CLOEXEC);
 
 filter_pid = child_open(transport_filter_argv, NULL, 077, NULL, NULL,
-  &fd_write, pfd[pipe_write], NULL, FALSE);
+  &fd_write, &fd_read, NULL, FALSE);
 
-close(pfd[pipe_write]);
 if (filter_pid < 0)
   {
   save_errno = errno;
   goto TIDY_UP;
   }
 
-DEBUG(3) debug_printf("process %d running as transport filter\n", filter_pid);
+DEBUG(3)
+  {
+  debug_printf("process %d running as transport filter\n", filter_pid);
+  DEBUG(9) debug_printf("fd_write=%d fd_read=%d\n", fd_write, fd_read);
+  }
 
 /* Fork subprocess to write the message to the filter, and return the result
 via a(nother) pipe. While writing to the filter, we do not do the CRLF,
@@ -856,7 +868,10 @@ for (;;)
 
 if (len != 0) { save_errno = errno; goto TIDY_UP; }
 
-/* If requested, add a terminating "." line (SMTP output). */
+/* If requested, add a terminating "." line (SMTP output), turning off
+escaping beforehand. */
+
+nl_check_length = nl_escape_length = 0;
 
 if ((options & topt_end_dot) != 0 && !write_chunk(fd, ".\n", 2, use_crlf))
   {
@@ -887,7 +902,7 @@ if (!yield)
 
 /* Wait for the filter process to complete. */
 
-DEBUG(9) debug_printf("Waiting for filter process\n");
+DEBUG(9) debug_printf("waiting for filter process\n");
 if (filter_pid > 0 && (rc = child_close(filter_pid, 30)) != 0 && yield)
   {
   yield = FALSE;
@@ -1321,7 +1336,7 @@ int status;
 
 if ((pid = vfork()) == 0)
   {
-  int fd, i;
+  int i;
   char *argv[17 + MAX_CLMACROS];
 
   /* Disconnect entirely from the parent process. */
@@ -1376,14 +1391,7 @@ if ((pid = vfork()) == 0)
   argv[i++] = id;
   argv[i++] = NULL;
 
-  /* Arrange for the channel to be on stdin, with all other file descriptors
-  closed, except for the queue runner's synchronizing pipe if present, and the
-  debug_file while debugging). */
-
-  for (fd = mac_maxfd; fd >= 0; fd--)
-    if (fd != socket_fd && fd != queue_run_pipe &&
-      (debug_file == NULL || fd != fileno(debug_file)))
-        close(fd);
+  /* Arrange for the channel to be on stdin. */
 
   if (socket_fd != 0)
     {
@@ -1576,7 +1584,7 @@ total length can be no longer than the length of the original. */
 
 s = cmd;
 ss = store_get((int)strlen(s) + 1);
-while (isspace(*s)) s++;
+while (isspace((uschar)*s)) s++;
 
 while (*s != 0 && argcount < max_args)
   {
@@ -1625,12 +1633,12 @@ while (*s != 0 && argcount < max_args)
 
   /* Argument not in quotes is terminated by EOL or white space */
 
-  else while (*s != 0 && !isspace(*s)) *ss++ = *s++;
+  else while (*s != 0 && !isspace((uschar)*s)) *ss++ = *s++;
 
   /* Terminate the current argument, and skip trailing spaces. */
 
   *ss++ = 0;
-  while (isspace(*s)) s++;
+  while (isspace((uschar)*s)) s++;
   }
 
 argv[argcount] = (char *)0;
