@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -59,6 +59,8 @@ optionlist appendfile_transport_options[] = {
       (void *)(offsetof(transport_instance, bsmtp_helo)) },
   { "check_group",       opt_bool,
       (void *)(offsetof(appendfile_transport_options_block, check_group)) },
+  { "check_owner",       opt_bool,
+      (void *)(offsetof(appendfile_transport_options_block, check_owner)) },
   { "check_string",      opt_stringptr,
       (void *)(offsetof(appendfile_transport_options_block, check_string)) },
   { "create_directory",  opt_bool,
@@ -83,6 +85,8 @@ optionlist appendfile_transport_options[] = {
       (void *)(offsetof(appendfile_transport_options_block, from_hack)) },
   { "group",             opt_expand_gid | opt_public,
       (void *)(offsetof(transport_instance, gid)) },
+  { "lock_fcntl_timeout", opt_time,
+      (void *)(offsetof(appendfile_transport_options_block, lock_fcntl_timeout)) },
   { "lock_interval",     opt_time,
       (void *)(offsetof(appendfile_transport_options_block, lock_interval)) },
   { "lock_retries",      opt_int,
@@ -123,6 +127,8 @@ optionlist appendfile_transport_options[] = {
       (void *)(offsetof(appendfile_transport_options_block, quota)) },
   { "quota_filecount",   opt_int,
       (void *)(offsetof(appendfile_transport_options_block, quota_filecount)) },
+  {"quota_size_regex",   opt_stringptr,
+      (void *)(offsetof(appendfile_transport_options_block, quota_size_regex)) },
   { "quota_warn_message", opt_stringptr | opt_public,
       (void *)(offsetof(transport_instance, warn_message)) },
   { "quota_warn_threshold", opt_stringptr,
@@ -163,6 +169,7 @@ appendfile_transport_options_block appendfile_transport_option_defaults = {
   "\n",           /* suffix */
   "anywhere",     /* create_file_string (string value for create_file) */
   NULL,           /* quota */
+  NULL,           /* quota_size_regex */
   NULL,           /* quota_warn_threshold */
   NULL,           /* maildir_tag */
   NULL,           /* mailstore_prefix */
@@ -177,6 +184,7 @@ appendfile_transport_options_block appendfile_transport_option_defaults = {
   0700,           /* dirmode */
   0600,           /* lockfile_mode */
   30*60,          /* lockfile_timeout */
+  0,              /* lock_fcntl_timeout */
   10,             /* lock_retries */
    3,             /* lock_interval */
   10,             /* maildir_retries */
@@ -185,6 +193,7 @@ appendfile_transport_options_block appendfile_transport_option_defaults = {
   FALSE,          /* allow_fifo */
   FALSE,          /* allow_symlink */
   FALSE,          /* check_group */
+  TRUE,           /* check_owner */
   TRUE,           /* create_directory */
   FALSE,          /* notify_comsat */
   TRUE,           /* require_lockfile */
@@ -280,7 +289,7 @@ for (i = 0; i < 2; i++)
       rest++;
       }
 
-    while (isspace(*rest)) rest++;
+    while (isspace((uschar)*rest)) rest++;
 
     if (*rest != 0)
       {
@@ -334,11 +343,7 @@ if (ob->filename != NULL && ob->dirname != NULL)
   "only one of \"file\" or \"directory\" can be specified", tblock->name);
 
 /* If a file name was specified, it must be an absolute path. Can check here
-only if there are no expansions. Quota_filecount must not be given. The default
-locking depends on whether MBX is set or not. Change the built-in default if
-none of the lock options has been explicitly set. At least one form of locking
-is required in all cases, but mbx locking and fcntl locking are mutually
-exclusive. */
+only if there are no expansions. Quota_filecount must not be given. */
 
 if (ob->filename != NULL)
   {
@@ -348,33 +353,40 @@ if (ob->filename != NULL)
   if (ob->quota_filecount > 0)
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s transport:\n  "
     "quota_filecount must not be set without \"directory\"", tblock->name);
-
-  #ifdef SUPPORT_MBX
-  if (ob->mbx_format)
-    {
-    if(!ob->set_use_lockfile && !ob->set_use_fcntl && !ob->set_use_mbx_lock)
-      {
-      ob->use_lockfile = ob->use_fcntl = FALSE;
-      ob->use_mbx_lock = TRUE;
-      }
-    else if (ob->use_mbx_lock)
-      {
-      if (!ob->set_use_lockfile) ob->use_lockfile = FALSE;
-      if (!ob->set_use_fcntl) ob->use_fcntl = FALSE;
-      }
-    }
-
-  if (ob->use_fcntl && ob->use_mbx_lock)
-    log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s transport:\n  "
-      "use_fcntl_lock and use_mbx_lock are mutually exclusive", tblock->name);
-  #endif
-
-  if (!ob->use_fcntl && (!ob->use_lockfile || !ob->require_lockfile) &&
-      !ob->use_mbx_lock)
-    log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s transport:\n  "
-      "no locking configured", tblock->name);
   }
 
+/* The default locking depends on whether MBX is set or not. Change the
+built-in default if none of the lock options has been explicitly set. At least
+one form of locking is required in all cases, but mbx locking and fcntl locking
+are mutually exclusive. */
+
+#ifdef SUPPORT_MBX
+
+if (ob->mbx_format)
+  {
+  if(!ob->set_use_lockfile && !ob->set_use_fcntl && !ob->set_use_mbx_lock)
+    {
+    ob->use_lockfile = ob->use_fcntl = FALSE;
+    ob->use_mbx_lock = TRUE;
+    }
+  else if (ob->use_mbx_lock)
+    {
+    if (!ob->set_use_lockfile) ob->use_lockfile = FALSE;
+    if (!ob->set_use_fcntl) ob->use_fcntl = FALSE;
+    }
+  }
+
+if (ob->use_fcntl && ob->use_mbx_lock)
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s transport:\n  "
+    "use_fcntl_lock and use_mbx_lock are mutually exclusive", tblock->name);
+
+#endif
+
+if (!ob->use_fcntl && (!ob->use_lockfile || !ob->require_lockfile) &&
+    !ob->use_mbx_lock)
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s transport:\n  "
+    "no locking configured", tblock->name);
+
 /* If a directory name was specified, it must be an absolute path. Can check
 here only if there are no expansions. Only one of maildir or mailstore may be
 specified, and if quota_filecount is given, so must quota be. */
@@ -558,6 +570,7 @@ char *format =
   ((appendfile_transport_options_block *)(tblock->options_block))->file_format;
 char data[256];
 int len = read(cfd, data, sizeof(data));
+int sep = 0;
 char *s;
 
 DEBUG(9) debug_printf("checking file format\n");
@@ -568,11 +581,11 @@ if (len == 0) return tblock;
 
 /* Search the formats for a match */
 
-while ((s = string_nextinlist(&format,':',big_buffer,big_buffer_size))!= NULL)
+while ((s = string_nextinlist(&format,&sep,big_buffer,big_buffer_size))!= NULL)
   {
   int slen = (int)strlen(s);
   BOOL match = len >= slen && strncmp(data, s, slen) == 0;
-  char *tp = string_nextinlist(&format, ':', big_buffer, big_buffer_size);
+  char *tp = string_nextinlist(&format, &sep, big_buffer, big_buffer_size);
   if (match)
     {
     transport_instance *tt;
@@ -606,18 +619,25 @@ return NULL;
 /* This function is called if quota is set for one of the delivery modes that
 delivers into a specific directory. It scans the directory and stats all the
 files in order to get a total size and count. This is an expensive thing to do,
-but some people are prepared to bear the cost.
+but some people are prepared to bear the cost. Alternatively, if size_regex is
+set, it is used as a regex to try to extract the size from the file name, a
+strategy that some people use on maildir files on systems where the users have
+no shell access.
+
+Note: Any problems can be written to debugging output, but cannot be written to
+the log, because we are running as an unprivileged user here.
 
 Arguments:
   dirname       the name of the directory
   countptr      where to add the file count (because this function recurses)
+  regex         a compiled regex to get the size from a name
 
 Returns:        the sum of the sizes of the stattable files
                 zero if the directory cannot be opened
 */
 
 static int
-check_dir_size(char *dirname, int *countptr)
+check_dir_size(char *dirname, int *countptr, pcre *regex)
 {
 DIR *dir;
 int sum = 0;
@@ -639,11 +659,30 @@ while ((ent = readdir(dir)) != NULL)
 
   if (!string_format(buffer, sizeof(buffer), "%s/%s", dirname, name))
     {
-    log_write(0, LOG_PANIC|LOG_MAIN, "name too long in check_dir_size: "
-      "dir=%s name=%s", dirname, name);
+    DEBUG(9) debug_printf("check_dir_size: name too long: dir=%s name=%s",
+      dirname, name);
     continue;
     }
 
+  /* If there's a regex, try to find the size using it */
+
+  if (regex != NULL)
+    {
+    int ovector[6];
+    if (pcre_exec(regex,NULL, name, (int)strlen(name), 0, 0, ovector,6) >= 2)
+      {
+      int size;
+      strncpy(buffer, name + ovector[2], ovector[3] - ovector[2]);
+      size = atoi(buffer);
+      sum += size;
+      DEBUG(9)
+        debug_printf("check_dir_size: size from %s is %d\n", name, size);
+      continue;
+      }
+    }
+
+  /* No regex or no match for the regex */
+
   if (stat(buffer, &statbuf) < 0)
     {
     DEBUG(9) debug_printf("check_dir_size: stat error %d for %s: %s\n",
@@ -654,7 +693,7 @@ while ((ent = readdir(dir)) != NULL)
   if ((statbuf.st_mode & S_IFREG) != 0)
     sum += statbuf.st_size;
   else if ((statbuf.st_mode & S_IFDIR) != 0)
-    sum += check_dir_size(buffer, &count);
+    sum += check_dir_size(buffer, &count, regex);
   }
 
 closedir(dir);
@@ -667,6 +706,51 @@ return sum;
 
 
 
+/*************************************************
+*         Apply a lock to a file descriptor      *
+*************************************************/
+
+/* This function applies a lock to a file descriptor, using a blocking or
+non-blocking lock, depending on the timeout value.
+
+Arguments:
+  fd          the file descriptor
+  type        F_WRLCK or F_RDLCK
+  timeout     non-zero to use blocking locking
+
+Returns:      yield of the fcntl() call, with errno preserved;
+              sigalrm_seen set if there has been a timeout
+*/
+
+static int
+apply_lock(int fd, int type, int timeout)
+{
+int yield;
+struct flock lock_data;
+lock_data.l_type = type;
+lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
+
+sigalrm_seen = FALSE;
+
+if (timeout > 0)
+  {
+  int save_errno;
+  os_non_restarting_signal(SIGALRM, sigalrm_handler);
+  alarm(timeout);
+  yield = fcntl(fd, F_SETLKW, &lock_data);
+  save_errno = errno;
+  alarm(0);
+  signal(SIGALRM, SIG_IGN);
+  errno = save_errno;
+  }
+else yield = fcntl(fd, F_SETLK, &lock_data);
+
+return yield;
+}
+
+
+
+
 
 #ifdef SUPPORT_MBX
 /*************************************************
@@ -762,6 +846,11 @@ exim logs. Any errors must be handled by setting appropriate return codes.
 Note that the default setting for addr->transport_return is DEFER, so it need
 not be set unless some other value is required.
 
+The code below calls geteuid() rather than getuid() to get the current uid
+because in weird configurations not running setuid root there may be a
+difference. In the standard configuration, where setuid() has been used in the
+delivery process, there will be no difference between the uid and the euid.
+
 (1)  If the addr->pfr flag is set and the local_part field starts with '/' then
      this is a delivery to a file after .forward or alias expansion. Otherwise,
      there must be a configured file name or directory name.
@@ -909,17 +998,18 @@ appendfile_transport_options_block *ob =
   (appendfile_transport_options_block *)(tblock->options_block);
 struct stat statbuf;
 char *path;
-char *filename = NULL;     /* keep picky compiler happy */
+char *filename = NULL;     /* Keep picky compiler happy */
 char *hitchname = NULL;
 char *dataname = NULL;
 char *lockname = NULL;
 char *newname = NULL;
+char *nametag = NULL;
 char *cr = "";
 struct utimbuf times;
-flock_t lock_data;
 BOOL isdirectory = FALSE;
-uid_t uid = getuid();
-gid_t gid = getgid();
+BOOL isfifo = FALSE;
+uid_t uid = geteuid();     /* See note above */
+gid_t gid = getegid();
 int mbformat;
 int mode = (addr->mode > 0)? addr->mode : ob->mode;
 int saved_size = 0;
@@ -1289,12 +1379,16 @@ if (!isdirectory)
   FCNTL LOCKING
 
   If use_fcntl_lock is set, then Exim gets an exclusive fcntl() lock on the
-  mailbox once it is open. This is done with a non-blocking lock. Failures to
-  lock cause retries after a sleep, but only for a certain number of tries. A
-  blocking lock is deliberately not used so that we don't hold the mailbox
-  open. This minimizes the possibility of the NFS problem described under LOCK
-  FILES above, if for some reason NFS deliveries are happending without lock
-  files.
+  mailbox once it is open. This is done by default with a non-blocking lock.
+  Failures to lock cause retries after a sleep, but only for a certain number
+  of tries. A blocking lock is deliberately not used so that we don't hold the
+  mailbox open. This minimizes the possibility of the NFS problem described
+  under LOCK FILES above, if for some reason NFS deliveries are happening
+  without lock files. However, the use of a non-blocking lock and sleep, though
+  the safest approach, does not give the best performance on very busy systems.
+  A blocking lock plus timeout does better. Therefore Exim has an option to
+  allow it to work this way. If lock_fcntl_timeout is set greater than zero, it
+  enables the use of blocking fcntl() calls.
 
   MBX LOCKING
 
@@ -1309,7 +1403,12 @@ if (!isdirectory)
   The exclusive lock on the /tmp file prevents any other MBX client from
   updating the mailbox in any way. When writing is finished, if an exclusive
   lock on the mailbox can be obtained, indicating there are no current sharers,
-  the /tmp file is unlinked. */
+  the /tmp file is unlinked.
+
+  The fcntl() calls for getting these locks are by default non-blocking, as for
+  non-mbx locking, but can be made blocking by setting lock_fcntl_timeout.
+  As MBX delivery doesn't work over NFS, it probably makes sense to set it
+  for any MBX deliveries. */
 
 
   /* Build a lock file if configured to do so - the existence of a lock
@@ -1405,6 +1504,7 @@ if (!isdirectory)
 
   for (i = 0; i < ob->lock_retries; i++)
     {
+    int sleep_before_retry = TRUE;
     file_opened = FALSE;
 
     if((use_lstat? lstat(filename, &statbuf) : stat(filename, &statbuf)) != 0)
@@ -1489,11 +1589,12 @@ if (!isdirectory)
       int oldmode = (int)statbuf.st_mode;
       int inode = statbuf.st_ino;
       BOOL islink = (oldmode & S_IFMT) == S_IFLNK;
-      BOOL isfifo = FALSE;
 
-      /* Always check owner. */
+      isfifo = FALSE;        /* In case things are changing */
 
-      if (statbuf.st_uid != uid)
+      /* Check owner if required - the default. */
+
+      if (ob->check_owner && statbuf.st_uid != uid)
         {
         addr->basic_errno = ERRNO_BADUGID;
         addr->message = string_sprintf("mailbox %s%s has wrong uid "
@@ -1656,7 +1757,8 @@ if (!isdirectory)
         goto RETURN;
         }
 
-      if (statbuf.st_uid != uid || (ob->check_group && statbuf.st_gid != gid))
+      if ((ob->check_owner && statbuf.st_uid != uid) ||
+          (ob->check_group && statbuf.st_gid != gid))
         {
         addr->basic_errno = ERRNO_BADUGID;
         addr->message =
@@ -1691,17 +1793,17 @@ if (!isdirectory)
     rules if configured to do so. If a lock file is also required, it was
     created above and hd was left >= 0. At least one form of locking is
     required by the initialization function. If locking fails here, close the
-    file and go round the loop all over again, after waiting for a bit. */
+    file and go round the loop all over again, after waiting for a bit, unless
+    blocking locking was used. */
 
     file_opened = TRUE;
+    if (ob->lock_fcntl_timeout > 0) sleep_before_retry = FALSE;
 
     /* Simple fcntl() locking */
 
     if (ob->use_fcntl)
       {
-      lock_data.l_type = F_WRLCK;
-      lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
-      if (fcntl(fd, F_SETLK, &lock_data) >= 0) break;
+      if (apply_lock(fd, F_WRLCK, ob->lock_fcntl_timeout) >= 0) break;
       }
 
     /* MBX locking rules */
@@ -1709,9 +1811,8 @@ if (!isdirectory)
     #ifdef SUPPORT_MBX
     else if (ob->use_mbx_lock)
       {
-      lock_data.l_type = F_RDLCK;
-      lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
-      if (fcntl(fd, F_SETLK, &lock_data) >= 0 && fstat(fd, &statbuf) >= 0)
+      if (apply_lock(fd, F_RDLCK, ob->lock_fcntl_timeout) >= 0 &&
+          fstat(fd, &statbuf) >= 0)
         {
         sprintf(mbx_lockname, "/tmp/.%lx.%lx", (long)statbuf.st_dev,
           (long)statbuf.st_ino);
@@ -1745,9 +1846,7 @@ if (!isdirectory)
 
         chmod (mbx_lockname, 0600);
 
-        lock_data.l_type = F_WRLCK;
-        lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
-        if (fcntl(mbx_lockfd, F_SETLK, &lock_data) >= 0) break;
+        if (apply_lock(mbx_lockfd, F_WRLCK, ob->lock_fcntl_timeout) >= 0) break;
 
         DEBUG(9) debug_printf("failed to lock %s: %s\n", mbx_lockname,
           strerror(errno));
@@ -1769,7 +1868,19 @@ if (!isdirectory)
     close(fd);
     fd = -1;
     use_lstat = TRUE;             /* Reset to use lstat first */
-    sleep(ob->lock_interval);
+
+
+    /* If a blocking call timed out, break the retry loop if the total time
+    so far is not less than than retries * interval. */
+
+    if (sigalrm_seen &&
+        (i+1) * ob->lock_fcntl_timeout >= ob->lock_retries * ob->lock_interval)
+      i = ob->lock_retries;
+
+    /* Wait a bit before retrying, except when it was a blocked fcntl() that
+    caused the problem. */
+
+    if (i < ob->lock_retries && sleep_before_retry) sleep(ob->lock_interval);
     }
 
   /* Test for exceeding the maximum number of tries. Either the file remains
@@ -1864,10 +1975,25 @@ the tmp subdirectory, with a prescribed form of name. */
 else if (mbformat == mbf_maildir)
   {
   char *subdirs[] = { "/tmp", "/new", "/cur" };
-  char *tag = "";
 
   DEBUG(9) debug_printf("delivering in maildir format in %s\n", path);
 
+  nametag = ob->maildir_tag;
+
+  /* Check that nametag expands successfully; a hard failure causes a panic
+  return. The actual expansion for use happens again later, when $message_size
+  is accurately known. */
+
+  if (nametag != NULL && expand_string(nametag) == NULL &&
+      !expand_string_forcedfail)
+    {
+    addr->transport_return = PANIC;
+    addr->message = string_sprintf("Expansion of \"%s\" (maildir_tag "
+      "for %s transport) failed: %s", nametag, tblock->name,
+      expand_string_message);
+    return;
+    }
+
   /* First ensure that the path we have is a directory; if it does not exist,
   create it. Then make sure the tmp, new & cur subdirs of the maildir are
   there. If not we abort the delivery (even though the cur subdir is not
@@ -1954,63 +2080,15 @@ else if (mbformat == mbf_maildir)
       }
     }
 
-  /* All relevant directories now exist. Set up the maildir tag, if it is
-  configured, by copying the expanded, validated string into the tag variable,
-  preceded by a colon. Otherwise leave tag as the empty string. */
-
-  if (ob->maildir_tag != NULL)
-    {
-    char *optr;
-    char *iptr = expand_string(ob->maildir_tag);
-
-    /* Expansion failure is soft if forced */
-
-    if (iptr == NULL)
-     {
-     if (!expand_string_forcedfail)
-       {
-       addr->transport_return = PANIC;
-       addr->message = string_sprintf("Expansion of \"%s\" (maildir_tag "
-	 "for %s transport) failed: %s", ob->maildir_tag, tblock->name,
-	 expand_string_message);
-       return;
-       }
-     }
-
-    /* The validated string will never be longer than the input string + 1.
-    Ignore characters other than [A-Za-z0-9.:_-,], and put a colon at the
-    start. */
-
-    else
-      {
-      optr = tag = store_get(strlen(iptr) + 2);
-      *optr++ = ':';
-
-      while (*iptr != 0)
-	{
-	if (isalnum(*iptr) ||
-	    *iptr == '.' || *iptr == ':' ||
-	    *iptr == '_' || *iptr == '-' || *iptr == ',')
-	  *optr++ = *iptr;
-	iptr++;
-	}
-      *optr = 0;
-
-      /* If the string ends up empty, scrap it. */
-
-      if (tag[1] == 0) *tag = 0;
-      }
-    }
-
-  /* Attempt to open the temporary file a limited number of times. I think this
-  rather scary-looking for statement is actually OK. If open succeeds, the loop
-  is broken; if not, there is a test on the value of i. Get the time again
-  afresh each time round the loop. */
+  /* All relevant directories now exist. Attempt to open the temporary file a
+  limited number of times. I think this rather scary-looking for statement is
+  actually OK. If open succeeds, the loop is broken; if not, there is a test
+  on the value of i. Get the time again afresh each time round the loop. */
 
   for (i = 1;; i++)
     {
-    char *basename = string_sprintf("%lu.%lu.%s%s", time(NULL), getpid(),
-      primary_hostname, tag);
+    char *basename = string_sprintf("%lu.%lu.%s", time(NULL), getpid(),
+      primary_hostname);
 
     filename = dataname = string_sprintf("tmp/%s", basename);
     newname = string_sprintf("new/%s", basename);
@@ -2025,15 +2103,6 @@ else if (mbformat == mbf_maildir)
         filename, strerror(errno));
       }
 
-    /* If the name is too long, try again without the tag */
-
-    else if (errno == ENAMETOOLONG)
-      {
-      DEBUG(2) debug_printf("maildir_tag (%s) makes filename too long\n", tag);
-      *tag = 0;
-      continue;    /* Repeat without delay */
-      }
-
     /* Too many retries - give up */
 
     if (i >= ob->maildir_retries)
@@ -2208,7 +2277,22 @@ will include the new one we are about to write. This is expensive, but happens
 only if configured. */
 
 if (isdirectory && (ob->quota_value > 0 || ob->quota_warn_threshold_value > 0))
-  saved_size = check_dir_size(path, &dircount);
+  {
+  const char *error;
+  int offset;
+  pcre *regex = NULL;
+  if (ob->quota_size_regex != NULL)
+    {
+    regex = pcre_compile(ob->quota_size_regex, PCRE_COPT, &error,&offset, NULL);
+    if (regex == NULL)
+      {
+      DEBUG(9) debug_printf("check_dir_size: regular expression error: "
+        "%s at offset %d while compiling %s", error, offset,
+        ob->quota_size_regex);
+      }
+    }
+  saved_size = check_dir_size(path, &dircount, regex);
+  }
 
 /* If there is a local quota setting, check that we are not going to exceed it
 with this message. */
@@ -2371,9 +2455,10 @@ if (temp_fd >= 0 && ob->mbx_format)
   }
 #endif
 
-/* Force out the remaining data to check for any errors */
+/* Force out the remaining data to check for any errors; some OS don't allow
+fsync() to be called for a FIFO. */
 
-if (yield == OK && fsync(fd) < 0) yield = DEFER;
+if (yield == OK && !isfifo && fsync(fd) < 0) yield = DEFER;
 
 /* Handle error while writing the file. Control should come here directly after
 the error, with the reason in errno. In the case of expansion failure in prefix
@@ -2381,6 +2466,8 @@ or suffix, it will be ERRNO_EXPANDFAIL. */
 
 if (yield != OK)
   {
+  addr->special_action = SPECIAL_NONE;     /* Cancel any quota warning */
+
   /* Save the error number. If positive, it will ultimately cause a strerror()
   call to generate some text. */
 
@@ -2473,7 +2560,8 @@ if (yield != OK)
 appended files. Remove the default backstop error number. For a directory, now
 is the time to rename the file with a unique name. As soon as such a name
 appears it may get used by another process, so we close the file first and
-check that all is well. */
+check that all is well. If a tag exists, expand it and add it to the name,
+first resetting $message_size to the accurate size. */
 
 else
   {
@@ -2501,10 +2589,47 @@ else
       yield = DEFER;
       }
 
+    /* File is successfully written. Arrange to rename it, adding the expanded
+    tag value to the new name if required. First, correct the message size to
+    the accurate value. */
+
     else
       {
+      char *renamename = newname;
       fd = -1;
-      if (rename(filename, newname) < 0)
+      message_size = transport_count;
+
+      /* The expansion of nametag was checked above, before the file was
+      opened. It either succeeded, or provoked a soft failure. So any failure
+      here can be treated as soft. Ignore non-printing characters and / and put
+      a colon at the start if the first character is alphanumeric. */
+
+      if (nametag != NULL)
+        {
+        char *iptr = expand_string(nametag);
+        if (iptr != NULL)
+          {
+          char *etag = store_get(strlen(iptr) + 2);
+          char *optr = etag;
+          while (*iptr != 0)
+	    {
+	    if (isgraph((uschar)*iptr) && *iptr != '/')
+              {
+              if (optr == etag && isalnum((uschar)*iptr)) *optr++ = ':';
+              *optr++ = *iptr;
+              }
+	    iptr++;
+	    }
+          *optr = 0;
+          renamename = string_sprintf("%s%s", newname, etag);
+          }
+        }
+
+      /* Do the rename. If the name is too long, try again without the tag */
+
+      if (rename(filename, renamename) < 0 &&
+         (errno != ENAMETOOLONG ||
+          (renamename = newname, rename(filename, renamename) < 0)))
         {
         addr->basic_errno = errno;
         addr->message = string_sprintf("while renaming %s as %s",
@@ -2513,7 +2638,7 @@ else
         }
       else
         {
-        DEBUG(9) debug_printf("renamed %s as %s\n", filename, newname);
+        DEBUG(9) debug_printf("renamed %s as %s\n", filename, renamename);
         filename = dataname = NULL;   /* Prevents attempt to unlink at end */
         }
       }
@@ -2543,7 +2668,8 @@ addr->transport_return = yield;
 is closed above, except in cases of error which goto RETURN, when we also need
 to remove the original file(s). For MBX locking, if all has gone well, before
 closing the file, see if we can get an exclusive lock on it, in which case we
-can unlink the /tmp lock file before closing it. If everything has gone right
+can unlink the /tmp lock file before closing it. This is always a non-blocking
+lock; there's no need to wait if we can't get it. If everything has gone right
 but close fails, defer the message. Then unlink the lock file, if present. This
 point in the code is jumped to from a number of places when errors are
 detected, in order to get the file closed and the lock file tidied away. */
@@ -2555,6 +2681,7 @@ if (mbx_lockfd >= 0)
   {
   if (yield == OK)
     {
+    flock_t lock_data;
     lock_data.l_type = F_WRLCK;
     lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
     if (fcntl(fd, F_SETLK, &lock_data) >= 0)
