@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -273,6 +273,7 @@ argv = *argvptr;
 
 if (ob->allow_commands != NULL)
   {
+  int sep = 0;
   char *s, *p;
   char buffer[256];
 
@@ -285,7 +286,7 @@ if (ob->allow_commands != NULL)
     return FALSE;
     }
 
-  while ((p = string_nextinlist(&s, ':', buffer, sizeof(buffer))) != NULL)
+  while ((p = string_nextinlist(&s, &sep, buffer, sizeof(buffer))) != NULL)
     {
     if (strcmp(p, argv[0]) == 0) { permitted = TRUE; break; }
     }
@@ -325,11 +326,12 @@ for it. */
 
 if (argv[0][0] != '/')
   {
+  int sep = 0;
   char *p;
   char *listptr = ob->path;
   char buffer[1024];
 
-  while ((p = string_nextinlist(&listptr, ':', buffer, sizeof(buffer))) != NULL)
+  while ((p = string_nextinlist(&listptr, &sep, buffer, sizeof(buffer))) != NULL)
     {
     struct stat statbuf;
     sprintf(big_buffer, "%.256s/%.256s", p, argv[0]);
@@ -449,9 +451,9 @@ pipe_transport_entry(
   address_item *addr)              /* address(es) we are working on */
 {
 pid_t pid, outpid;
-int fd, rc;
-int pipefd[2];
+int fd_in, fd_out, rc;
 int envcount = 0;
+int envsep = 0;
 pipe_transport_options_block *ob =
   (pipe_transport_options_block *)(tblock->options_block);
 int timeout = ob->timeout;
@@ -479,7 +481,7 @@ the command is supplied as one of the pipe transport's options. */
 if (addr->local_part[0] == '|')
   {
   cmd = addr->local_part + 1;
-  while (isspace(*cmd)) cmd++;
+  while (isspace((uschar)*cmd)) cmd++;
   expand_arguments = addr->expand_pipe;
   }
 else
@@ -555,7 +557,7 @@ if (envlist != NULL)
     }
   }
 
-while ((ss = string_nextinlist(&envlist, ':', big_buffer, big_buffer_size))
+while ((ss = string_nextinlist(&envlist, &envsep, big_buffer, big_buffer_size))
        != NULL)
    {
    if (envcount > sizeof(envp)/sizeof(char *) - 2)
@@ -578,11 +580,12 @@ if (dont_deliver)
 
 
 /* Handling the output from the pipe is tricky. If a file for catching this
-output is provided, we could just hand that fd over to the pipe, but this isn't
-very safe because the pipe might loop and carry on writing for ever (which is
-exactly what happened in early versions of Exim). Therefore we must hand over a
-pipe fd, read our end of the pipe and count the number of bytes that come
-through, chopping the sub-process if it exceeds some limit.
+output is provided, we could in theory just hand that fd over to the process,
+but this isn't very safe because it might loop and carry on writing for
+ever (which is exactly what happened in early versions of Exim). Therefore we
+use the standard child_open() function, which creates pipes. We can then read
+our end of the output pipe and count the number of bytes that come through,
+chopping the sub-process if it exceeds some limit.
 
 However, this means we want to run a sub-process with both its input and output
 attached to pipes. We can't handle that easily from a single parent process
@@ -593,25 +596,13 @@ handle this is too great - simpler just to run another process to do the
 reading of the output pipe. */
 
 
-/* Make the pipe for handling the output - do this always, even if a
-return_file is not provided. */
-
-if (pipe(pipefd) < 0)
-  {
-  addr->transport_return = DEFER;
-  addr->message = string_sprintf(
-    "Failed to create pipe for handling output in %s transport",
-      tblock->name);
-  return;
-  }
-
 /* As this is a local transport, we are already running with the required
 uid/gid and current directory, so pass NULL to child_open to indicate no
 change. Request that the new process be a process group leader, so we
 can kill it and all its children on a timeout. */
 
-if ((pid = child_open(argv, envp, ob->umask, NULL, NULL, &fd,
-     pipefd[pipe_write], NULL, TRUE)) < 0)
+if ((pid = child_open(argv, envp, ob->umask, NULL, NULL, &fd_in, &fd_out,
+    NULL, TRUE)) < 0)
   {
   addr->transport_return = DEFER;
   addr->message = string_sprintf(
@@ -620,10 +611,6 @@ if ((pid = child_open(argv, envp, ob->umask, NULL, NULL, &fd,
   return;
   }
 
-/* Close off the end of the output pipe we are not using. */
-
-close(pipefd[pipe_write]);
-
 /* Now fork a process to handle the output that comes down the pipe. */
 
 if ((outpid = fork()) < 0)
@@ -633,7 +620,8 @@ if ((outpid = fork()) < 0)
   addr->message = string_sprintf(
     "Failed to create process for handling output in %s transport",
       tblock->name);
-  close(pipefd[pipe_read]);
+  close(fd_in);
+  close(fd_out);
   return;
   }
 
@@ -646,9 +634,9 @@ the subprocess to fail. */
 if (outpid == 0)
   {
   int count = 0;
-  close(fd);
+  close(fd_in);
   set_process_info("reading output from |%s", cmd);
-  while ((rc = read(pipefd[pipe_read], big_buffer, big_buffer_size)) > 0)
+  while ((rc = read(fd_out, big_buffer, big_buffer_size)) > 0)
     {
     if (addr->return_file >= 0)
       write(addr->return_file, big_buffer, rc);
@@ -663,11 +651,11 @@ if (outpid == 0)
       break;
       }
     }
-  close(pipefd[pipe_read]);
+  close(fd_out);
   _exit(0);
   }
 
-close(pipefd[pipe_read]);  /* Not used in this process */
+close(fd_out);  /* Not used in this process */
 
 
 /* Carrying on now with the main parent process. Attempt to write the message
@@ -700,20 +688,20 @@ if (tblock->local_smtp != local_smtp_off)
     addr->errors_address : sender_address;
 
   if (tblock->bsmtp_helo &&
-      !transport_write_string(fd, "HELO %s%s", primary_hostname, eol))
+      !transport_write_string(fd_in, "HELO %s%s", primary_hostname, eol))
     goto END_WRITE;
 
-  if (!transport_write_string(fd, "MAIL FROM:<%s>%s", sender, eol))
+  if (!transport_write_string(fd_in, "MAIL FROM:<%s>%s", sender, eol))
     goto END_WRITE;
 
   for (a = addr; a != NULL; a = a->next)
     {
-    if (!transport_write_string(fd,
+    if (!transport_write_string(fd_in,
         "RCPT TO:<%s@%s>%s", a->local_part, a->domain, eol))
       goto END_WRITE;
     }
 
-  if (!transport_write_string(fd, "DATA%s", eol)) goto END_WRITE;
+  if (!transport_write_string(fd_in, "DATA%s", eol)) goto END_WRITE;
   }
 
 /* Now any other configured prefix. */
@@ -728,12 +716,13 @@ if (ob->prefix != NULL)
       "transport) failed: %s", ob->prefix, tblock->name, expand_string_message);
     return;
     }
-  if (!transport_write_block(fd, prefix, (int)strlen(prefix))) goto END_WRITE;
+  if (!transport_write_block(fd_in, prefix, (int)strlen(prefix)))
+    goto END_WRITE;
   }
 
 /* Now the actual message - the options were set at initialization time */
 
-if (!transport_write_message(addr, fd, ob->options, 0, tblock->add_headers,
+if (!transport_write_message(addr, fd_in, ob->options, 0, tblock->add_headers,
   tblock->remove_headers, ob->check_string, ob->escape_string))
     goto END_WRITE;
 
@@ -749,13 +738,14 @@ if (ob->suffix != NULL)
       "transport) failed: %s", ob->suffix, tblock->name, expand_string_message);
     return;
     }
-  if (!transport_write_block(fd, suffix, (int)strlen(suffix))) goto END_WRITE;
+  if (!transport_write_block(fd_in, suffix, (int)strlen(suffix)))
+    goto END_WRITE;
   }
 
 /* If local_smtp, write the terminating dot. */
 
 if (tblock->local_smtp != local_smtp_off &&
-    !transport_write_string(fd, ".%s", eol))
+    !transport_write_string(fd_in, ".%s", eol))
   goto END_WRITE;
 
 /* Flag all writing completed successfully. */
@@ -770,7 +760,7 @@ END_WRITE:
 pipe. */
 
 signal(SIGALRM, SIG_IGN);
-(void) close(fd);
+(void) close(fd_in);
 
 /* Handle errors during writing. For timeouts, set the timeout for waiting for
 the child process to 1 second. If the process at the far end of the pipe died
@@ -906,6 +896,7 @@ if ((rc = child_close(pid, timeout)) != 0)
       char *s = ob->temp_errors;
       char *ss, *p;
       int size, ptr, i;
+      int sep = 0;
       char buffer[64];
 
       addr->transport_return = FAIL;
@@ -913,7 +904,7 @@ if ((rc = child_close(pid, timeout)) != 0)
       /* Scan the list of temporary failure codes; if any match, the
       result becomes DEFER. */
 
-      while ((p = string_nextinlist(&s, ':', buffer, sizeof(buffer))) != NULL)
+      while ((p = string_nextinlist(&s, &sep, buffer, sizeof(buffer))) != NULL)
         {
         if (rc == atoi(p)) { addr->transport_return = DEFER; break; }
         }
