@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -49,6 +49,8 @@ optionlist smtp_transport_options[] = {
       (void *)(offsetof(smtp_transport_options_block, hosts)) },
   { "hosts_override",       opt_bool,
       (void *)(offsetof(smtp_transport_options_block, hosts_override)) },
+  { "hosts_randomize",      opt_bool,
+      (void *)(offsetof(smtp_transport_options_block, hosts_randomize)) },
   { "interface",            opt_stringptr,
       (void *)(offsetof(smtp_transport_options_block, interface)) },
   { "keepalive",            opt_bool,
@@ -107,6 +109,7 @@ smtp_transport_options_block smtp_transport_option_defaults = {
   FALSE,           /* dns_search_parents */
   TRUE,            /* delay_after_cutoff */
   FALSE,           /* hosts_override */
+  FALSE,           /* hosts_randomize */
   TRUE,            /* keepalive */
   FALSE,           /* max_rcpt_serialize */
   TRUE             /* retry_include_ip_address */
@@ -231,7 +234,7 @@ if (ob->hosts_override && ob->hosts != NULL) tblock->overrides_hosts = TRUE;
 /* If there are any fallback hosts listed, build a chain of host items
 for them, but do not do any lookups at this time. */
 
-host_build_hostlist(&(ob->fallback_hostlist), ob->fallback_hosts);
+host_build_hostlist(&(ob->fallback_hostlist), ob->fallback_hosts, FALSE);
 
 /* If an interface is set, check that it is a valid IP address. */
 
@@ -602,12 +605,12 @@ goto SEND_QUIT;
           {
           int rc;
           int len = (int)strlen(au->public_name);
-          while (isspace(*p)) p++;
+          while (isspace((uschar)*p)) p++;
 
           if (strncmpic(au->public_name, p, len) != 0 ||
-              (p[len] != 0 && !isspace(p[len])))
+              (p[len] != 0 && !isspace((uschar)p[len])))
             {
-            while (*p != 0 && !isspace(*p)) p++;
+            while (*p != 0 && !isspace((uschar)*p)) p++;
             continue;
             }
 
@@ -1394,7 +1397,7 @@ if (hostlist == NULL || (ob->hosts_override && ob->hosts != NULL))
       s = expanded_hosts;
       }
 
-    host_build_hostlist(&hostlist, s);
+    host_build_hostlist(&hostlist, s, ob->hosts_randomize);
 
     /* If there was no expansion of hosts, save the host list for
     next time. */
@@ -1414,7 +1417,7 @@ network byte order, and that's what getservbyname() produces, so we have
 to use htons() if the configuration specified a port by number instead of
 by name. */
 
-if (isdigit(*ob->service))
+if (isdigit((uschar)(*ob->service)))
   {
   char *end;
   port = (int)htons((unsigned short)strtol(ob->service, &end, 0));
@@ -1537,7 +1540,10 @@ for (cutoff_retry = 0; expired &&
         if (ob->gethostbyname)
           rc = host_find_byname(host, &canonical_name, FALSE);
 
-        /* Do an MX-only lookup if the name is in mx_domains */
+        /* Do an MX-only lookup if the name is in mx_domains. If there is
+        a lookup defer in the checking, just proceed as if the host doesn't
+        match - this is a rarely (if ever?) used feature, and it doesn't lead
+        to a serious problem to do this. */
 
         else
           {
@@ -1613,7 +1619,9 @@ for (cutoff_retry = 0; expired &&
 
     /* If the queue_smtp option is set, or the domain is in queue_smtp_domains,
     we don't actually want to attempt any deliveries. When doing a queue run,
-    queue_smtp_domains is unset. */
+    queue_smtp_domains is unset. If there is a lookup defer in queue_smtp_
+    domains, proceed as if the domain were not in it. We don't want to hold up
+    all SMTP deliveries! */
 
     if (!deliver_force && (queue_smtp ||
         match_isinlist(addrlist->domain, &queue_smtp_domains,TRUE,TRUE,NULL)))
@@ -1644,8 +1652,8 @@ for (cutoff_retry = 0; expired &&
       If either of these retry records are actually read, the keys used are
       returned to save recomputing them later. */
 
-      host_is_expired = retry_check_address(host, ob->retry_include_ip_address,
-        &retry_host_key, &retry_message_key);
+      host_is_expired = retry_check_address(addrlist->domain, host,
+        ob->retry_include_ip_address, &retry_host_key, &retry_message_key);
 
       DEBUG(2) debug_printf("%s [%s] status = %s\n", host->name,
         (host->address == NULL)? "" : host->address,
