@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with verifying things. */
@@ -258,113 +258,118 @@ while (addr_new != NULL)
     log_write(0, LOG_PANIC_DIE, "Unqualified address found: %s", addr->orig);
 
   /* Determine locality - this sets "local_part", "domain", and "local"
-  fields. Pass back the locality if a variable is supplied. */
+  fields. Pass back the locality if a variable is supplied. If we cannot
+  determine the locality (lookup defer), behave as if directing or routing
+  deferred. An error will have been set for the address. */
 
-  deliver_setlocal(addr);
-
-  if (local_domain != NULL) *local_domain = addr->local;
-
-  /* If we are handing an EXPN command and this is the original address,
-  and it is not local, we don't proceed any further. (Actually, if it's
-  expn, this will always be the original address, since we don't process
-  generated addresses any more.) */
-
-  if (expn && addr == addr_orig && !addr->local)
+  if (!deliver_setlocal(addr)) rc = DEFER; else
     {
-    fprintf(f, "550 Not a local domain\r\n");
-    return FALSE;
-    }
+    if (local_domain != NULL) *local_domain = addr->local;
 
-  /* DEBUG: show what's been done to this address */
+    /* If we are handing an EXPN command and this is the original address,
+    and it is not local, we don't proceed any further. (Actually, if it's
+    expn, this will always be the original address, since we don't process
+    generated addresses any more.) */
 
-  DEBUG(7)
-    {
-    debug_printf("address %s\n", addr->orig);
-    debug_printf("  local_part=%s domain=%s\n  %s\n",
-      addr->local_part, addr->domain,
-      (addr->local)? "domain is local" : "domain is not local");
-    }
-
-  /* Handle a local address with the directors, or a remote address with the
-  routers. Note that a director may set up local or remote delivery. */
-
-  verify_forced_errmsg = NULL;
-
-  rc = (addr->local)?
-    direct_address(addr, &addr_local, &addr_remote, &addr_new, &addr_succeed,
-      verify_type) :
-    route_address(addr, &addr_local, &addr_remote, &addr_new, verify_type);
-
-
-  #ifdef NEVER
-  /* This is some embryonic code for doing a verification by calling back to
-  the hosts that an address routed to. It is of course expensive, but apart
-  from that there are potential problems, so I haven't for the moment pursued
-  it. Some big domains (e.g. aol.com) accept anything and check later, so you
-  don't gain anything by this check in any case. If this code is ever required,
-  the option sender_full_verify_domains needs to be created, and really some
-  means of varying the port and the timeout are also required. What should
-  happen when running -bh? Another problem is that ideally, the error from the
-  remote host should be logged, but because it is in the addr structure, it
-  gets lost, since that is local to the verification process. Actually, this
-  is a general problem. */
-
-  /* If directing/routing succeeded and set up a list of hosts, and
-  sender_verify_full_domains is set for this domain, do the full call-back
-  verification for a sender. This is expensive. */
-
-  if (rc == OK &&
-      addr->host_list != NULL &&
-      (options & vopt_is_recipient) == 0 &&
-      sender_full_verify_domains != NULL &&
-      match_isinlist(addr->domain,&sender_full_verify_domains,FALSE,TRUE,NULL))
-    {
-    host_item *host;
-    BOOL done = FALSE;
-
-    HDEBUG(1) debug_printf("Attempting full verification using callback\n");
-
-    for (host = addr->host_list; !done && host != NULL; host = host->next)
+    if (expn && addr == addr_orig && !addr->local)
       {
-      int fd = smtp_connect(host, 1225, NULL, 30, TRUE);
-      if (fd < 0) continue;
+      fprintf(f, "550 Not a local domain\r\n");
+      return FALSE;
+      }
 
-      if (smtp_read_response(fd, big_buffer, big_buffer_size, '2', 30) &&
-          smtp_write_command(fd, "HELO %s\r\n", primary_hostname) &&
-          smtp_read_response(fd, big_buffer, big_buffer_size, '2', 30) &&
-          smtp_write_command(fd, "MAIL FROM:<>\r\n") &&
-          smtp_read_response(fd, big_buffer, big_buffer_size, '2', 30) &&
-          smtp_write_command(fd, "RCPT TO:<%s>\r\n", addr->orig))
+    /* DEBUG: show what's been done to this address */
+
+    DEBUG(7)
+      {
+      debug_printf("address %s\n", addr->orig);
+      debug_printf("  local_part=%s domain=%s\n  %s\n",
+        addr->local_part, addr->domain,
+        (addr->local)? "domain is local" : "domain is not local");
+      }
+
+    /* Handle a local address with the directors, or a remote address with the
+    routers. Note that a director may set up local or remote delivery. */
+
+    verify_forced_errmsg = NULL;
+
+    rc = (addr->local)?
+      direct_address(addr, &addr_local, &addr_remote, &addr_new, &addr_succeed,
+        verify_type) :
+      route_address(addr, &addr_local, &addr_remote, &addr_new, verify_type);
+
+
+    #ifdef NEVER
+    /* This is some embryonic code for doing a verification by calling back to
+    the hosts that an address routed to. It is of course expensive, but apart
+    from that there are potential problems, so I haven't for the moment pursued
+    it. Some big domains (e.g. aol.com) accept anything and check later, so you
+    don't gain anything by this check in any case. If this code is ever
+    required, the option sender_full_verify_domains needs to be created, and
+    really some means of varying the port and the timeout are also required.
+    What should happen when running -bh? Another problem is that ideally, the
+    error from the remote host should be logged, but because it is in the addr
+    structure, it gets lost, since that is local to the verification process.
+    Actually, this is a general problem. */
+
+    /* If directing/routing succeeded and set up a list of hosts, and
+    sender_verify_full_domains is set for this domain, do the full call-back
+    verification for a sender. This is expensive. */
+
+    if (rc == OK &&
+        addr->host_list != NULL &&
+        (options & vopt_is_recipient) == 0 &&
+        sender_full_verify_domains != NULL &&
+        match_isinlist(addr->domain, &sender_full_verify_domains, FALSE,
+          TRUE, NULL))
+      {
+      host_item *host;
+      BOOL done = FALSE;
+
+      HDEBUG(1) debug_printf("Attempting full verification using callback\n");
+
+      for (host = addr->host_list; !done && host != NULL; host = host->next)
         {
-        if (!smtp_read_response(fd, big_buffer, big_buffer_size, '2', 30))
+        int fd = smtp_connect(host, 1225, NULL, 30, TRUE);
+        if (fd < 0) continue;
+
+        if (smtp_read_response(fd, big_buffer, big_buffer_size, '2', 30) &&
+            smtp_write_command(fd, "HELO %s\r\n", primary_hostname) &&
+            smtp_read_response(fd, big_buffer, big_buffer_size, '2', 30) &&
+            smtp_write_command(fd, "MAIL FROM:<>\r\n") &&
+            smtp_read_response(fd, big_buffer, big_buffer_size, '2', 30) &&
+            smtp_write_command(fd, "RCPT TO:<%s>\r\n", addr->orig))
           {
-          if (errno == 0 && big_buffer[0] == '5')
+          if (!smtp_read_response(fd, big_buffer, big_buffer_size, '2', 30))
             {
-            rc = FAIL;
-            addr->message = string_sprintf("check with host %s failed: %s",
-              host->name, big_buffer);
-            done = TRUE;
+            if (errno == 0 && big_buffer[0] == '5')
+              {
+              rc = FAIL;
+              addr->message = string_sprintf("check with host %s failed: %s",
+                host->name, big_buffer);
+              done = TRUE;
+              }
             }
+          else done = TRUE;
           }
-        else done = TRUE;
+
+        smtp_write_command(fd, "QUIT\r\n");
+        close(fd);
         }
 
-      smtp_write_command(fd, "QUIT\r\n");
-      close(fd);
-      }
+      /* Failure to connect to any host, or any other error other than a 5xx
+      response is a temporary error. */
 
-    /* Failure to connect to any host, or any other error other than a 5xx
-    response is a temporary error. */
-
-    if (host == NULL)
-      {
-      rc = DEFER;
-      addr->message =
-        string_sprintf("could not connect to MX host for domain %s",
-          addr->domain);
+      if (host == NULL)
+        {
+        rc = DEFER;
+        addr->message =
+          string_sprintf("could not connect to MX host for domain %s",
+            addr->domain);
+        }
       }
-    }
-  #endif /* NEVER */
+    #endif /* NEVER */
+
+    }   /* Routing/directing done, or setlocal() failed */
 
 
   /* If a director generated a new address as a routing ploy rather than
@@ -653,7 +658,7 @@ for (i = 0; i < 3; i++)
 
     while (*s != 0)
       {
-      while (isspace(*s) || *s == ',') s++;
+      while (isspace((uschar)*s) || *s == ',') s++;
       if (*s == 0) continue;        /* Skip if header is empty */
 
       ss = parse_find_address_end(s, FALSE);
@@ -668,7 +673,7 @@ for (i = 0; i < 3; i++)
       verification gets done. */
 
       if (strncmp(s, sender_address, senderlen) == 0 &&
-                (s[senderlen] == 0 || isspace(s[senderlen])))
+                (s[senderlen] == 0 || isspace((uschar)s[senderlen])))
         {
         new_ok = sender_yield;
         HDEBUG(2) debug_printf("same as sender\n");
@@ -687,7 +692,7 @@ for (i = 0; i < 3; i++)
         {
         header_name *header_names = (strncmpic(h->text, "resent-", 7) == 0)?
           header_names_resent : header_names_normal;
-        while (ss > s && isspace(ss[-1])) ss--;
+        while (ss > s && isspace((uschar)ss[-1])) ss--;
         verify_address_parse_error =
           string_sprintf("syntax error in '%s' header when scanning for sender: "
             "%s in \"%.*s\"",
@@ -1259,7 +1264,7 @@ if (t != NULL && t[1] != 0)
 if (*ss == '!')
   {
   matchyield = hm_no;
-  while (isspace(*(++ss)));
+  while (isspace((uschar)(*(++ss))));
   }
 else matchyield = hm_yes;
 
@@ -1340,7 +1345,7 @@ on a masked IP network, in textual form. */
 if (strncmp(ss, "net", 3) == 0 && (semicolon = strchr(ss, ';')) != NULL)
   {
   int mlen = 0;
-  for (t = ss + 3; isdigit(*t); t++) mlen = mlen * 10 + *t - '0';
+  for (t = ss + 3; isdigit((uschar)*t); t++) mlen = mlen * 10 + *t - '0';
   if (t >= ss + 3 && *t++ == '-')
     {
     int insize;
@@ -1389,7 +1394,7 @@ if (strncmp(ss, "net", 3) == 0 && (semicolon = strchr(ss, ';')) != NULL)
       {
       tt = buffer;
       filename = semicolon + 1;
-      while (isspace(*filename)) filename++;
+      while (isspace((uschar)*filename)) filename++;
       }
 
     /* Now do the actual lookup; note that there is no search_close() because
@@ -1409,7 +1414,7 @@ comprise only letters, digits, full stops, and hyphens (the constituents of
 domain names). Allow underscores, as they are all too commonly found. Sigh. */
 
 for (t = ss; *t != 0; t++)
-  if (!isalnum(*t) && *t != '.' && *t != '-' && *t != '_') break;
+  if (!isalnum((uschar)*t) && *t != '.' && *t != '-' && *t != '_') break;
 
 /* If the pattern is a complete domain name, with no fancy characters, look up
 its IP address and match against that. Note that a multi-homed host will add
@@ -1522,6 +1527,7 @@ verify_check_this_host(char **listptr, BOOL find_failed_return,
 BOOL endyield = FALSE;
 BOOL ffr = find_failed_return;
 BOOL log_unknown = FALSE;
+int sep = 0;
 char *ss;
 char *list = *listptr;
 char *ot = NULL;
@@ -1553,7 +1559,7 @@ to a FALSE return on matching. The value of yield is set to the match return;
 this means that when the end of the list is reached, the return depends on
 whether the final item was negated or not. */
 
-while ((ss = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
+while ((ss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
   {
   /* If the host item is "+allow_unknown", change the value returned for a
   subsequent failed reverse lookup. Also for "+warn_unknown", but in that
@@ -1611,7 +1617,7 @@ while ((ss = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
     if (*ss == '!')
       {
       yes = FALSE;
-      while (isspace(*(++ss)));
+      while (isspace((uschar)(*(++ss))));
       }
     else yes = TRUE;
 
@@ -1632,11 +1638,11 @@ while ((ss = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
       if ((ss = strchr(fbuffer, '#')) != NULL) *ss = 0; /* trailing comment */
 
       ss = fbuffer + (int)strlen(fbuffer);              /* trailing space */
-      while (ss > fbuffer && isspace(ss[-1])) ss--;
+      while (ss > fbuffer && isspace((uschar)ss[-1])) ss--;
       *ss = 0;
 
       ss = fbuffer;
-      while (isspace(*ss)) ss++;                        /* leading space */
+      while (isspace((uschar)*ss)) ss++;                /* leading space */
 
       if (*ss == 0) continue;                           /* ignore empty */
 
@@ -1673,6 +1679,7 @@ while ((ss = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
         return ffr;
         }
       }
+    fclose(f);
     }
   }
 
